var models = require('../models');
var path = require('path');
const helper = require('../routes/common/helper');
const argon2 = require('argon2');



const sendPasswordResetEmail = async (req, res, next) => {
    try {
      const { email } = req.query;
  
      const newPassword = generateRandomPassword(16);
  
      const user = await models.User.findOne({
        where: {
          email: email
        }
      });
  
      if (!user) {
        return res.status(404).json({ error: 'User not found' });
      }

      const hashedPassword = await argon2.hash(newPassword);
  
      await user.update({
        password: hashedPassword,
      });
  
      helper.sendPasswordResetMail({
        recipient: email,
        emailSubject: 'API User Password Reset',
        templateFileName: 'api_user_passwrod_reset.html',
        userName: email,
        password: newPassword,
      });
  
      return res.status(200).json({
        success: true,
        message: 'Password reset email sent successfully',
      });
    } catch (error) {
      console.error('Error in sendPasswordResetEmail:', error);
      res.status(500).json({ error: 'Internal Server Error' });
    }
};
  
function generateRandomPassword(length) {
    const characters = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789';
    let password = '';
    for (let i = 0; i < length; i++) {
        const randomIndex = Math.floor(Math.random() * characters.length);
        password += characters.charAt(randomIndex);
    }
    return password;
}
  
module.exports = {
  sendPasswordResetEmail
};