'use strict';

module.exports = {
  up: async (queryInterface, Sequelize) => {
    await queryInterface.createTable('CustomUserGroups', {
      id: {
        allowNull: false,
        autoIncrement: true,
        primaryKey: true,
        type: Sequelize.INTEGER,
      },
      groupName: {
        type: Sequelize.STRING(255),
        allowNull: true,
      },
      groupDescription: {
        type: Sequelize.STRING(255),
        allowNull: true,
      },
      sql_query_id: {
        type: Sequelize.INTEGER,
        allowNull: true,
        references: {
          model: 'UserGroupSQLQuery',
          key: 'id',
        },
      },
      raw_query_id: {
        type: Sequelize.INTEGER,
        allowNull: true,
        references: {
          model: 'UserGroupRawQuery',
          key: 'id',
        },
      },
      runtime_bind_id: {
        type: Sequelize.BIGINT,
        allowNull: true,
        unique: true,
      },
      createdAt: {
        allowNull: false,
        type: Sequelize.DATE,
      },
      updatedAt: {
        allowNull: false,
        type: Sequelize.DATE,
      },
    });

    await queryInterface.addConstraint('CustomUserGroups', {
      fields: ['sql_query_id'],
      type: 'foreign key',
      name: 'custom_user_groups_sql_query_fk',
      references: {
        table: 'UserGroupSQLQuery',
        field: 'id',
      },
      onDelete: 'SET NULL',
      onUpdate: 'CASCADE',
    });

    await queryInterface.addConstraint('CustomUserGroups', {
      fields: ['raw_query_id'],
      type: 'foreign key',
      name: 'custom_user_groups_raw_query_fk',
      references: {
        table: 'UserGroupRawQuery',
        field: 'id',
      },
      onDelete: 'SET NULL',
      onUpdate: 'CASCADE',
    });
  },

  down: async (queryInterface, Sequelize) => {
    await queryInterface.removeConstraint('CustomUserGroups', 'custom_user_groups_sql_query_fk');
    await queryInterface.removeConstraint('CustomUserGroups', 'custom_user_groups_raw_query_fk');
    await queryInterface.dropTable('CustomUserGroups');
  },
};
