'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up(queryInterface, Sequelize) {
    // Fetch all rows where runtime_bind_id is NULL
    const [localUsers] = await queryInterface.sequelize.query(
      `SELECT * FROM LocalUsers WHERE runtime_bind_id IS NULL`
    );

    // Loop through each user and set a unique runtime_bind_id
    for (let i = 0; i < localUsers.length; i++) {
      const uniqueUnixTimestamp = Math.floor(Date.now() / 1000) + i;
      await queryInterface.sequelize.query(
        `UPDATE LocalUsers SET runtime_bind_id = ? WHERE id = ?`,
        {
          replacements: [uniqueUnixTimestamp, localUsers[i].id],
          type: queryInterface.sequelize.QueryTypes.UPDATE
        }
      );
    }
  },

  async down(queryInterface, Sequelize) {
    // Add reverting commands here.
    // For this specific migration, you might not be able to revert the change
    // because you can't set it back to NULL if you don't know which rows were updated.
    // Alternatively, you can set them back to NULL for the purpose of this example,
    // but be aware this is usually not a safe operation for real-world scenarios.
    await queryInterface.sequelize.query(
      `UPDATE LocalUsers SET runtime_bind_id = NULL WHERE runtime_bind_id IS NOT NULL`
    );
  }
};
