'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up(queryInterface, Sequelize) {
    const linux = [
      {
        software: 'Avast Antivirus',
        process: ['avast']
      },
      {
        software: 'Avira Antivirus',
        process: ['AntiVir']
      },
      {
        software: 'Bitdefender Antivirus',
        process: ['BDLDaemon']
      },
      {
        software: 'ESET Antivirus',
        process: ['esets_daemon', 'eset/eea/sbin/startd']
      },
      {
        software: 'Kaspersky Antivirus',
        process: ['kesl']
      },
      {
        software: 'McAfee Antivirus',
        process: ['nails']
      },
      {
        software: 'Norton Antivirus',
        process: ['SymDaemon']
      },
      {
        software: 'Sophos Antivirus',
        process: ['savd', 'savservice']
      },
      {
        software: 'Total Antivirus',
        process: ['TotalAV']
      },
      {
        software: 'Trend Micro Antivirus',
        process: ['iCoreService', '/opt/ds_agent/ds_agent']
      },
      {
        software: 'Webroot SecureAnywhere Antivirus',
        process: ['WSDaemon']
      },
      {
        software: 'Windows Defender Antivirus',
        process: ['wdavdaemon']
      },
      {
        software: 'CrowdStrike',
        process: ['falcond']
      }
    ];

    const getCurrentDateTime = () => {
      const now = new Date();
      return now
        .toISOString()
        .slice(0, 19)
        .replace('T', ' ');
    };

    for (let i = 0; i < linux.length; i++) {
      let os = 'linux';
      let software = linux[i].software;
      let binding_name = linux[i].software + '_' + os;
      const createdAt = getCurrentDateTime();
      const updatedAt = getCurrentDateTime();

      const [softwareInstance] = await queryInterface.sequelize.query(
        `
          INSERT INTO SoftwareInstances (binding_name, software, os, is_edr, createdAt, updatedAt) VALUES
          ('${binding_name}', '${software}', '${os}', 1, '${createdAt}', '${updatedAt}')
        `
      );

      let software_id = softwareInstance;
      for (let j = 0; j < linux[i].process.length; j++) {
        let process = linux[i].process[j];
        await queryInterface.sequelize.query(
          `
            INSERT INTO SoftwareProcessBindings (software_id, process, createdAt, updatedAt) VALUES
            (${software_id}, '${process}', '${createdAt}', '${updatedAt}')
          `
        );
      }
    }
  },

  async down(queryInterface, Sequelize) {
    const [instance] = await queryInterface.sequelize.query(
      `
        SELECT id
        FROM SoftwareInstances si 
        WHERE si.is_edr = 1 AND si.os = 'linux'
      `
    );

    for (let i = 0; i < instance.length; i++) {
      let id = instance[i].id;
      await queryInterface.sequelize.query(
        `
          DELETE FROM SoftwareProcessBindings  
          WHERE software_id = ${id}
        `
      );

      await queryInterface.sequelize.query(
        `
          DELETE FROM SoftwareInstances  
          WHERE id = ${id}
        `
      );
    }
  }
};
