'use strict';

module.exports = {
  up: async (queryInterface, Sequelize) => {
    const securityPivots = [];

    const devicePolicies = await queryInterface.sequelize.query(
      'SELECT id, antivirus FROM DevicePolicies WHERE name != :policyName AND CHAR_LENGTH(antivirus) > 0 AND antivirus IS NOT NULL',
      {
        replacements: { policyName: 'Default device policies' },
        type: Sequelize.QueryTypes.SELECT,
      }
    );

    if(devicePolicies.length > 0) {
      for (const policy of devicePolicies) {
        const softwareInstances = await queryInterface.sequelize.query(
          'SELECT id, binding_name FROM SoftwareInstances WHERE binding_name LIKE :substring',
          {
            replacements: { substring: `%${policy.antivirus}%` },
            type: Sequelize.QueryTypes.SELECT,
          }
        );
  
        const getCurrentDateTime = () => {
          const now = new Date();
          return now
            .toISOString()
            .slice(0, 19)
            .replace('T', ' ');
        };
  
        for (const softwareInstance of softwareInstances) {
          securityPivots.push({
            policy_id: policy.id,
            software_id: softwareInstance.id,
            createdAt: getCurrentDateTime(),
            updatedAt: getCurrentDateTime(),
          });
        }
      }
  
      await queryInterface.bulkInsert('SecurityPolicyPivots', securityPivots, {});
      
      return Promise.resolve();
    }
  },

  down: async (queryInterface, Sequelize) => {
    await queryInterface.bulkDelete('SecurityPolicyPivots', null, {});
  },
};
