'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up (queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      // insert onboarding reports in privileges table
      await queryInterface.sequelize.query(
        `
          INSERT INTO Privilege (id, name, is_crud_allowed, privilege_id) VALUES
            (63, "Onboarding Reports", 1, 2)
        `, 
        { transaction }
      );

      // ADMIN:
      // insert in permissions
      const [adminPermission] = await queryInterface.sequelize.query(
        `
          INSERT INTO Permissions (create_permission, read_permission, update_permission, delete_permission, privilege_id) VALUES
            (1, 1, 1, 1, 63)
        `,
        { transaction }
      );

      // insert in template permissions of admin template (id = 2)
      const [adminTempPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO TemplatePermissions (temp_id, permission_id) VALUES
            (2, ${adminPermission})
        `,
        { transaction }
      );

      // Fetch all admin users
      const [adminUsers] = await queryInterface.sequelize.query(
          `
            SELECT DISTINCT u.id
              FROM Users u 
              INNER JOIN UserTemplates ut ON ut.user_id = u.id
              INNER JOIN TemplatePermissions tp ON tp.id = ut.temp_permission_id 
              INNER JOIN Templates t ON t.id = tp.temp_id 
              INNER JOIN Permissions p ON p.id = tp.permission_id 
              WHERE t.id = 2
        `,
        { transaction }
      );

      // insert in  user template
      if (adminUsers.length > 0) {
        const adminUserTemplateData = adminUsers.map(user => ({
          user_id: user.id,
          temp_permission_id: adminTempPermissions
        }));
        
        await queryInterface.bulkInsert('UserTemplates', adminUserTemplateData, { transaction });
      }

      // REPORT VIEWER:
      // insert in permissions
      const [rpPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO Permissions (create_permission, read_permission, update_permission, delete_permission, privilege_id) VALUES
            (1, 1, 1, 1, 63)
        `,
        { transaction }
      );

      // insert in template permissions of report viewer template (id = 5)
      const [rpTempPermissions] = await queryInterface.sequelize.query(
        `
          INSERT INTO TemplatePermissions (temp_id, permission_id) VALUES
            (5, ${rpPermissions})
        `,
        { transaction }
      );

      // Fetch all report viewer users
      const [rpUsers] = await queryInterface.sequelize.query(
          `
            SELECT DISTINCT u.id
              FROM Users u 
              INNER JOIN UserTemplates ut ON ut.user_id = u.id
              INNER JOIN TemplatePermissions tp ON tp.id = ut.temp_permission_id 
              INNER JOIN Templates t ON t.id = tp.temp_id 
              INNER JOIN Permissions p ON p.id = tp.permission_id 
              WHERE t.id = 5
        `,
        { transaction }
      );

      // insert in  user template
      if (rpUsers.length > 0) {
        const rpUserTemplateData = rpUsers.map(user => ({
          user_id: user.id,
          temp_permission_id: rpTempPermissions
        }));

        await queryInterface.bulkInsert('UserTemplates', rpUserTemplateData, { transaction });
      }

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  },

  async down (queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      await queryInterface.sequelize.query(
        `
          DELETE FROM Privilege
          WHERE id = 63
        `
      );

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  }
};
