'use strict';

/** @type {import('sequelize-cli').Migration} */
module.exports = {
  async up(queryInterface, Sequelize) {
    let transaction;
    try {
      transaction = await queryInterface.sequelize.transaction();

      // reduce column varchar lengths (required for unique constraint)
      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sDiscoveryData
            MODIFY COLUMN source_ip VARCHAR(50),
            MODIFY COLUMN destination_ip VARCHAR(50),
            MODIFY COLUMN destination_port VARCHAR(50),
            MODIFY COLUMN protocol VARCHAR(50);
        `,
        { transaction }
      );

      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sManualRules
            MODIFY COLUMN source_ip VARCHAR(50),
            MODIFY COLUMN destination_ip VARCHAR(50),
            MODIFY COLUMN destination_port VARCHAR(50),
            MODIFY COLUMN protocol VARCHAR(50);
        `,
        { transaction }
      );

      // add unique constraint
      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sDiscoveryData
            ADD CONSTRAINT Unique_S2S_Discovery UNIQUE (source_ip, destination_ip, destination_port, protocol);
        `,
        { transaction }
      );

      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sManualRules
            ADD CONSTRAINT Unique_S2S_Manual UNIQUE (source_ip, destination_ip, destination_port, protocol);
        `,
        { transaction }
      );

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  },

  async down(queryInterface, Sequelize) {
    let transaction;

    try {
      transaction = await queryInterface.sequelize.transaction();

      // remove unique constraint
      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sDiscoveryData
            DROP CONSTRAINT Unique_S2S_Discovery;
        `,
        { transaction }
      );

      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sManualRules
            DROP CONSTRAINT Unique_S2S_Manual;
        `,
        { transaction }
      );

      // change column varchar lengths back to original
      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sDiscoveryData
            MODIFY COLUMN source_ip VARCHAR(255),
            MODIFY COLUMN destination_ip VARCHAR(255),
            MODIFY COLUMN destination_port VARCHAR(255),
            MODIFY COLUMN protocol VARCHAR(255);
        `,
        { transaction }
      );

      await queryInterface.sequelize.query(
        `
          ALTER TABLE S2sManualRules
            MODIFY COLUMN source_ip VARCHAR(50),
            MODIFY COLUMN destination_ip VARCHAR(50),
            MODIFY COLUMN destination_port VARCHAR(50),
            MODIFY COLUMN protocol VARCHAR(50);
        `,
        { transaction }
      );

      await transaction.commit();
    } catch (error) {
      if (transaction) await transaction.rollback();
      throw error;
    }
  }
};
