'use strict';

module.exports = {
  up: async (queryInterface, Sequelize) => {
    const macAvNames = [
      'Avast Antivirus',
      'Avira Antivirus',
      'Bitdefender Antivirus',
      'ESET Antivirus',
      'Kaspersky Antivirus',
      'Sophos Antivirus',
      'Total Antivirus',
      'Trend Micro Antivirus',
      'Webroot SecureAnywhere Antivirus',
      'CrowdStrike'
    ];

    const linuxAvNames = [
      'ESET Antivirus',
      'Sophos Antivirus',
      'CrowdStrike'
    ];

    const macAvProcesses = {
      'Avast Antivirus': '/Applications/Avast.app/Contents/Backend/services/com.avast.daemon',
      'Avira Antivirus': '/Library/Application Support/Avira Security/privd.app/Contents/MacOS/privd',
      'Bitdefender Antivirus': '/Applications/BitdefenderVirusScanner.app/Contents/MacOS/BitdefenderVirusScanner',
      'ESET Antivirus': '/Applications/ESET Endpoint Antivirus.app/Contents/MacOS/startd',
      'Kaspersky Antivirus': '/Library/Application Support/Kaspersky Lab/KAV/Applications/Kaspersky Anti-Virus Agent.app/Contents/MacOS/kav_agent',
      'Sophos Antivirus': '/Applications/Sophos/Sophos Home.app/Contents/MacOS/Sophos Home',
      'Total Antivirus': '/Applications/TotalAV.app/Contents/MacOS/TotalAV',
      'Trend Micro Antivirus': '/Library/PrivilegedHelperTools/com.trendmicro.HouseCallForMacHelper',
      'Webroot SecureAnywhere Antivirus': '/Applications/Webroot SecureAnywhere.app/Contents/MacOS/WSDaemon.app/Contents/MacOS/WSDaemon',
      'CrowdStrike': '/Applications/Falcon.app/Contents/MacOS/Falcon'
    };

    const linuxAvProcesses = {
      'ESET Antivirus': '/opt/eset/eea/sbin/startd',
      'Sophos Antivirus': '/opt/sophos-spl/base/bin/sophos_watchdog,/opt/sophos-spl/plugins/av/sbin/av,/opt/sophos-spl/plugins/av/sbin/sophos_threat_detector',
      'CrowdStrike': '/opt/CrowdStrike/falcond'
    };

    // Loop through each macOS antivirus and check for its existence
    for (const avName of macAvNames) {
      const macAvInstance = await queryInterface.sequelize.query(
        `SELECT id FROM SoftwareInstances WHERE software = :avName AND os = 'macos';`,
        { 
          replacements: { avName },
          type: Sequelize.QueryTypes.SELECT 
        }
      );
      
      if (macAvInstance.length > 0) {
        const softwareId = macAvInstance[0].id;
        const daemonProcess = macAvProcesses[avName];

        await queryInterface.sequelize.query(
          `UPDATE SoftwareProcessBindings SET process = :daemonProcess WHERE software_id = :softwareId;`,
          {
            replacements: { daemonProcess, softwareId },
            type: Sequelize.QueryTypes.UPDATE
          }
        );
        
        console.log(`Updated process for ${avName} with software_id ${softwareId}: ${daemonProcess}`);
      }
    }

    // Loop through each linux antivirus and check for its existence
    for (const avName of linuxAvNames) {
      const linuxAvInstance = await queryInterface.sequelize.query(
        `SELECT id FROM SoftwareInstances WHERE software = :avName AND os = 'linux';`,
        { 
          replacements: { avName },
          type: Sequelize.QueryTypes.SELECT 
        }
      );
      
      if (linuxAvInstance.length > 0) {
        const softwareId = linuxAvInstance[0].id;
        const daemonProcess = linuxAvProcesses[avName];

        await queryInterface.sequelize.query(
          `UPDATE SoftwareProcessBindings SET process = :daemonProcess WHERE software_id = :softwareId;`,
          {
            replacements: { daemonProcess, softwareId },
            type: Sequelize.QueryTypes.UPDATE
          }
        );
        
        console.log(`Updated process for ${avName} with software_id ${softwareId}: ${daemonProcess}`);
      }
    }
  },

  down: async (queryInterface, Sequelize) => {
  }
};
