'use strict';

module.exports = {
  up: async (queryInterface, Sequelize) => {
    const macAvNames = [
      'Avira Antivirus',
      'ESET Antivirus',
      'CrowdStrike',
    ];

    const macAvProcesses = {
      'Avira Antivirus': '/Library/Application Support/Avira Security/privd.app/Contents/MacOS/privd',
      'ESET Antivirus': '/Applications/ESET Endpoint Security.app/Contents/MacOS/esets_daemon',
      'CrowdStrike': '/Applications/Falcon.app/Contents/PlugIns/StaticAnalysis.bundle/Contents/XPCServices/FXService.xpc/Contents/MacOS/FXService',
    };

    const macos = [
      {
        software: 'VIPRE Antivirus',
        process: ['/Library/Vipre/Agent/vipre']
      },
    ]

    const getCurrentDateTime = () => {
      const now = new Date();
      return now
        .toISOString()
        .slice(0, 19)
        .replace('T', ' ');
    };


    const [vipreExist] = await queryInterface.sequelize.query(`SELECT id FROM SoftwareInstances WHERE software = 'VIPRE Antivirus' AND os = 'macos'`);

    if(!vipreExist.length){
      for (let i = 0; i < macos.length; i++) {
        let os = 'macos';
        let software = macos[i].software;
        let binding_name = macos[i].software + '_' + os;
        const createdAt = getCurrentDateTime();
        const updatedAt = getCurrentDateTime();
  
        const [softwareInstance] = await queryInterface.sequelize.query(
          `
            INSERT INTO SoftwareInstances (binding_name, software, os, is_edr, createdAt, updatedAt) VALUES
            ('${binding_name}', '${software}', '${os}', 1, '${createdAt}', '${updatedAt}')
          `
        );
  
        let software_id = softwareInstance;
        for (let j = 0; j < macos[i].process.length; j++) {
          let process = macos[i].process[j];
          await queryInterface.sequelize.query(
            `
              INSERT INTO SoftwareProcessBindings (software_id, process, createdAt, updatedAt) VALUES
              (${software_id}, '${process}', '${createdAt}', '${updatedAt}')
            `
          );
        }
      }
    }

    // Loop through each macOS antivirus and check for its existence
    for (const avName of macAvNames) {
      const macAvInstance = await queryInterface.sequelize.query(
        `SELECT id FROM SoftwareInstances WHERE software = :avName AND os = 'macos';`,
        { 
          replacements: { avName },
          type: Sequelize.QueryTypes.SELECT 
        }
      );
      
      if (macAvInstance.length > 0) {
        const softwareId = macAvInstance[0].id;
        const daemonProcess = macAvProcesses[avName];

        await queryInterface.sequelize.query(
          `UPDATE SoftwareProcessBindings SET process = :daemonProcess WHERE software_id = :softwareId;`,
          {
            replacements: { daemonProcess, softwareId },
            type: Sequelize.QueryTypes.UPDATE
          }
        );
      }
    }
  },

  down: async (queryInterface, Sequelize) => {
    const [vipreExist] = await queryInterface.sequelize.query(`SELECT id FROM SoftwareInstances WHERE software = 'VIPRE Antivirus' AND os = 'macos'`);

    if(vipreExist.length) {
      const macosBindingName = 'VIPRE Antivirus_macos';
  
      const [macosInstance] = await queryInterface.sequelize.query(
        `
          SELECT id
          FROM SoftwareInstances
          WHERE binding_name = '${macosBindingName}' AND os = 'macos'
        `
      );
  
      // Delete the macOS process bindings first
      if (macosInstance.length > 0) {
        const software_id = macosInstance[0].id;
        await queryInterface.sequelize.query(
          `
            DELETE FROM SoftwareProcessBindings
            WHERE software_id = ${software_id}
          `
        );
    
        // Now delete the SoftwareInstance record
        await queryInterface.sequelize.query(
          `
            DELETE FROM SoftwareInstances
            WHERE id = ${software_id}
          `
        );
      }
    }
  }
};
