const nodemailer = require('nodemailer');
const template = require('swig');
let settings = require('../routes/common/settings');
const models = require('../models');
const Queue = require('./queue');
const { LICENSE_EXPIRY_EMAIL } = require('../utils/common');
const moment = require('moment');
const { getToken, sendEmail } = require('../utils/msal');

class EmailQueue extends Queue {
  constructor(queueName, opts) {
    super(queueName, opts)
  }

  isReminderAlreadySent() {
    return models.License.findOne({
      order: [['id', 'DESC']],
      limit: 1
    })
      .then(license => {
        if (license) {
          const {
            expiry_reminder_sent_at,
          } =  license;

          const lastReminderSentAt = moment(expiry_reminder_sent_at);
          return lastReminderSentAt.isSame(moment(), 'day');
        }
      })
  }

  async consumer(msg) {

    const {
      recipients,
      subject,
      file,
      context,
      callbacks,
      attachments,
      cronType
    } = msg;

    // return if expiry mail already sent today 
    if(cronType === LICENSE_EXPIRY_EMAIL && await this.isReminderAlreadySent()) return;

    const OAuthConfig = await models.Settings.findOne({
      where: {
        id: 1
      }
    });

    const config = await settings.getSmtpSettings();

    const transporter = nodemailer.createTransport(config.transporter);

    console.log('Email Consumer: ', msg);

    let compile = template.compileFile(file);
    let content = compile(context);

    let mailOptions = {
      from: config.from,
      to: recipients,
      subject: subject,
      html: content,
      client: msg?.client,
      attachments
    };

    if(!OAuthConfig.oauth_config){
      console.log('Sending email...');
  
      // send mail with defined transport object
      await transporter.sendMail(mailOptions);
  
      console.log(`Email sent to ${mailOptions.to}...`);
    }else if(OAuthConfig.oauth_config){
      console.log('Sending email using OAuth...');

      (async () => {
        const accessToken = await getToken();
      
        if (accessToken) {
          await sendEmail(accessToken, mailOptions);
        }
      })();
    }
  }
}

module.exports = new EmailQueue('email-queue');