const models = require('../models');

const {upload} = require('../routes/common/content-upload');
const {controllerZipGenerator} = require('../dao/controllers');
const {gatewayZipGenerator} = require('../dao/gateways');
const Queue = require('./queue');

/**
 * Class for uploading to queue
 * @extends Queue
 */
class UploadToContent extends Queue {

  constructor(queueName, opts) {
    super(queueName, opts)
  }

  async consumer(data) {
    console.log('Consumer received: ', data);
    if (data.originalInstaller && data.component === 'controller') {
      const controllers = await models.Controller.findAll({
        include: [
          {
            model: models.ComponentRole,
            as: 'Role',
            attributes: ['role']
          }
        ]
      });

      await models.Controller.update({
        installer_status: 'pending'
      }, {
        where: {}
      })

      for (const c of controllers) {
        console.log(`Generating controller package for: ${c.id}`);
        await controllerZipGenerator(c, '0', false);
      }

    } else if (data.originalInstaller && data.component === 'gateway') {
      const gateways = await models.Gateway.findAll({
        where: {
          buildOs: data.os
        }
      });

      await models.Gateway.update({
        installer_status: 'pending'
      }, {
        where: {
          buildOs: data.os
        }
      })

      for (const g of gateways) {
        console.log(`Generating controller package for: ${g.id}`);
        await gatewayZipGenerator(g, '0', false);
      }
    } else {
      await upload(data);
    }
    console.log('Content Uploaded: ', data);
  }
}

module.exports = new UploadToContent('content-queue', {consumerInstance: 1});