var express = require('express');
var router = express.Router();
const verifyPermissions = require('../../middlewares/verifyPermissions');

const {
  createBackup,
  getBackups,
  downloadUrl,
  deleteBackup,
  restoreBackup
} = require('../../dao/backups');

const backValidators = require('../../validators/backups');
const validate = require('../../validators/validate');
const validatePassword = require('../../middlewares/validatePassword');

router.post('/download', verifyPermissions('System Backup'), createBackup);

router.get('/', verifyPermissions('System Backup'), getBackups);

router.post('/restore/:id', validatePassword, backValidators.restoreBackup(), validate, verifyPermissions('System Backup'), restoreBackup);

router.get('/downloadUrl/:id', backValidators.downloadBackup(), validate, downloadUrl);

router.delete('/:id', backValidators.deleteBackup(), validate, verifyPermissions('System Backup'), deleteBackup);

module.exports = router;
