const express = require('express');

const router = express.Router();
const serverBridgelayer = require('../../dao/serverbridge');
const validatePassword = require('../../middlewares/validatePassword');
const verifyPermissions = require('../../middlewares/verifyPermissions');

const policiesValidator = require('../../validators/policies');
const validate = require('../../validators/validate');

router.post(
  '/b2b-bridge',
  policiesValidator.createB2BBridge(),
  validate,
  verifyPermissions('RDI Bridges'),
  serverBridgelayer.createB2BBridge
);

router.post(
  '/rbi-bridge',
  policiesValidator.createRBIBridge(),
  validate,
  verifyPermissions('RBI Bridges'),
  serverBridgelayer.createRBIBridge
);

router.post(
  '/',
  policiesValidator.createBridge(),
  validate,
  verifyPermissions([
    'Server Bridges',
    'Device Bridge (x86)',
    'Device Bridge (ARM)',
    'RDI Bridges'
  ]),
  serverBridgelayer.createBridge
);

router.put(
  '/bridge/b2b-bridge/:id(\\d+)',
  // policiesValidator.updateBridge(),
  // validate,
  verifyPermissions('RDI Bridges'),
  serverBridgelayer.updateB2Bbridge
);

router.put(
  '/bridge/rbi-bridge/:id(\\d+)',
  verifyPermissions('RBI Bridges'),
  serverBridgelayer.updateRBIbridge
);

router.put(
  '/bridge/:id(\\d+)',
  policiesValidator.updateBridge(),
  validate,
  verifyPermissions([
    'Server Bridges',
    'Device Bridge (x86)',
    'Device Bridge (ARM)'
  ]),
  serverBridgelayer.updatebridge
);

router.get('/', 
  verifyPermissions(['Server Bridges','Device Bridge (x86)','Device Bridge (ARM)','RDI Bridges']), 
  serverBridgelayer.getBridges);

router.get('/physical_devices', verifyPermissions(['Server Bridges','Device Bridge (x86)','Device Bridge (ARM)']), serverBridgelayer.getPhysicalDevices);

router.get('/devices', verifyPermissions(['Server Bridges','Device Bridge (x86)','Device Bridge (ARM)']), serverBridgelayer.getDevices);

router.delete(
  '/bridge/:id(\\d+)',
  validatePassword,
  policiesValidator.deletePolicy(),
  validate,
  verifyPermissions([
    'Server Bridges',
    'Device Bridge (x86)',
    'Device Bridge (ARM)',
    'RDI Bridges'
  ]),
  serverBridgelayer.deleteBridge
);

router.get(
  '/bridge/:id(\\d+)',
  verifyPermissions(['Device Bridge (x86)','Device Bridge (ARM)','RDI Bridges']),
  policiesValidator.getPolicy(),
  validate,
  serverBridgelayer.getSingleBridge
);

router.patch(
  '/bridge/:id(\\d+)/enable/:value(false|true)',
  policiesValidator.setEnableValue(),
  validate,
  verifyPermissions([
    'Server Bridges',
    'Device Bridge (x86)',
    'Device Bridge (ARM)',
    'RDI Bridges'
  ]),
  serverBridgelayer.setEnable
);

router.get('/getServerClients', 
  verifyPermissions(['Server Bridges','Device Bridge (x86)','Device Bridge (ARM)','RDI Bridges']), 
  serverBridgelayer.getServerClients);

router.get('/bridge/download/:id(\\d+)', serverBridgelayer.downloadBridge);

router.get(
  '/bridge/downloadPackage/:token',
  serverBridgelayer.downloadBridgeFromLink
);

router.get('/bridge/sendBridge/:id(\\d+)', serverBridgelayer.sendBridge);

module.exports = router;
