const router = require('express').Router();
const serviceGroup = require('../../dao/servicegroups');
const serviceGroupValidator = require('../../validators/servicegroups');
const validate = require('../../validators/validate');
const verifyPermissions = require('../../middlewares/verifyPermissions');

router.get('/', verifyPermissions('Service Groups'), serviceGroup.getAllGroups);
router.get('/available', serviceGroup.getAvailableGroups);
router
  .route('/services/:groupId')
  .get(verifyPermissions('Service Groups'),serviceGroup.getGroupServicesWithPagination);
router.get(
  '/getAllServices/:groupId/:searchServiceTerm',
  serviceGroup.getAllMacthingServices
);
router.post('/', serviceGroupValidator.addServiceGroup(), validate, verifyPermissions('Service Groups'), serviceGroup.addGroup);
router.post('/addService/:groupId/:serviceId', verifyPermissions('Service Groups'), serviceGroup.addService);
router.delete('/deleteService/:groupId/:serviceId', verifyPermissions('Service Groups'), serviceGroup.deleteService);
router.get('/:id', verifyPermissions('Service Groups'), serviceGroup.getGroupDetail);
router.put('/:id', serviceGroupValidator.updateServiceGroup(), validate, verifyPermissions('Service Groups'), serviceGroup.editGroup);
router.delete('/bulk', verifyPermissions('Service Groups'), serviceGroup.bulkDeleteGroups);
router.delete('/:id', serviceGroupValidator.deleteServiceGroup(), validate, verifyPermissions('Service Groups'), serviceGroup.deleteGroup);

module.exports = router;
