const router = require('express').Router();
const verifyPermissions = require('../../middlewares/verifyPermissions');

const { 
  getTemplates,
  getTemplateUsersCount,
  getCreatedTemplates,
  getTemplateById,
  getUserTemplate,
  getPossibleAdditionalPermissions,
  getAdditionalPermissions,
  getCreatedAdditionalPermissions,
  createTemplate,
  createAdditionalPermissions,
  updateTemplate,
  updateAdditionalTemplate,
  deleteTemplate,
  deleteAdditionalPermissions,
} = require('../../dao/templates');

router.get('/', getTemplates);

router.get('/created', verifyPermissions('Roles And Permissions'), getCreatedTemplates);

router.get('/user', getUserTemplate);

router.get('/additional-permissions', getAdditionalPermissions);

router.get('/additional-possible', getPossibleAdditionalPermissions);

router.get('/:id', getTemplateById);

router.get('/additional-requests/:id', getCreatedAdditionalPermissions);

router.get('/user-count/:id', verifyPermissions('Roles And Permissions'), getTemplateUsersCount);

router.post('/', verifyPermissions('Roles And Permissions'), createTemplate);

router.post('/additional', verifyPermissions('Roles And Permissions'), createAdditionalPermissions);

router.put('/:id', verifyPermissions('Roles And Permissions'), updateTemplate);

router.put('/additional/:id', verifyPermissions('Roles And Permissions'), updateAdditionalTemplate);

router.delete('/:id', verifyPermissions('Roles And Permissions'), deleteTemplate);

router.delete('/additional/:id', verifyPermissions('Roles And Permissions'), deleteAdditionalPermissions);

module.exports = router;