var express = require('express');
var router = express.Router();
const {
  getWorkLoads,
  getAvailableWorkloadDevices,
  createWorkload,
  getWorkloadById,
  editWorkloadById,
  deleteWorkloadById,
  workloadBulkAction,
  saveWorkDiscovery,
  getS2SDiscoveryData,
  getPaginatedS2SDiscoveryData,
  toggleS2sDiscovery,
  s2sDiscoveryManualData,
  getManualRuleData,
  getWorkloadDevices,
  deleteManualRule,
  saveMicroSegTag,
  getAllWorkloadDevices,
  getAllS2SDiscoveryData,
  saveAllWorkDiscovery,
  getAllPaginatedS2SDiscoveryData,
  allS2sDiscoveryManualData,
  saveAllMicroSegTag,
  deleteFromAllManualRules,
  deleteFromAllAutoRules,
  getAllManualRuleData,
  getRefreshWorkloads,
  getWorklaodName,
  flushDiscovery
} = require('../../dao/workloads');

const validatePassword = require('../../middlewares/validatePassword');
const validation = require('../../validators/workload');
const validate = require('../../validators/validate');
const s2sMicrosegLicenseMiddleware = require('../../middlewares/validateS2SLicense');
const verifyPermissions = require('../../middlewares/verifyPermissions');
const { bulkDeleteGroups } = require('../../dao/servicegroups');

router.use(s2sMicrosegLicenseMiddleware);

router
  .route('/')
  .post(validation.createWorkLoad(), validate, verifyPermissions('Workloads'), createWorkload)
  .get(verifyPermissions(['Users','Workloads']),getWorkLoads);

router
  .route('/detail/:workloadId')
  .get(verifyPermissions('Workloads'),getWorkloadById)
  .put(validation.editWorkload(), validate, verifyPermissions('Workloads'), editWorkloadById)
  .delete(verifyPermissions('Workloads'), deleteWorkloadById);

router.route('/bulk-action').post(verifyPermissions('Workloads'), workloadBulkAction);

router.get(
  '/devices/available',
  verifyPermissions(['Microsegments','Workloads']),
  validation.getAvailableWorloadDevices(),
  validate,
  getAvailableWorkloadDevices
);

router.put('/save-work-discovery/:id', verifyPermissions(['Workloads', 'Microsegments']), saveWorkDiscovery);

router.put('/save-work-discovery/', verifyPermissions('Microsegments'), saveAllWorkDiscovery);

router.put('/save-microseg-tag/:id', verifyPermissions('Workloads'), saveMicroSegTag);

router.put('/save-microseg-tag/', verifyPermissions('Microsegments'), saveAllMicroSegTag);

router.get('/work-discovery/:workloadId', verifyPermissions('Workloads'), getS2SDiscoveryData);

router.get('/get-worklaod-name/:workloadId', verifyPermissions('Workloads'), getWorklaodName);

router.get('/all-work-discovery/', verifyPermissions('Microsegments'), getAllS2SDiscoveryData);

router.get('/refresh/:workloadId', getRefreshWorkloads);

router.get('/work-paginated-discovery',verifyPermissions('Workloads'), getPaginatedS2SDiscoveryData);

router.get('/all-work-paginated-discovery', verifyPermissions('Microsegments'), getAllPaginatedS2SDiscoveryData);

router.patch('/work-discovery/toggle/:discoveryId', verifyPermissions(['Workloads', 'Microsegments']), toggleS2sDiscovery);

router.post('/work-discovery-manual/:workloadId', verifyPermissions(['Workloads']), s2sDiscoveryManualData);

router.post('/work-discovery-manual/', verifyPermissions('Microsegments'), allS2sDiscoveryManualData);

router.get('/work-discovery-manual/:workloadId', verifyPermissions('Workloads'), getManualRuleData);

router.get('/work-discovery-manual/', verifyPermissions('Microsegments'), getAllManualRuleData);

router.get('/workload-devices/:workloadId', verifyPermissions('Workloads'), getWorkloadDevices);

router.get('/all-workload-devices', verifyPermissions('Microsegments'), getAllWorkloadDevices);


router.delete('/work-discovery-manual/:workloadId', verifyPermissions(['Workloads', 'Microsegments']), deleteManualRule);

router.delete('/work-discovery-manual', verifyPermissions('Workloads'), deleteFromAllManualRules);

router.delete('/work-discovery-auto', verifyPermissions(['Microsegments','Workloads']), deleteFromAllAutoRules);

router.delete('/flush-discovery/', verifyPermissions('Workloads'), flushDiscovery);

module.exports = router;
