const fs = require('fs');
const path = require('path');
const helper = require('../common/helper');

module.exports = {
  prepareLSC: function(ldapConfig, dbConfig) {
    const isOLdap = ldapConfig.LDAP_user.indexOf('uid=') === 0;
    const dirType = isOLdap ? 'oldap' : 'ad';
    console.log(">>> dirType ", dirType);
    return helper
      .readFile(path.join(__dirname, '..', '..', 'lsc', `sql-map-config.d/Clients.${dirType}.xml`))
      .then(clientsXML => {
        return helper.writeFile(path.join(__dirname, '..', '..', 'lsc', 'sql-map-config.d/Clients.xml'), clientsXML);
      })
      .then(() => {
        return helper.readFile(path.join(__dirname, '..', '..', 'lsc', `lsc.${dirType}.xml`));
      })
      .then(lscXML => {
        const {
          LDAP_dc,
          LDAP_url,
          LDAP_user: LDAP_USERNAME,
          LDAP_password: LDAP_PASSWORD
        } = ldapConfig;

        const variableMap = {
          LDAP_URL: `${LDAP_url}/${LDAP_dc}`,
          LDAP_DC: LDAP_dc,
          LDAP_USERNAME,
          LDAP_PASSWORD,
          DB_URL: `jdbc:mysql://${dbConfig.host}:3306/${dbConfig.database}`,
          DB_USERNAME: dbConfig.username,
          DB_PASSWORD: dbConfig.password
        };

        return helper.writeFile(
          path.join(__dirname, '..', '..', 'lsc', 'lsc.xml'),
          lscXML.replace(/\$\{(.*)\}/g, (fullMatch, variableName) => {
            return variableMap[variableName];
          })
        );
      });
  },
  schedule: function(cron, syncTime, cleanTime) {
    return new Promise((resolve, reject) => {
      cron.send({
        type: 'SCHEDULE',
        expression: `*/${syncTime} * * * *`,
        taskName: 'LDAP_SYNC'
      });

      resolve();
    });
  },
  destroy: function(cron) {
    return new Promise((resolve, reject) => {
      cron.send({
        type: 'DESTROY',
        taskName: 'LDAP_SYNC'
      });

      cron.send({
        type: 'DESTROY',
        taskName: 'LDAP_CLEAN'
      });

      resolve();
    });
  }
};
