const { body, param } = require('express-validator');

const getGateway = () => [
    ...id(param),
];

const addGateway = () => [
    ...Name(body),
    ...buildOs(body),
    ...controller(body),
    ...deployment_location(body),
    ...email(body),
    ...hardware_id(body),
    ...public_ipv4_address(body),
    ...public_ipv4_port(body),
    ...port(body),
    ...protocol(body),
    ...ipv4_address(body),
    ...spa_port(body),
];

const updateGateway = () => [
    ...Name(body),
    ...buildOs(body),
    ...controller(body),
    ...deployment_location(body),
    ...email(body),
    ...hardware_id(body),
    ...public_ipv4_address(body),
    ...public_ipv4_port(body),
    ...port(body),
    ...protocol(body),
    ...ipv4_address(body),
    ...spa_port(body),
    ...id(param)
];

const Name = check => [
    check('gateway.Name')
    .exists()
    .not()
    .isEmpty()
];

const buildOs = check => [
    check('gateway.buildOs')
    .exists()
    .not()
    .isEmpty()
];

const controller = check => [
    check('gateway.controller')
    .exists()
    .not()
    .isEmpty()
];

const deployment_location = check => [
    check('gateway.deployment_location')
    .exists()
    .not()
    .isEmpty()
];


const email = check => [
    check('gateway.email')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
];

const hardware_id = check => [
    check('gateway.hardware_id')
    .exists()
    .not()
    .isEmpty()
];


const port = check => [
    check('gateway.port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid private port.');
        }
        return true;
    })
    .withMessage('Private port is required.')
];

const protocol = check => [
    check('gateway.protocol')
    .exists()
    .not()
    .isEmpty()
];


const ipv4_address = check => [
    check('gateway.ipv4_address')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const validIpExpression = /^(([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])\.){3}([0-9]|[1-9][0-9]|1[0-9]{2}|2[0-4][0-9]|25[0-5])$/;
        if (!validIpExpression.test(value)) {
            throw new Error("Invalid private ip.")
        }
        return true;
    })
    .withMessage('Private IP is required.')
]

const public_ipv4_address = check => [
    check('gateway.public_ipv4_address')
    .exists()
    .not()
    .isEmpty()
    .custom(value => {
        const validIpExpression = /^(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|\*)(?:,\s*(?:(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)\.(?:25[0-5]|2[0-4]\d|[01]?\d\d?)|\*))*$/;
        if (!validIpExpression.test(value)) {
            throw new Error("Public IP list contain invalid IP(s).")
        }    
        return true;
    })
    .withMessage('Public IP is required.')
]


const public_ipv4_port = check => [
    check('gateway.public_ipv4_port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid public port.');
        }
        return true;
    })
    .withMessage('Public port is required.')
];


const spa_port = check => [
    check('gateway.spa_port')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom(value => {
        const validPortExpression = /^([0-9]{1,4}|[1-5][0-9]{4}|6[0-4][0-9]{3}|65[0-4][0-9]{2}|655[0-2][0-9]|6553[0-5])$/;
        if (!validPortExpression.test(value)) {
            throw new Error('Invalid private port.');
        }
        return true;
    })
    .withMessage('Private port is required.')
];

const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
]

module.exports = {
    getGateway,
    addGateway,
    updateGateway
}