const { body, param } = require('express-validator');


const updateLdapUser = () => [
    ...LDAP_user(body),
    ...LDAP_url(body),
    ...LDAP_connectTimeout(body),
    ...LDAP_timeout(body),
    ...LDAP_dc(body),
    ...id(param)
]

const addLdapUser = () => [
    ...LDAP_user(body),
    ...LDAP_url(body),
    ...LDAP_connectTimeout(body),
    ...LDAP_timeout(body),
    ...LDAP_dc(body)
]


const deleteLdap = () => [
    ...id(param)
]


const LDAP_user = check => [
    check('ldap.LDAP_user')
    .exists()
    .not()
    .isEmpty()
]


const LDAP_url = check => [
    check('ldap.LDAP_url')
    .exists()
    .not()
    .isEmpty()
]


const LDAP_connectTimeout = check => [
    check('ldap.LDAP_connectTimeout')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value) => {
        if(value < 10){
            throw new Error("Invalid Connection Timout value.");
        }
        return true;
    })
]


const LDAP_timeout = check => [
    check('ldap.LDAP_timeout')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .custom((value) => {
        if(value < 10){
            throw new Error("Invalid Timeout value.");
        }
        return true;
    })
]


const LDAP_dc = check => [
    check('ldap.LDAP_dc')
    .exists()
    .not()
    .isEmpty()
]

const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
]


module.exports = {
    updateLdapUser,
    addLdapUser,
    deleteLdap
}