const { body, param } = require('express-validator');
var httpErrors = require('http-errors');


const createLocalUser = () => [
    ...name(body),
    ...email(body),
    ...userName(body),
]


const updateLocalUser = () => [
    ...name(body),
    ...email(body),
    ...userName(body),
    ...id(param)
]


const forgotPassword = () => [
    ...resetPassword(body)   
]


const id = check => [
    check('id')
    .exists()
    .not()
    .isEmpty()
    .isNumeric()
    .withMessage('Invalid Id.')
]

const name = check => [
    check('localUser.name')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
]

const email = check => [
    check('localUser.email')
    .exists()
    .not()
    .isEmpty()
    .isEmail()
    .normalizeEmail({ gmail_remove_dots: false })
    .withMessage('Invalid Email.')
];

const userName = check => [
    check('localUser.userName')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Username is required.')
];

const password = check => [
    check('localUser.password')
    .exists()
    .not()
    .isEmpty()
    .custom((value) => {

        if(!isValidPassword(value)){
            throw new httpErrors.UnprocessableEntity('Use between 8 to 31 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks');
        }

        return true;
    })
    .withMessage('Password is required.')
]

const updatePassword = check => [
    check('localUser.password')
    .custom((value) => {
        if(value && value.trim()){
            if(!isValidPassword(value)){
                throw new httpErrors.UnprocessableEntity('Use between 8 to 31 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks');
            }
        }
        return true;
    })
    .withMessage('Password is required.')
]

const resetPassword = check => [
    check('password')
    .exists()
    .not()
    .isEmpty()
    .custom((value) => {

        if(!isValidPassword(value)){
            throw new httpErrors.UnprocessableEntity('Use between 8 to 31 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks');
        }

        return true;
    })
    .withMessage('Password is required.')
]

const isValidPassword = (password) => {
    const passwordExpression = /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,31}$/;
    if(!passwordExpression.test(password)){
        return false;
    }
    return true;
}

module.exports = {
    createLocalUser,
    updateLocalUser,
    forgotPassword
}