const { body, param, query } = require('express-validator');
const { isArray } = require('lodash');

const createPolicy = () => {
  return [
    ...name(body),
    ...accessRestriction(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};

const updatePolicy = () => {
  return [
    ...name(body),
    ...accessRestriction(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};

const getAllOs = () => {
  return [...id(query)];
};

const getPolicy = () => {
  return [...id(param)];
};

const deletePolicy = () => {
  return [...id(param)];
};

const setEnableValue = () => {
  return [...id(param), ...value(param)];
};

const createDevicePolicy = () => {
  return [
    ...name(body),
    ...antivirus(body),
    ...isEPPClient(body),
    ...isBlocked(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body),
    ...riskScore(body)
  ];
};

const updateDevicePolicy = () => {
  return [
    ...name(body),
    ...antivirus(body),
    ...isEPPClient(body),
    ...isBlocked(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body),
    ...riskScore(body)
  ];
};

const getDevicePolicy = () => {
  return [...id(param)];
};

const deleteDevicePolicy = () => {
  return [...id(param)];
};

//SOFTWARE PROCESS POLICIES

const createSoftwareProcessPolicy = () => {
  return [
    ...name(body),
    ...isBlocked(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};
const updateSoftwareProcessPolicy = () => {
  return [
    ...name(body),
    ...isBlocked(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};
const getSoftwareProcessPolicy = () => {
  return [...id(param)];
};
const deleteSoftwareProcessPolicy = () => {
  return [...id(param)];
};

//SOFTWARE POLICIES

const createSoftwarePolicy = () => {
  return [
    ...name(body),
    // ...gracePeriod(body),
    ...client(body),
    ...clientGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};
const updateSoftwarePolicy = () => {
  return [
    ...name(body),
    // ...gracePeriod(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body)
  ];
};
const getSoftwarePolicy = () => {
  return [...id(param)];
};
const deleteSoftwarePolicy = () => {
  return [...id(param)];
};

//OS POLICIES

const createOsPolicy = () => {
  return [...os(body), ...osVersion(body), ...isBlocked(body)];
};

const updateOsPolicy = () => {
  return [...os(body), ...osVersion(body), ...isBlocked(body)];
};

const getOsPolicy = () => {
  return [...id(param)];
};

const deleteOsPolicy = () => {
  return [...id(param)];
};

//Server Bridges validators

const createBridge = () => {
  return [...name(body), ...clientIdCheck(body), ...deviceIds(body)];
};

const createB2BBridge = () => {
  return [...name(body), ...clientIdCheck(body), ...userIds(body)];
};

const createRBIBridge = () => {
  return [...name(body), ...clientIdCheck(body), ...userIds(body)];
};

const updateBridge = () => {
  return [...name(body), ...clientIdCheck(body), ...deviceIds(body)];
};

// Routing Policy

const createRoutingPolicy = () => {
  return [
    ...name(body),
    ...gateway(body),
    ...gatewayGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allGateways(body),
    ...allServices(body)
  ];
};

const getRoutingPolicy = () => {
  return [...id(param)];
};

const updateRoutingPolicy = () => {
  return [
    ...name(body),
    ...gateway(body),
    ...gatewayGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allGateways(body),
    ...allServices(body)
  ];
};

const deleteRoutingPolicy = () => {
  return [...id(param)];
};

// Data volume policy
const createDataVolumePolicy = () => {
  return [
    ...name(body),
    ...client(body),
    ...clientGroup(body),
    ...allClients(body),
    ...allServices(body),
    ...clientDownloadRate(body),
    ...clientUploadRate(body)
  ];
};
const updateDataVolumePolicy = () => {
  return [
    ...name(body),
    ...client(body),
    ...clientGroup(body),
    ...service(body),
    ...serviceGroup(body),
    ...allClients(body),
    ...allServices(body),
    ...clientDownloadRate(body),
    ...clientUploadRate(body)
  ];
};
const getDataVolumePolicy = () => {
  return [...id(param)];
};
const deleteDataVolumePolicy = () => {
  return [...id(param)];
};

const deviceIds = check => [
  check('deviceIds')
    .exists()
    .custom((value, { req }) => {
      if (value && !Array.isArray(value)) {
        throw new Error('DeviceIds should be an array');
      }
      const set = [...new Set(value)];

      if (set.length != value.length) {
        throw new Error('DeviceIds must be unique');
      }

      const regex = /^([0-9A-Fa-f]{2}[:-]){5}([0-9A-Fa-f]{2})$/;
      value.forEach(val => {
        if (!regex.test(val)) {
          throw new Error('DeviceIds should be proper mac addresses');
        }
      });
      return true;
    })
];

const userIds = check => [
  check('userIds')
    .exists()
    .custom((value, { req }) => {
      if (value && !Array.isArray(value)) {
        throw new Error('UserIds should be an array');
      }
      const set = [...new Set(value)];

      if (set.length != value.length) {
        throw new Error('UserIds must be unique');
      }

      return true;
    })
];

const clientIdCheck = check => [
  check('clientId')
    .exists()
    .not()
    .isEmpty()
    .withMessage('ClientId is required.')
];

const name = check => [
  check('name')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Name is required.')
];

const riskScore = check => [
  check('risk_score_threshold')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Risk score is required.')
    .isNumeric('Risk score must be an integer')
];

const id = check => [
  check('id')
    .exists()
    .isNumeric()
    .not()
    .isEmpty()
    .withMessage('Id is required.')
];

const client = check => [
  check('client')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_clients && value && !Array.isArray(value)) {
        throw new Error('Client should be an array');
      }
      return true;
    })
];

const clientGroup = check => [
  check('clientGroup')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_clients && value && !Array.isArray(value)) {
        throw new Error('Client Group should be an array');
      }
      return true;
    })
];

const service = check => [
  check('service')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_services && value && !Array.isArray(value)) {
        throw new Error('Service should be an array');
      }
      return true;
    })
];

const serviceGroup = check => [
  check('serviceGroup')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_services && value && !Array.isArray(value)) {
        throw new Error('Service group should be an array');
      }
      return true;
    })
];

const allClients = check => [
  check('all_clients')
    // .exists()
    .custom((value, { req }) => {
      if (
        value &&
        !(typeof value === 'boolean') &&
        !(value === 'true' || value === 'false')
      )
        throw new Error('Value should be boolean');
      return true;
    })
];

const allServices = check => [
  check('all_services')
    // .exists()
    .custom((value, { req }) => {
      if (
        value &&
        !(typeof value === 'boolean') &&
        !(value === 'true' || value === 'false')
      )
        throw new Error('Value should be boolean');
      return true;
    })
];

const allGateways = check => [
  check('all_gateways')
    // .exists()
    .custom((value, { req }) => {
      if (
        value &&
        !(typeof value === 'boolean') &&
        !(value === 'true' || value === 'false')
      )
        throw new Error('Value should be boolean');
      return true;
    })
];

const gatewayGroup = check => [
  check('gatewayGroup')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_clients && value && !Array.isArray(value)) {
        throw new Error('Gateway Group should be an array');
      }
      return true;
    })
];

const gateway = check => [
  check('gateway')
    .exists()
    .custom((value, { req }) => {
      if (!req.body.all_clients && value && !Array.isArray(value)) {
        throw new Error('Gateway should be an array');
      }
      return true;
    })
];

const value = check => [
  check('value')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Value is required.')
    .custom((value, { req }) => {
      if (value !== 'true' && value !== 'false')
        throw new Error('Value should be boolean');
      return true;
    })
];

const accessRestriction = check => [
  check('access_restriction').custom((value, { req }) => {
    if (value && !(req.body.access_start_time && req.body.access_end_time)) {
      throw new Error('Access start time and end time is required');
    } else if (
      value &&
      req.body.access_start_time &&
      req.body.access_end_time
    ) {
      const regex = /^([0-1][0-9]|2[0-3]):([0-5][0-9])$/i;
      if (!regex.test(req.body.access_start_time)) {
        throw new Error(
          'Invalid value for start, value must be between, 00:00 - 23:59'
        );
      }

      if (!regex.test(req.body.access_end_time)) {
        throw new Error(
          'Invalid value for end time, value must be between, 00:00 - 23:59'
        );
      }
    }

    if (value && !req.body.days) {
      throw new Error('Week days are required');
    } else if (value && req.body.days) {
      if (!Array.isArray(req.body.days))
        throw new Error('Week days should be an array');

      const max = Math.max.apply(null, req.body.days);
      const min = Math.min.apply(null, req.body.days);

      if (isNaN(max) || max > 6) {
        throw new Error(
          `Maximum value for days less than or equal to 6, got ${max}`
        );
      }

      if (isNaN(min) || min < 0) {
        throw new Error(
          `Minimum value for days should be greater than or equal to 0, got ${min}`
        );
      }
    }

    if (value && !req.body.months) {
      throw new Error('Months are required');
    } else if (value && req.body.months) {
      if (!Array.isArray(req.body.months))
        throw new Error('Months should be an array');

      const max = Math.max.apply(null, req.body.months);
      const min = Math.min.apply(null, req.body.months);

      if (isNaN(max) || max > 12) {
        throw new Error(
          `Maximum value for months should be less than or equal to 12, got ${max}`
        );
      }

      if (isNaN(min) || min < 1) {
        throw new Error(
          `Minimum value for months should be greater than or equal to 1, got ${min}`
        );
      }
    }

    return true;
  })
];

const os = check => [
  check('os')
    .exists()
    .not()
    .isEmpty()
    .withMessage('OS is required.')
];

const antivirus = check => [
  check('antivirus').custom((value, { req }) => {
    if (req.body.os && (!value || !Array.isArray(value) || value.length === 0)) {
      throw new Error('Antivirus is required as a non-empty array.');
    }
    if (value && !Array.isArray(value)) {
      throw new Error('Antivirus must be an array.');
    }
    return true;
  })
];


const osVersion = check => [
  check('os_version').custom((value, { req }) => {
    if (req.body.os && !(value || '').trim())
      throw new Error('OS version is required.');

    if (value && typeof value !== 'string') {
      throw new Error('Value must be string.');
    }
    return true;
  })
];

const isEPPClient = check => [
  check('is_epp_client')
    .exists()
    .not()
    .isEmpty()
    .withMessage('OS is required.')
    .custom((value, { req }) => {
      if (
        value &&
        !(typeof value === 'boolean') &&
        !(value === 'true' || value === 'false')
      )
        throw new Error('Value should be boolean');
      return true;
    })
];

const isBlocked = check => [
  check('is_blocked')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Blocked value is required.')
    .custom((value, { req }) => {
      if (
        value &&
        !(typeof value === 'boolean') &&
        !(value === 'true' || value === 'false')
      )
        throw new Error('Value should be boolean');
      return true;
    })
];

// Validator for data volume policy

const clientUploadRate = check => [
  check('client_upload_rate')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Tx Threshold is required.')
];

const clientDownloadRate = check => [
  check('client_download_rate')
    .exists()
    .not()
    .isEmpty()
    .withMessage('Rx Threshold is required.')
];

module.exports = {
  createPolicy,
  getPolicy,
  updatePolicy,
  deletePolicy,
  setEnableValue,
  createDevicePolicy,
  updateDevicePolicy,
  getDevicePolicy,
  deleteDevicePolicy,
  createSoftwareProcessPolicy,
  getSoftwareProcessPolicy,
  updateSoftwareProcessPolicy,
  deleteSoftwareProcessPolicy,
  createSoftwarePolicy,
  getSoftwarePolicy,
  updateSoftwarePolicy,
  deleteSoftwarePolicy,
  createOsPolicy,
  updateOsPolicy,
  getOsPolicy,
  deleteOsPolicy,
  createBridge,
  updateBridge,
  getAllOs,
  getRoutingPolicy,
  createRoutingPolicy,
  updateRoutingPolicy,
  deleteRoutingPolicy,
  createDataVolumePolicy,
  updateDataVolumePolicy,
  getDataVolumePolicy,
  deleteDataVolumePolicy,
  createB2BBridge,
  createRBIBridge
};
