import { NgModule } from '@angular/core';
import { RouterModule, Routes } from '@angular/router';
import { RolesResolve } from './users/roles.resolve';
import { AuthGuard } from './guard/auth.guard';
import { RecoverPasswordGuard } from './guard/recover-password.guard';
import { CheckToken } from './auth-guard.service';
import { WorkloadDiscoveryModule } from './workload/workload-discovery/workload-discovery.module';
import { CreateSaasBridgeModule } from './saas-bridge/create-saas-bridge/create-saas-bridge.module';
import { UserGroupsComponent } from './users/user-groups/user-groups.component';
import { ChangePasswordGuard } from './guard/change-password.guard';
import { SecrityQuestionsComponent } from './security-questions/security-questions.component';
import { ChangeQuestionGuard } from './guard/change-question.guard';
import { RecoverComponent } from './recover-password/recover.component';
import { S2SGuard } from './guard/s2s.guard';
import { FreshLogin } from './guard/fresh-login.guard';
import { RoleGuard } from './role-guard.service';
import { AdditionalPermissionsGuard } from './guard/additional-permissions.guard';

const routes: Routes = [
  { path: '', redirectTo: '/dashboard', pathMatch: 'full' },
  {
    path: 'dashboard',
    canActivate: [AuthGuard, RoleGuard, FreshLogin],
    loadChildren: () =>
      import('./dashboard/dashboard.module').then(m => m.DashboardModule),
    data: { roleUrl: 'Dashboard' }
  },
  {
    path: 'admin',
    loadChildren: () =>
      import('./users/user-list/user-list.module').then(m => m.UserListModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Admin Users' }
  },
  {
    path: 'organizations',
    loadChildren: () =>
      import('./organization-list-view/organization-list-view.module').then(
        m => m.OrganizationListViewModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Organizations' }
  },
  {
    path: 'create-organization',
    loadChildren: () =>
      import('./create-organization/create-organization.module').then(
        m => m.CreateOrganizationModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Organizations' }
  },
  {
    path: 'create-organization/:id',
    loadChildren: () =>
      import('./create-organization/create-organization.module').then(
        m => m.CreateOrganizationModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Organizations' }
  },
  {
    path: 'view-organization/:id',
    loadChildren: () =>
      import('./create-organization/create-organization.module').then(
        m => m.CreateOrganizationModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Organizations' }
  },
  {
    path: 'users',
    loadChildren: () =>
      import('./local-users-list/local-users-list.module').then(
        m => m.LocalUsersListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Users' }
  },
  {
    path: 'view-user/:id',
    loadChildren: () =>
      import('./create-local-user/create-local-user.module').then(
        m => m.CreateLocalUserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Users' }
  },
  {
    path: 'create-user',
    loadChildren: () =>
      import('./create-local-user/create-local-user.module').then(
        m => m.CreateLocalUserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Users' }
  },
  {
    path: 'workloads',
    loadChildren: () =>
      import('./workload/workload-list/workload-list.module').then(
        m => m.WorkloadListModule
      ),
    canActivate: [AuthGuard, RoleGuard, S2SGuard],
    data: { roleUrl: 'Workloads' }
  },
  {
    path: 'workload-devices',
    loadChildren: () =>
      import('./workload/workload-devices/workload-devices.module').then(
        m => m.WorkloadDevicesModule
      ),
    canActivate: [AuthGuard, RoleGuard, S2SGuard],
    data: { roleUrl: 'Workload Devices' }
  },
  {
    path: 'view-workload-device/:id',
    loadChildren: () =>
      import('./workload/create-workload-devices/create-workload-devices.module').then(
        m => m.CreateWorkloadDevicesModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Workload Devices' }
  },
  {
    path: 'micro-segments',
    loadChildren: () =>
      import('./workload/microsegments/microsegments.module').then(
        m => m.MicrosegmentsModule
      ),
    canActivate: [AuthGuard, S2SGuard],
    data: { roleUrl: 'Microsegments' }
  },
  {
    path: 'microsegments/:id',
    loadChildren: () =>
      import('./create-microsegment/microsegment.module').then(
        m => m.MicrosegmentModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Workloads Management' }
  },
  {
    path: 'microsegments/:id/:deviceId/:microsegmentId',
    loadChildren: () =>
      import('./create-microsegment/microsegment.module').then(
        m => m.MicrosegmentModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Workloads Management' }
  },
  {
    path: 'workloads/:id',
    loadChildren: () =>
      import('./workload/create-workload/create-workload.module').then(
        m => m.CreateWorkloadModule
      ),
    canActivate: [AuthGuard, RoleGuard, S2SGuard],
    data: { roleUrl: 'Workloads' }
  },
  {
    path: 'workload/:id/discovery',
    loadChildren: () => {
      return WorkloadDiscoveryModule;
    },
    canActivate: [AuthGuard, RoleGuard, S2SGuard],
    data: { roleUrl: 'Workloads' }
  },
  {
    path: 'create-workload',
    loadChildren: () =>
      import('./workload/create-workload/create-workload.module').then(
        m => m.CreateWorkloadModule
      ),
    canActivate: [AuthGuard, RoleGuard, S2SGuard],
    data: { roleUrl: 'Workloads Management' }
  },
  {
    path: 'create-device',
    loadChildren: () =>
      import('./create-device/create-device.module').then(
        m => m.CreateDeviceModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Devices' }
  },
  {
    path: 'create-software',
    loadChildren: () =>
      import('./create-software/create-software.module').then(
        m => m.CreateSoftwareModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Master List' }
  },
  {
    path: 'create-software-process-binding',
    loadChildren: () =>
      import(
        './create-software-process-binding/create-software-process-binding.module'
      ).then(m => m.CreateSoftwareProcessBindingModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Process Binding' }
  },
  {
    path: 'device/:id',
    loadChildren: () =>
      import('./create-device/create-device.module').then(
        m => m.CreateDeviceModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Devices' }
  },
  {
    path: 'software/:id',
    loadChildren: () =>
      import('./create-software/create-software.module').then(
        m => m.CreateSoftwareModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Master List' }
  },
  {
    path: 'software-process-binding/:id',
    loadChildren: () =>
      import(
        './create-software-process-binding/create-software-process-binding.module'
      ).then(m => m.CreateSoftwareProcessBindingModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Master List' }
  },
  {
    path: 'view-device/:id',
    loadChildren: () =>
      import('./create-device/create-device.module').then(
        m => m.CreateDeviceModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Devices' }
  },
  {
    path: 'user/:id',
    loadChildren: () =>
      import('./create-local-user/create-local-user.module').then(
        m => m.CreateLocalUserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Users' }
  },
  {
    path: 'assign',
    loadChildren: () =>
      import('./assign-service/assign-list/assign-list.module').then(
        m => m.AssignListModule
      ),
    canActivate: [AuthGuard]
  },

  {
    path: 'ldaps',
    loadChildren: () =>
      import('./ldap/ldapuser-list/ldapuser-list.module').then(
        m => m.LdapuserListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'LDAP Configuration' }
  },
  {
    path: 'services',
    loadChildren: () =>
      import('./manage-service/manage-list/manage-list.module').then(
        m => m.ManageListModule
      ),
    canActivate: [AuthGuard, RoleGuard, FreshLogin],
    data: { roleUrl: 'Services' }
  },
  {
    path: 'view-service/:id',
    loadChildren: () =>
      import(
        './manage-service/manage-list/view-detail-service/view-detail-service.module'
      ).then(m => m.ViewDeatilServiceModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Services' }
  },
  {
    path: 'create-admin',
    loadChildren: () =>
      import('./users/create-user/create-user.module').then(
        m => m.CreateUserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Admin Users' },
    resolve: {
      roles: RolesResolve
    }
  },
  {
    path: 'create-ldapuser',
    loadChildren: () =>
      import('./ldap/creat-ldapuser/create-ldapuser.module').then(
        m => m.CreateLdapuserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'LDAP Configuration' }
  },
  {
    path: 'create-gateway',
    loadChildren: () =>
      import('./gateway/create-gateway/create-gateway.module').then(
        m => m.CreateGatewayModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways' }
  },
  {
    path: 'gateways',
    loadChildren: () =>
      import('./gateway/gateway-list/gateway-list.module').then(
        m => m.GatewayListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways' }
  },
  {
    path: 'gateway/:id',
    loadChildren: () =>
      import('./gateway/gateway-detail/gateway-detail.module').then(
        m => m.GatewayDetailModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways' }
  },
  {
    path: 'gatewayGroups',
    loadChildren: () =>
      import('./gateway-groups/gateway-group-list/gateway-groups.module').then(
        m => m.GatewayGroupsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways Groups' }
  },
  {
    path: 'gatewayGroups/:id',
    loadChildren: () =>
      import(
        './gateway-groups/gateway-group-detail/gateway-group-detail.module'
      ).then(m => m.GatewayGroupDetailModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways Groups' }
  },
  {
    path: 'create-gateway-group/:id',
    loadChildren: () =>
      import(
        './gateway-groups/create-gatway-group/create-gatway-group.module'
      ).then(m => m.CreateGatwayGroupModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways Groups' }
  },
  {
    path: 'create-gateway-group',
    loadChildren: () =>
      import(
        './gateway-groups/create-gatway-group/create-gatway-group.module'
      ).then(m => m.CreateGatwayGroupModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways Groups' }
  },
  {
    path: 'saas-bridge',
    loadChildren: () =>
      import('./saas-bridge/saas-bridge.module').then(m => m.SaasBridgeModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'SaaS Bridges' }
  },
  {
    path: 'saas-bridge/create',
    loadChildren: () => {
      return CreateSaasBridgeModule;
    },
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'SaaS Bridges' }
  },
  {
    path: 'saas-bridge/edit/:id',
    loadChildren: () => {
      return CreateSaasBridgeModule;
    },
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'SaaS Bridges' }
  },
  {
    path: 'server-bridge',
    loadChildren: () => import('./iot/iot.module').then(m => m.IotModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Server Bridges' }
  },
  {
    path: 'rdi-bridge',
    loadChildren: () => import('./iot/iot.module').then(m => m.IotModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RDI Bridges' }
  },
  {
    path: 'rbi-bridge',
    loadChildren: () => import('./iot/iot.module').then(m => m.IotModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RBI Bridges' }
  },
  {
    path: 'create/server-bridge',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Server Bridges' }
  },
  {
    path: 'create/iot-bridge',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (ARM)' }
  },
  {
    path: 'create/client-bridge',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (x86)' }
  },
  {
    path: 'client-bridge',
    loadChildren: () => import('./iot/iot.module').then(m => m.IotModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (x86)' }
  },
  {
    path: 'create/rdi-bridge',
    loadChildren: () =>
      import('./create-b2b-bridge/create-b2b-bridge.module').then(
        m => m.CreateB2bBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RDI Bridges' }
  },
  {
    path: 'create/rbi-bridge',
    loadChildren: () =>
      import('./create-rbi-bridge/create-rbi-bridge.module').then(
        m => m.CreateRbiBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RBI Bridges' }
  },
  {
    path: 'iot-bridge',
    loadChildren: () => import('./iot/iot.module').then(m => m.IotModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (ARM)' }
  },
  {
    path: 'edit/server-bridge/:id',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Server Bridges' }
  },
  {
    path: 'edit/rdi-bridge/:id',
    loadChildren: () =>
      import('./create-b2b-bridge/create-b2b-bridge.module').then(
        m => m.CreateB2bBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RDI Bridges' }
  },
  {
    path: 'edit/rbi-bridge/:id',
    loadChildren: () =>
      import('./create-rbi-bridge/create-rbi-bridge.module').then(
        m => m.CreateRbiBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'RBI Bridges' }
  },
  {
    path: 'edit/client-bridge/:id',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (x86)' }
  },
  {
    path: 'edit/iot-bridge/:id',
    loadChildren: () =>
      import('./create-server-bridge/create-server-bridge.module').then(
        m => m.CreateServerBridgeModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Bridge (ARM)' }
  },
  {
    path: 'gateways/:id',
    loadChildren: () =>
      import('./gateway/create-gateway/create-gateway.module').then(
        m => m.CreateGatewayModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Gateways' }
  },
  {
    path: 'controllers',
    loadChildren: () =>
      import('./controller/controller-list/controller-list.module').then(
        m => m.ControllerListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Controllers' }
  },
  {
    path: 'serviceGroups',
    loadChildren: () =>
      import('./policy/policy.module').then(m => m.PolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Service Groups' }
  },
  {
    path: 'serviceGroups/:id',
    loadChildren: () =>
      import('./policy/service-group-detail/service-group-detail.module').then(
        m => m.ServiceGroupDetailModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Service Groups' }
  },
  {
    path: 'create-service-group/:id',
    loadChildren: () =>
      import('./create-policy/create-policy.module').then(
        m => m.CreatePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Service Groups' }
  },
  {
    path: 'create-service-group',
    loadChildren: () =>
      import('./create-policy/create-policy.module').then(
        m => m.CreatePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Service Groups' }
  },
  {
    path: 'create-controller',
    loadChildren: () =>
      import('./controller/create-controller/create-controller.module').then(
        m => m.CreateControllerModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Controllers' }
  },
  {
    path: 'access-policy',
    loadChildren: () =>
      import('./access-policy/access-policy.module').then(
        m => m.AccessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Entitlements' }
  },
  {
    path: 'device-access-policy',
    loadChildren: () =>
      import('./device-access-policy/device-access-policy.module').then(
        m => m.DeviceAccessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Device Entitlements' }
  },
  {
    path: 'user-agent-access-policy',
    loadChildren: () =>
      import('./user-agent-access-policy/user-agent-access-policy.module').then(
        m => m.UserAgentAccessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'User Entitlements' }
  },
  {
    path: 'create-access-policy/:policyType',
    loadChildren: () =>
      import('./create-access-policy/create-access-policy.module').then(
        m => m.CreateAccessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'User Entitlements' }
  },
  {
    path: 'accessPolicy/:policyType/:id',
    loadChildren: () =>
      import('./create-access-policy/create-access-policy.module').then(
        m => m.CreateAccessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'User Entitlements' }
  },
  {
    path: 'device-policy',
    loadChildren: () =>
      import('./device-policy/device-policy.module').then(
        m => m.DevicePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Security Policy' }
  },
  {
    path: 'software-process-policy',
    loadChildren: () =>
      import('./software-process-policy/software-process-policy.module').then(
        m => m.SoftwareProcessPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Process Policy' }
  },
  {
    path: 'software-policy',
    loadChildren: () =>
      import('./software-policy/software-policy.module').then(
        m => m.SoftwarePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Policy' }
  },
  {
    path: 'create-device-policy',
    loadChildren: () =>
      import('./create-device-policy/create-device-policy.module').then(
        m => m.CreateDevicePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Security Policy' }
  },
  {
    path: 'create-software-process-policy',
    loadChildren: () =>
      import(
        './create-software-process-policy/create-software-process-policy.module'
      ).then(m => m.CreateSoftwareProcessPolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Process Policy' }
  },
  {
    path: 'create-software-policy',
    loadChildren: () =>
      import('./create-software-policy/create-software-policy.module').then(
        m => m.CreateSoftwarePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Policy' }
  },
  {
    path: 'os-policy',
    loadChildren: () =>
      import('./os-policy/os-policy.module').then(m => m.OsPolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'OS Policy' }
  },
  {
    path: 'create-os-policy',
    loadChildren: () =>
      import('./create-os-policy/create-os-policy.module').then(
        m => m.CreateOsPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'OS Policy' }
  },
  {
    path: 'osPolicy/:id',
    loadChildren: () =>
      import('./create-os-policy/create-os-policy.module').then(
        m => m.CreateOsPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'OS Policy' }
  },
  {
    path: 'devicePolicy/:id',
    loadChildren: () =>
      import('./create-device-policy/create-device-policy.module').then(
        m => m.CreateDevicePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Security Policy' }
  },
  {
    path: 'softwarePolicy/:id',
    loadChildren: () =>
      import('./create-software-policy/create-software-policy.module').then(
        m => m.CreateSoftwarePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Policy' }
  },
  {
    path: 'softwareProcessPolicy/:id',
    loadChildren: () =>
      import(
        './create-software-process-policy/create-software-process-policy.module'
      ).then(m => m.CreateSoftwareProcessPolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Process Policy' }
  },
  {
    path: 'routing-policy',
    loadChildren: () =>
      import('./routing-policy/routing-policy.module').then(
        m => m.RoutingPolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Routing Policy' }
  },
  {
    path: 'routing-policy/create',
    loadChildren: () =>
      import(
        './routing-policy/create-routing-policy/create-routing-policy.module'
      ).then(m => m.CreateRoutingPolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Routing Policy' }
  },
  {
    path: 'routing-policy/edit/:id',
    loadChildren: () =>
      import(
        './routing-policy/create-routing-policy/create-routing-policy.module'
      ).then(m => m.CreateRoutingPolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Routing Policy' }
  },
  {
    path: 'controllers/:id',
    loadChildren: () =>
      import('./controller/create-controller/create-controller.module').then(
        m => m.CreateControllerModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Controllers' }
  },
  {
    path: 'clients/:type/:id',
    loadChildren: () =>
      import('./client/create-client/create-client.module').then(
        m => m.CreateClientModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Client Management' }
  },
  {
    path: 'clients/:type/:id/:clone',
    loadChildren: () =>
      import('./client/create-client/create-client.module').then(
        m => m.CreateClientModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Client Management' }
  },
  {
    path: 'clients/:type',
    loadChildren: () =>
      import('./client/client-list/client-list.module').then(
        m => m.ClientListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Client Management' }
  },
  {
    path: 'create-client/:type',
    loadChildren: () =>
      import('./client/create-client/create-client.module').then(
        m => m.CreateClientModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Client Management' }
  },
  {
    path: 'create-manage',
    loadChildren: () =>
      import('./manage-service/create-manage/create-manage.module').then(
        m => m.CreateManageModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Services' }
  },
  {
    path: 'admin/:id',
    loadChildren: () =>
      import('./users/create-user/create-user.module').then(
        m => m.CreateUserModule
      ),
    canActivate: [AuthGuard],
    resolve: {
      roles: RolesResolve
    }
  },
  {
    path: 'ldap/:id',
    loadChildren: () =>
      import('./ldap/creat-ldapuser/create-ldapuser.module').then(
        m => m.CreateLdapuserModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'LDAP Configuration' }
  },
  {
    path: 'services/:id',
    loadChildren: () =>
      import('./manage-service/create-manage/create-manage.module').then(
        m => m.CreateManageModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Services' }
  },
  {
    path: 'updates',
    loadChildren: () =>
      import('./updates/updates-list/updates-list.module').then(
        m => m.UpdatesListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Update' }
  },
  {
    path: 'create-update',
    loadChildren: () =>
      import('./updates/create-update/create-update.module').then(
        m => m.CreateUpdateModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Update' }
  },
  {
    path: 'updates/:id',
    loadChildren: () =>
      import('./updates/create-update/create-update.module').then(
        m => m.CreateUpdateModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Update' }
  },
  {
    path: 'settings/:whichSetting',
    loadChildren: () =>
      import('./settings/settings.module').then(m => m.SettingsModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Settings' }
  },
  {
    path: 'license',
    loadChildren: () =>
      import('./license/license.module').then(m => m.LicenseModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'License Management' }
  },
  {
    path: 'backups',
    loadChildren: () =>
      import('./backups/backups.module').then(m => m.BackupsModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'System Backup' }
  },
  {
    path: 'restore',
    loadChildren: () =>
      import('./restore/restore.module').then(m => m.RestoreModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'System Backup' }
  },
  {
    path: 'clients-access-logs',
    loadChildren: () =>
      import('./client/client-access-log/client-access-log.module').then(
        m => m.ClientAccessLogModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Client' }
  },
  {
    path: 'admin-access-logs',
    loadChildren: () =>
      import('./admin-access-log/admin-access-log.module').then(
        m => m.AdminAccessLogModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Admin' }
  },
  {
    path: 'devices',
    loadChildren: () =>
      import('./devices-list/devices-list.module').then(
        m => m.DevicesListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Devices' }
  },
  {
    path: 'softwares',
    loadChildren: () =>
      import('./softwares-list/softwares-list.module').then(
        m => m.SoftwaresListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Software Master List' }
  },
  {
    path: 'idp',
    loadChildren: () =>
      import('./idp-management/idp-list/idp-list.module').then(
        m => m.IdpListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'IDP Management' }
  },
  {
    path: 'create-idp',
    loadChildren: () =>
      import('./idp-management/create-idp/create-idp.module').then(
        m => m.CreateIdpModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'IDP Management' }
  },
  {
    path: 'idps/:id',
    loadChildren: () =>
      import('./idp-management/create-idp/create-idp.module').then(
        m => m.CreateIdpModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'IDP Management' }
  },
  {
    path: 'login',
    loadChildren: () => import('./login/login.module').then(m => m.LoginModule),
    canActivate: [CheckToken]
  },
  {
    path: 'gauth/:token',
    loadChildren: () => import('./gauth/gauth.module').then(m => m.GauthModule)
  },
  {
    path: 'deviceid/:token',
    loadChildren: () =>
      import('./deviceid/deviceid.module').then(m => m.DeviceidModule)
  },
  {
    path: 'onboard/:token',
    loadChildren: () =>
      import('./onboard/onboard.module').then(m => m.OnboardModule)
  },
  {
    path: 'fresh-login',
    loadChildren: () =>
      import('./fresh-login/fresh-login.module').then(m => m.FreshLoginModule),
    canActivate: [AuthGuard]
  },
  {
    path: 'verification/:token',
    loadChildren: () =>
      import('./verification/verification.module').then(
        m => m.VerificationModule
      )
  },
  {
    path: 'localUserResetPassword/:token',
    loadChildren: () =>
      import(
        './local-user-forgot-password/local-user-forgot-password.module'
      ).then(m => m.LocalUserForgotPasswordModule)
  },
  {
    path: 'deviceid/:token/:devid',
    loadChildren: () =>
      import('./deviceid/deviceid.module').then(m => m.DeviceidModule)
  },
  {
    path: 'notfound',
    loadChildren: () =>
      import('./notfound/notfound.module').then(m => m.NotfoundModule)
  },
  {
    path: 'mobile',
    loadChildren: () =>
      import('./mobile-client/mobile-client.module').then(
        m => m.MobileClientModule
      )
  },
  {
    path: 'forgot-password',
    loadChildren: () =>
      import('./forgot-password/forgot-password.module').then(
        m => m.ForgotPasswordModule
      )
  },
  {
    path: 'recover-password/:token',
    loadChildren: () =>
      import('./recover-password/recover.module').then(m => m.RecoverModule),
    canActivate: [RecoverPasswordGuard]
  },
  {
    path: 'change-creds',
    canActivate: [ChangePasswordGuard],
    loadChildren: () =>
      import('./recover-password/recover.module').then(m => m.RecoverModule)
  },
  {
    path: 'groups',
    loadChildren: () =>
      import('./group/group-list/group-list.module').then(
        m => m.GroupListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Groups' }
  },
  {
    path: 'group/:id',
    loadChildren: () =>
      import('./group/group-detail/group-detail.module').then(
        m => m.GroupDetailModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Groups' }
  },
  {
    path: 'create-group',
    loadChildren: () =>
      import('./group/create-group/create-group.module').then(
        m => m.CreateGroupModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Groups' }
  },
  {
    path: 'create-group/:id',
    loadChildren: () =>
      import('./group/create-group/create-group.module').then(
        m => m.CreateGroupModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Groups' }
  },
  {
    path: 'reports',
    loadChildren: () =>
      import('./reportings/reportings.module').then(m => m.ReportingsModule),
    canActivate: [AuthGuard, RoleGuard, FreshLogin],
    data: { roleUrl: 'All Reports' }
  },
  {
    path: 'users_reports',
    loadChildren: () =>
      import('./users-reports/users-reports.module').then(
        m => m.UsersReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Users Reports' }
  },
  {
    path: 'devices_reports',
    loadChildren: () =>
      import('./devices-reports/devices-reports.module').then(
        m => m.DevicesReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Devices Reports' }
  },
  {
    path: 'services_reports',
    loadChildren: () =>
      import('./services-reports/services-reports.module').then(
        m => m.ServicesReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Services Reports' }
  },
  {
    path: 'policy_reports',
    loadChildren: () =>
      import('./policy-reports/policy-reports.module').then(
        m => m.PolicyReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Policy Reports' }
  },
  {
    path: 'infrastructure_reports',
    loadChildren: () =>
      import('./infrastructure-reports/infrastructure-reports.module').then(
        m => m.InfrastructureReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Infrastructure Reports' }
  },
  {
    path: 'clients_reports',
    loadChildren: () =>
      import('./clients-reports/clients-reports.module').then(
        m => m.ClientsReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Clients Reports' }
  },
  {
    path: 'onboarding_reports',
    loadChildren: () =>
      import('./onboarding-reports/onboarding-reports.module').then(
        m => m.OnboardingReportsModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Onboarding Reports' }
  },
  {
    path: 'logout',
    loadChildren: () =>
      import('./logout/logout.module').then(m => m.LogoutModule),
    canActivate: [AuthGuard]
  },
  {
    path: 'alerts',
    canActivate: [AuthGuard, RoleGuard, FreshLogin],
    data: { roleUrl: 'Alerts' },
    loadChildren: () =>
      import('./alert-list/alert-list.module').then(m => m.AlertListModule)
  },
  {
    path: 'forbidden_softwares',
    loadChildren: () =>
      import('./forbidden-software-list/forbidden-software-list.module').then(
        m => m.ForbiddenSoftwareListModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Unauthorized Software' }
  },
  {
    path: 'software_process_binding',
    loadChildren: () =>
      import('./software-process-binding/software-process-binding.module').then(
        m => m.SoftwareProcessBindingModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Process Binding' }
  },
  {
    path: 'data-volume-policy',
    loadChildren: () =>
      import('./data-volume-policy/data-volume-policy.module').then(
        m => m.DataVolumePolicyModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Data Volume Policy' }
  },
  {
    path: 'create-data-volume-policy',
    loadChildren: () =>
      import(
        './create-data-volume-policy/create-data-volume-policy.module'
      ).then(m => m.CreateDataVolumePolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Data Volume Policy' }
  },
  {
    path: 'edit-data-volume-policy/:id',
    loadChildren: () =>
      import(
        './create-data-volume-policy/create-data-volume-policy.module'
      ).then(m => m.CreateDataVolumePolicyModule),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Data Volume Policy' }
  },
  {
    path: 'manage-microsegment/:id',
    loadChildren: () =>
      import('./microsegmentation/microsegmentation.module').then(
        m => m.MicrosegmentationModule
      ),
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Workloads' }
  },
  {
    path: 'customUserGroups',
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'User Groups' },
    loadChildren: () =>
      import('./users/user-groups/user-groups.module').then(
        m => m.UserGroupsModule
      )
  },
  {
    path: 'customUserGroups/:id',
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'User Groups' },
    loadChildren: () =>
      import('./users/user-groups-edit/user-groups-edit.module').then(
        m => m.UserGroupsEditModule
      )
  },
  {
    path: 'security-questions',
    canActivate: [ChangeQuestionGuard],
    loadChildren: () =>
      import('./security-questions/security-questions.module').then(
        m => m.SecrityQuestionsModule
      )
  },
  {
    path: 'create_roles_and_permissions',
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './roles-and-permission/create-roles-and-permission/create-roles-and-permission.module'
      ).then(m => m.CreateRolesAndPermissionModule)
  },
  {
    path: 'view_roles_and_permissions',
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './roles-and-permission/view-roles-and-permission/view-roles-and-permission.module'
      ).then(m => m.ViewRolesAndPermissionModule)
  },
  {
    path: 'edit_roles_and_permissions/:id',
    canActivate: [AuthGuard, RoleGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './roles-and-permission/edit-roles-and-permission/edit-roles-and-permission.module'
      ).then(m => m.EditRolesAndPermissionModule)
  },
  {
    path: 'view_additional_permissions',
    canActivate: [AuthGuard, RoleGuard, AdditionalPermissionsGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './additional-permissions/view-additional-permissions/view-additional-permissions.module'
      ).then(m => m.ViewAdditionalPermissionsModule)
  },
  {
    path: 'additional_permissions_details/:id',
    canActivate: [AuthGuard, RoleGuard, AdditionalPermissionsGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './additional-permissions/additional-permissions-details/additional-permissions-details.module'
      ).then(m => m.AdditionalPermissionsDetailsModule)
  },
  {
    path: 'create_additional_permissions',
    canActivate: [AuthGuard, RoleGuard, AdditionalPermissionsGuard],
    data: { roleUrl: 'Roles And Permissions' },
    loadChildren: () =>
      import(
        './additional-permissions/create-additional-permissions/create-additional-permissions.module'
      ).then(m => m.CreateAdditionalPermissionsModule)
  },
  {
    path: '**',
    redirectTo: '/dashboard'
  }
];
@NgModule({
  imports: [RouterModule.forRoot(routes)],
  exports: [RouterModule]
})
export class AppRoutingModule {}
