import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { ClientService } from '../client.service';
import { PreserveSearch } from '../../models/preserve-search';
import { Pagination } from '../../models/pagination';
import { timer } from 'rxjs';
import { ClientAccessLog } from '../../models/client_access_log';
import { Subscription } from 'rxjs';
import { Popup } from '../../Popup';
import { AuthenticationService } from '../../login/authentication.service';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import * as moment from 'moment';
import { first, tap } from 'rxjs/operators';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { ClientAccessStatusEnum } from '../../shared/emuns/client.enum';
import { KeyboardKeyCodeEnum } from '../../shared/emuns/global.enum';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { RoleEnum } from '../../shared/emuns/global.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { VersionService } from '../../../app/version/version.service';
import { Title } from '@angular/platform-browser';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-client-access-log',
  templateUrl: './client-access-log.component.html',
  styleUrls: ['./client-access-log.component.css']
})
export class ClientAccessLogComponent implements OnInit {
  public alert: GlobalMessageClass;
  public malert: GlobalMessageClass;
  purgeAlert: Popup = new Popup();
  private timerSubscription: Subscription;
  public search: PreserveSearch = new PreserveSearch();
  public pagination: Pagination = new Pagination();
  public accessLogs: any = {};
  public statusColor: Array<string> = [
    'label label-danger',
    'label label-warning',
    'label label-success bg-success',
    'label label-success',
    'label label-info'
  ];
  public logTypes: Array<Object> = [
    { index: 'null', name: 'ANY' },
    { index: 0, name: 'Failed Login' },
    { index: 1, name: 'Forcefully Closed' },
    { index: 2, name: 'Gracefully Closed' },
    { index: 3, name: 'Running' }
  ];
  public log_search_key: String = null;
  public log_type: any = 'null';
  public clients: Array<any>;
  public client_id = 'null';
  public client_os: string = 'null';
  public start_date: Object = null;
  public end_date: Object = null;
  public showFilter: boolean = false;
  purgeAllowed: boolean = false;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  public userrole: string = '';
  Role = RoleEnum;
  oss = [
    { value: 'null', display: 'ANY' },
    { value: DeviceOSEnum.Ubuntu, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.IOS, display: 'iOS' },
    { value: DeviceOSEnum.Android, display: 'Android' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];
  clientCurrentPage = 1;
  totalClientsPages;
  statusValues = [];
  quickFiltersStatus = [
    {
      isChecked: false,
      title: 'Failed Login',
      values: ClientAccessStatusEnum.FailedLogin
    },
    {
      isChecked: false,
      title: 'Forcefully Closed',
      values: ClientAccessStatusEnum.ForcefullyClosed
    },
    {
      isChecked: false,
      title: 'Gracefully Closed',
      values: ClientAccessStatusEnum.GracefullyClosed
    },
    {
      isChecked: false,
      title: 'Running',
      values: ClientAccessStatusEnum.Running
    }
  ];
  osValues = [];
  quickFiltersOs = [
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Desktop',
    //   value: DeviceOSEnum.Ubuntu
    // },
    // {
    //   isChecked: false,
    //   title: 'Ubuntu 16 Server',
    //   value: DeviceOSEnum.UbuntuServer
    // },
    {
      isChecked: false,
      title: 'Ubuntu Desktop',
      value: DeviceOSEnum.Ubuntu18
    },
    {
      isChecked: false,
      title: 'Ubuntu Server',
      value: DeviceOSEnum.UbuntuServer18
    },
    {
      isChecked: false,
      title: 'Windows',
      value: DeviceOSEnum.Windows
    },
    {
      isChecked: false,
      title: 'Windows Server',
      value: DeviceOSEnum.WindowsServer
    },
    {
      isChecked: false,
      title: 'Android',
      value: DeviceOSEnum.Android
    },
    {
      isChecked: false,
      title: 'iOS',
      value: DeviceOSEnum.IOS
    },
    {
      isChecked: false,
      title: 'macOS',
      value: DeviceOSEnum.Macintosh
    },
    {
      isChecked: false,
      title: 'CentOS Server',
      value: DeviceOSEnum.CentosServer
    }
  ];
  quickFiltersShowStatus = false;
  quickFiltersShowOs = false;

  loading: Boolean = true;
  dateRangePicker: Date[] = [];

  today: Date;

  pageSize = [10, 25, 50, 100, 200];
  sort: string;
  sortDir: string;

  timeZone = '';
  portalVersionInfo: string;
  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  handleModal = false;
  toggleRuleViewStyle;
  serviceDetails: any[] = [];
  isDarkMode : boolean = false;
  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private clientService: ClientService,
    private authenticationService: AuthenticationService,
    private globalMessageService: GlobalMessageService,
    public dialog: MatDialog,
    private versionService: VersionService,
    private titleService: Title,
    private rolesPermissionService: RolesAndPermissionService
  ) {
    this.titleService.setTitle('Access Logs | Client');
    this.today = new Date();

    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' || this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    if (typeof this.route.params['_value']['logtype'] != 'undefined') {
      this.log_type = this.route.params['_value']['logtype'];
    }
    if (typeof this.route.params['_value']['clientid'] != 'undefined') {
      this.client_id = this.route.params['_value']['clientid'];
    }
    if (typeof this.route.params['_value']['startDate'] != 'undefined') {
      var startDate =
        this.route.params['_value']['startDate'] != 'null'
          ? this.route.params['_value']['startDate'].split('-')
          : null;
      if (startDate != null) {
        startDate = {
          year: parseInt(startDate[0]),
          month: parseInt(startDate[1]),
          day: parseInt(startDate[2])
        };
      }
      this.start_date = startDate;
    }
    if (typeof this.route.params['_value']['endDate'] != 'undefined') {
      var endDate =
        this.route.params['_value']['endDate'] != 'null'
          ? this.route.params['_value']['endDate'].split('-')
          : null;
      if (endDate != null) {
        endDate = {
          year: parseInt(endDate[0]),
          month: parseInt(endDate[1]),
          day: parseInt(endDate[2])
        };
      }
      this.end_date = endDate;
    }
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }

    let timeZoneOffset = new Date().getTimezoneOffset();
    this.timeZone =
      'UTC' + (timeZoneOffset > 0 ? '-0' : '+0') + (timeZoneOffset / 60) * -1;
  }

  public showFilters(): void {
    this.log_search_key = null;
    this.showFilter = !this.showFilter;
  }

  submitForm(event) {
    if (event.keyCode == KeyboardKeyCodeEnum.Enter) {
      this.refreshData();
    }
  }

  showPurgePopup() {
    // this.purgeAlert.heading = 'Purge';
    // this.purgeAlert.body = 'Are you sure you want to purge access logs?';
    // this.purgeAlert.action = 'Purge';
    // this.purgeAlert.show = true;
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: 'Are you sure you want to Purge access logs',
      },
    });

    dialogRef.afterClosed().subscribe((result) => {
      if (result){
        this.purgeAction();
      }
    });
  }

  cancelPopup() {
    this.purgeAlert.show = false;
  }

  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    let pergeStatus = localStorage.getItem('purgeAllowed');
    this.getCRUDdata(53);
    if (pergeStatus == 'false') {
      this.purgeAllowed = false;
    } else {
      this.purgeAllowed = true;
    }
    let clientAccessLogSearch = this.search.getSearch('clientAccessLogSearch');
    if (clientAccessLogSearch) {
      this.pagination.currentPage = clientAccessLogSearch.current_page;
    }
    //this.getPage(this.pagination.currentPage);
    // this.getClients();
    this.refreshData();
    this.portalVersionInfo = this.versionService.portalVersion;
  }

  // searchMoreClients() {
  //   this.clientCurrentPage += 1;
  //   if (this.clientCurrentPage <= this.totalClientsPages) {
  //     this.getClients();
  //   } else {
  //     this.clientCurrentPage -= 1;
  //   }
  // }

  // private getClients() {
  //   this.clientService
  //     .getClients(
  //       this.clientCurrentPage,
  //       this.pagination.itemsPerPage,
  //       {},
  //       this.sort,
  //       this.sortDir,
  //       '',
  //       false,
  //       false
  //     )
  //     pipe(tap((res :any) => {
  //       this.loading = false;
  //       this.totalClientsPages = Math.ceil(res.recordsTotal / this.pagination.itemsPerPage);
  //       if (this.clientCurrentPage === 1) {
  //         this.clients = [{ user: 'null', Name: 'ANY' }, ...res.data];
  //       } else {
  //         this.clients = [...this.clients, ...res.data];
  //       }
  //     }))
  //     .subscribe({
  //       next: next => {},
  //       error: err => {
  //         this.loading = false;
  //         if (err.status === 403) {
  //           if (err._body == LicenseErrorEnum.Invalid) {
  //             this.authenticationService.showLicenseOnly(true);
  //             this.router.navigate(['/license']);
  //           } else {
  //             this.alert = {
  //               closed: false,
  //               type: 'danger',
  //               message: 'You are not authorized to access this page.'
  //             };
  //             this.globalMessageService.pushAlert(this.alert);
  //           }
  //         }
  //       },
  //       complete: () => {}
  //     });
  // }

  public refreshData(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    this.getPage(this.pagination.currentPage);
    this.subscribeToData();
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(10000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  public ngOnDestroy(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
  }

  resetSearch() {
    this.log_search_key = null;
    this.log_type = 'null';
    this.client_id = 'null';
    this.client_os = 'null';
    this.start_date = null;
    this.end_date = null;
    this.showFilter = false;
    this.sort = 'id';

    this.getPage(1);
  }

  purgeAction() {
    this.purgeAlert.show = false;
    this.clientService
      .purgeClientAccessLogs()
      .pipe(
        tap((res: any) => {
          if (res.success) {
            if (this.timerSubscription) {
              this.timerSubscription.unsubscribe();
            }
            this.refreshData();
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  getPage(page: number) {
    this.pagination.currentPage = page;
    this.loading = true;

    this.clientService
      .getClientAccessLogs(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        this.log_type,
        this.client_id,
        this.client_os,
        this.start_date,
        this.end_date,
        this.sort,
        this.sortDir,
        this.log_search_key
      )
      .pipe(
        tap((res: any) => {
          /*****************************
        /*  0 => Failed Login
        /*  1 => Exited Abruptly
        /*  2 => Gracefully Closed
        /*  3 => Running
        /*  4 => Failed
        ******************************/
          this.loading = false;
          for (let log of res.data || []) {
            if (!log.is_login_successfull) {
              log.status = ClientAccessStatusEnum.FailedLogin;
              log.event = 'Failed Login';
            } else if (
              !log.is_logout_successfull &&
              log.is_logout_successfull != null
            ) {
              log.status = ClientAccessStatusEnum.ForcefullyClosed;
              log.event = 'Forcefully Closed';
            } else {
              if (
                log.CONNECTION_ATTEMPT_TIME != null &&
                log.DISCONNECTION_TIME != null
              ) {
                log.status = ClientAccessStatusEnum.GracefullyClosed;
                log.event = 'Gracefully Closed';
              } else if (log.DISCONNECTION_TIME == null) {
                log.status = ClientAccessStatusEnum.Running;
                log.event = 'Running';
              } else {
                log.status = ClientAccessStatusEnum.Failed;
                log.event = 'Failed';
              }
            }
            if (
              log.CONNECTION_ATTEMPT_TIME != null &&
              log.DISCONNECTION_TIME != null
            ) {
              let diffInMs: number =
                Date.parse(log.DISCONNECTION_TIME) -
                Date.parse(log.CONNECTION_ATTEMPT_TIME);
              let diffInHours: number = diffInMs / 1000 / 60 / 60;
              let flatHours = Math.floor(diffInHours);
              log.sessionTime = '';
              if (flatHours > 0) {
                log.sessionTime = flatHours + ' Hours ';
              }
              let totalMinutesUsed = flatHours * 60;
              let minutesleft = diffInMs / 1000 / 60 - totalMinutesUsed;
              let flatMinutes = Math.floor(minutesleft);
              if (flatMinutes > 0) {
                log.sessionTime += flatMinutes + ' Minutes ';
              }
              let totalSecondsUsed = flatMinutes * 60;
              let secondsleft =
                diffInMs / 1000 - (totalSecondsUsed + totalMinutesUsed * 60);
              let flatSeconds = Math.floor(secondsleft);
              if (secondsleft > 0) {
                log.sessionTime += secondsleft + ' Seconds ';
              }
            }
          }
          this.accessLogs = res;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        },
        complete: () => {}
      });
  }

  clearfun() {
    this.log_search_key = '';
    this.getPage(this.pagination.currentPage);
  }

  dateHandler() {
    this.refreshData();
  }

  quickFilterStatusHandler() {
    this.quickFiltersShowOs = false;
    this.quickFiltersShowStatus = !this.quickFiltersShowStatus;
  }
  quickFilterOsHandler() {
    this.quickFiltersShowOs = !this.quickFiltersShowOs;
    this.quickFiltersShowStatus = false;
  }
  inputChanged(event) {
    event.stopPropagation();
  }
  applyStatusHandler(event) {
    const logTypeArray = this.statusValues;
    this.quickFiltersStatus.map(status => {
      if (status.isChecked) {
        logTypeArray.push(status.values);
      }
    });
    this.log_type = logTypeArray.length ? logTypeArray.toString() : 'null';
    this.refreshData();
    this.quickFiltersShowStatus = false;
  }

  clearStatusHandler(event) {
    event.stopPropagation();
    this.quickFiltersStatus = this.quickFiltersStatus.map(filter => {
      return {
        ...filter,
        isChecked: false
      };
    });
  }

  applyOsHandler() {
    const osArray = this.osValues;
    // this.quickFiltersOs.map(os => {
    //   if (os.isChecked) {
    //     osArray.push(os.value);
    //   }
    // });
    this.client_os = osArray.length ? osArray.toString() : 'null';
    this.refreshData();
    this.quickFiltersShowOs = false;
  }

  clearOsHandler(event) {
    event.stopPropagation();
    this.quickFiltersOs = this.quickFiltersOs.map(filter => {
      return {
        ...filter,
        isChecked: false
      };
    });
  }

  servicesAccessed(id) {
    this.handleModal = !this.handleModal;
    this.toggleRuleViewStyle = 'block';

    this.clientService.getClientServices(id)
      .subscribe(response => {
        this.serviceDetails = response.serviceDetails;
      }, error => {
        console.error('Error fetching client details:', error);
      });

  }

  handleCancelModal() {
    this.toggleRuleViewStyle = 'none';
  }

  dateRangePickerChange() {
    if (this.dateRangePicker[0]) {
      this.start_date = this.dateRangePicker[0];
    }

    if (this.dateRangePicker[1]) {
      this.end_date = this.dateRangePicker[1];
    }
    this.pagination.currentPage = 1;
    this.refreshData();
  }

  clearDate() {
    this.dateRangePicker = [];
    this.start_date = null;
    this.end_date = null;
    this.refreshData();
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.pagination.currentPage = 1;
      this.getPage(1);
    }, 0);
  }

  sortRecords(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;                                                                                                                       
  
    this.refreshData();
  }

  exportCSV() {
    this.clientService.exportCSV(
      this.log_type,
      this.client_id,
      this.client_os,
      this.start_date,
      this.end_date,
      this.log_search_key,
      this.portalVersionInfo
    );
  }
}
