import { Component, OnInit } from '@angular/core';
import {ClientsReportsService} from './reporting.service';
import { Pagination } from '../models/pagination';
import * as moment from 'moment';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-clients-reports',
  templateUrl: './clients-reports.component.html',
  styleUrls: ['./clients-reports.component.css']
})
export class ClientsReportsComponent implements OnInit {
  today: Date;
  ReportsOfAllClients:any = [];
  servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceSortDir: string = 'asc'
  serviceSort: string;

  summaryOfAllServicesPagination: Pagination = new Pagination();
  reportsOfAllClientsLoading:boolean = true;
  pageSizeSummaryOfServices = [10,25,50,100,200];
  
  constructor(
    private ClientsReportsService:ClientsReportsService,
    private titleService: Title
  ) {
      this.titleService.setTitle('Reports | Clients Reports');
      this.today = new Date();
  }

  ngOnInit() {
    this.summaryOfAllServicesPagination.id='clients_reports';
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
  }
  
  ngAfterViewInit() {
    
  }

  ngOnDestroy() {
  }


  getReportsOfAllServices(pagination: Pagination){
    this.reportsOfAllClientsLoading = true;
    this.ClientsReportsService.
    getServicesReports(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.serviceSortDir,
      this.serviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllClients = summary;
        this.reportsOfAllClientsLoading = false;
      },
      error: err=>{
        this.reportsOfAllClientsLoading = false;
      }
    })
  }


  clearDate (type) {
    if(type === APIDataTypeEnum.Service) {
      this.servicesDaterangepicker = [];
      this.serviceDateChange();
    }
  }


  onDateChange(type) {
    if(type === APIDataTypeEnum.Service) {
      if(!this.servicesDaterangepicker)
      this.servicesDaterangepicker = [];
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
    }
  }

  serviceDateChange(){
    if(!this.servicesDaterangepicker)
    this.servicesDaterangepicker = [];
    this.summaryOfAllServicesPagination.currentPage = 1;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
  }

  pageChangeSummaryOfAllServices(page:any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination);
  }


  sortServiceRecord(term: string){
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;
    this.getReportsOfAllServices(this.summaryOfAllServicesPagination)
  }


}




