import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { map } from 'rxjs/operators';
import { ConfigService } from '../services/config.service';

@Injectable()
export class OsService {
  private apiBaseUrl: string;
  private osUrl = '/osVersions';

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.osUrl = this.apiBaseUrl + this.osUrl;
  }

  getOsVersions() {
    return this.authHttp.get(`${this.osUrl}`).pipe(map(response => response));
  }

  deleteOsVersions(versionId) {
    return this.authHttp
      .delete(`${this.osUrl}/${versionId}`)
      .pipe(map(response => response));
  }

  saveNewOSVersion(deviceOS, newOSVersion) {
    return this.authHttp
      .post(`${this.osUrl}`, { deviceOS, newOSVersion })
      .pipe(map(response => response));
  }
}
