import { Component, OnInit } from '@angular/core';
import { LocalUser } from '../models/LocalUser';
import { LocalUserService } from './local-user.service';
import { DeviceService } from '../devices-list/device.service';
import { OrganizationService } from '../organization-list-view/organization.service';
import { Pagination } from '../models/pagination';
import { Router, ActivatedRoute } from '@angular/router';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { ValidDomainsService } from '../valid-domains.service';
import { Observable, Subject } from 'rxjs';
import { ClientService } from '../client/client.service';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { AuthenticationService } from '../login/authentication.service';
import { LicenseService } from '../license/license.service';
import { BsModalService } from 'ngx-bootstrap/modal';
import {
  debounceTime,
  distinctUntilChanged,
  switchMap,
  tap
} from 'rxjs/operators';
import { ModalComponent } from '../shared/modal/modal.component';
import * as sharedContent from '../shared/shared-content.constant';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { DeviceOSEnum, DeviceTypeEnum } from '../shared/emuns/client.enum';
import { IdentityProvierEnum } from '../shared/emuns/user.enum';
import { LicenseErrorEnum } from '../shared/emuns/license.enum';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { UsersReportsService } from '../users-reports/reporting.service';
import { WorkloadService } from '../workload/workload.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-local-user',
  templateUrl: './create-local-user.component.html',
  styleUrls: ['./create-local-user.component.css']
})
export class CreateLocalUserComponent implements OnInit {
  localUser = new LocalUser();
  pagination = new Pagination();
  groupPagination = new Pagination();
  policyPagination = new Pagination();
  entitlementPagination = new Pagination();
  servicePagination = new Pagination();
  sessionPagination = new Pagination();
  failSessionPagination = new Pagination();
  displayedOrg = null;
  displayedDept = null;
  confirmPasswordField = '';
  editing = false;
  searchWord;
  $searchDevices = new Subject();
  domains;
  reachableDomains;
  domainName;
  reachableDomainName;
  emailValid = true;
  OAuthEmailValid = true;
  loading: Boolean = false;
  devicePolicyLoading: Boolean = false;
  relatedPoliciesCount: number = 0;
  showConfirm = false;
  selectedDevice;
  organizations = [];
  departments = [{ deptId: 0, deptName: 'Name' }];
  organizationId = 0;
  devices: any = [];
  userServices: Array<Object> = [];
  entitlements: Array<Object> = [];
  userServiceGroup: Array<Object> = [];
  groupingdata: Array<Object> = [];
  entitlement: Array<Object> = [];
  sessionData: Array<Object> = [];
  authFailureData: Array<Object> = [];
  workloadData = { data: [] };
  clientId: number;
  deviceToAdd;
  userId;
  availableDevices = [];
  globalDarkening = false;
  alert: GlobalMessageClass;
  showOtpImage: boolean = false;
  OTPQcodeImageData: string = '';
  actionsDropdown: number = 0;
  initDone: boolean = false;
  serverSideDeviceFound: boolean = false;
  mobileDeviceFound: boolean = false;
  viewMode: boolean = false;
  invalidDeviceTypeCombo: boolean = false;
  license: any;
  isMfaDisabled: boolean = false;
  disableSaveButton = false;

  DeviceOS = DeviceOSEnum;
  DeviceType = DeviceTypeEnum;
  IdentityProvier = IdentityProvierEnum;

  searchSession: string = '';
  searchAuth: string = '';
  $searchUsersSessions = new Subject();
  $searchUsersAuth = new Subject();
  temporaryEmail : string = '';
  constructor(
    private localUserService: LocalUserService,
    private organizationService: OrganizationService,
    private deviceService: DeviceService,
    private router: Router,
    private globalMessageService: GlobalMessageService,
    private route: ActivatedRoute,
    private validDomainService: ValidDomainsService,
    private clientService: ClientService,
    private authenticationService: AuthenticationService,
    private licenseService: LicenseService,
    private modalService: BsModalService,
    private accessService: AccessPolicyService,
    private reportService: UsersReportsService,
    private workloadService: WorkloadService,
    private titleService: Title
  ) {}

  async ngOnInit() {
    this.disableSaveButton = false;
    this.loading = true;
    const id = this.route.snapshot.params.id;
    this.licenseService.getLicense().subscribe(response => {
      this.license = response.license;
      this.isMfaDisabled =
        this.license.mfa_feature == null ? true : !this.license.mfa_feature;
    });
    this.organizationService.getOrganizationNames().subscribe((orgs: any) => {
      this.organizations = orgs;
    });
    this.workloadService
      .getWorkloads(
        this.pagination.currentPage - 1,
        this.pagination.itemsPerPage,
        ''
      )
      .subscribe(response => {
        this.workloadData = response;
      });
    if (id) {
      this.userId = id;
      const response = (await this.localUserService
        .getSingleLocalUser(id)
        .toPromise()) as any;
      this.localUser = response.userData;
      this.globalDarkening = response.global_darkening;
      this.loading = false;
      this.localUser.password = '';
      this.editing = true;
      this.viewMode = this.router.url.includes('/view-user');
      this.getValidDomains(true);
      if (this.localUser.department) {
        this.organizationService
          .getDepartment(this.localUser.department)
          .subscribe((dept: any) => {
            this.displayedDept = dept.id;
            this.getAvailableDevices();
            this.organizationService
              .getOrganization(dept.organization_id)
              .subscribe((org: any) => {
                this.organizationService
                  .getAllDepartments(org.id)
                  .subscribe((departments: any) => {
                    this.departments = [];
                    departments.data.forEach(dept => {
                      this.departments.push({
                        deptId: dept.id,
                        deptName: dept.name
                      });
                    });
                  });
                this.displayedOrg = org.name;
                this.organizationChange(org.name);
              });
          });
      }
      this.getSessionsData(this.userId);
      this.getUserAuthFailureData(this.userId);
      this.getDevices(this.userId);
    } else {
      this.localUser.identity_provider = IdentityProvierEnum.Local;
      this.localUser.password = '';
      this.loading = false;
      this.getValidDomains(false);
    }
    if (this.viewMode) {
      this.titleService.setTitle('IAM | Users | View User');
    } else if (this.editing) {
      this.titleService.setTitle('IAM | Users | Edit User');
    } else {
      this.titleService.setTitle('IAM | Users | Add User');
    }
    this.subscribeToSearch();
    this.getUserServies();
    this.subscibeToSearch();
    this.subscibeToAuthSearch();
  }

  clearfunAuth() {
    this.searchAuth = '';
    this.getUserAuthFailureData(this.userId);
  }

  clearfun() {
    this.searchSession = '';
    this.getSessionsData(this.userId);
  }

  subscibeToSearch() {
    this.$searchUsersSessions
    .pipe(
      debounceTime(500),
      distinctUntilChanged(),
      switchMap((data: any) => {
        this.loading = true;
        this.searchSession = data;
        this.sessionPagination.currentPage = 1;
        return this.reportService
        .getSessionReport(this.userId, this.sessionPagination, 'desc', 'id', this.searchSession)
      })
    ).subscribe({
      next: (users: any) => {
        this.loading = false;
        const { recordsTotal } = users;
        this.sessionData = users.data;
        this.sessionPagination.totalItems = recordsTotal;
      },
      error: err => {
        this.loading = false;
      }
    });
  }

  subscibeToAuthSearch() {
    this.$searchUsersAuth
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          this.loading = true;
          this.searchAuth = data;
          this.failSessionPagination.currentPage = 1;
          return this.reportService
          .getSessionFailedReports(this.userId, this.failSessionPagination, 'desc', 'id', this.searchAuth)
        })
      ).subscribe({
        next: (auth: any) => {
          this.loading = false;
          const { recordsTotal } = auth;
          this.authFailureData = auth.data;
          this.failSessionPagination.totalItems = recordsTotal;
        },
        error: err => {
          this.loading = false;
        }
    });
  }

  getUserPoliciesById(id) {
    let userPolicyIds;
    let userEntitlements;
    const entIds = new Set();
    if (id) {
      this.accessService.getUserPolicyById(id).subscribe(
        (res: any) => {
          userPolicyIds = res.map(item => item.id);

          this.accessService
            .getAllPoliciesWithoutPagination('', 'asc', '', [], false)
            .subscribe(
              (entitlement: any) => {
                userEntitlements = entitlement.data?.filter(item =>
                  userPolicyIds?.includes(item.id)
                );
                this.entitlement = [
                  ...this.entitlement,
                  ...userEntitlements
                ]?.filter(item => {
                  if (!entIds?.has(item.id)) {
                    entIds?.add(item.id);
                    return true;
                  }
                  return false;
                });
                this.pagination.totalItems = this.entitlement.length;
              },
              error => {
                this.globalMessageService.pushAlert({
                  closed: false,
                  type: 'danger',
                  message: 'Failed to show entitlements',
                  sticky: false
                });
              }
            );
        },
        error => {
          userPolicyIds = [];
        }
      );
    }
  }

  getAvailableDevices() {
    if (this.displayedDept) {
      this.deviceService
        .getAvailableDevices(
          this.serverSideDeviceFound,
          this.mobileDeviceFound,
          this.displayedDept
        )
        .subscribe((devices: any) => {
          this.deviceService
          .clientsforAvailableDevices(devices)
          .subscribe((devices: any) => {
            this.availableDevices = devices;
          })

        });
    }
  }

  getUserServies() {
    if(this.editing || this.viewMode) {
      this.localUserService
      .getLocalUserServices(this.route.snapshot.params.id)
      .subscribe(
        (services: any) => {
          this.userServices = services.userServices;
          this.entitlements = services.entitlements;
          this.userServiceGroup = services.userServicesGroup;
          this.entitlementPagination.totalItems = services.entitlements.length;
          this.servicePagination.totalItems = services.userServices.length;
          this.groupPagination.totalItems = services.userServicesGroup.length;
          this.accessService
            .getAllPolicies(
              this.pagination,
              '',
              'asc',
              '',
              [services.client.id],
              false
            )
            .subscribe(entitlement => {
              this.entitlement = entitlement.data;
              this.pagination.totalItems = this.entitlement.length;
            });
        },
        error => {
          this.entitlements = [];
          this.userServices = [];
          this.userServiceGroup = [];
          this.groupingdata = [];
        }
      );
      this.getUserPoliciesById(this.route.snapshot.params.id);
    }
  }

  getSessionsData(userId) {
    this.reportService
      .getSessionReport(userId, this.sessionPagination, 'desc', 'id')
      .subscribe(response => {
        this.sessionData = response.data;
        this.sessionPagination.totalItems = response.recordsTotal;
      });
  }

  getUserAuthFailureData(userId) {
    this.reportService
      .getSessionFailedReports(userId, this.failSessionPagination, 'desc', 'id')
      .subscribe(response => {
        this.authFailureData = response.data;
        const { recordsTotal } = response;
        this.failSessionPagination.totalItems = recordsTotal;
      });
  }

  pageChange(page) {
    this.pagination.currentPage = page;
    if (this.searchWord) {
      this.$searchDevices.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getDevices(this.userId);
    }
  }

  sessionPageChange(page) {
    this.sessionPagination.currentPage = page;
    if (this.searchSession) {
      this.$searchUsersSessions.next({ searchWord: this.searchSession, typing: false });
    } else {
      this.getSessionsData(this.userId);
    }
  }

  authPageChange(page) {
    this.failSessionPagination.currentPage = page;
    if (this.searchAuth) {
      this.$searchUsersAuth.next({ searchWord: this.searchAuth, typing: false });
    } else {
      this.getUserAuthFailureData(this.userId);
    }
  }

  addDeviceToUser(event) {
    if (!this.deviceToAdd) {
      return;
    }

    const sameDepartment = this.devices.data.every(
      device => device.Department && device.Department.id === this.displayedDept
    );

    if (sameDepartment) {
      this.localUserService
        .addDeviceToUser(this.localUser.id, this.deviceToAdd)
        .subscribe(
          data => {
            this.getDevices(this.userId);
            this.deviceToAdd = null;
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'Device added successfully',
              sticky: false
            });
          },
          error => {
            this.deviceToAdd = null;
          }
        );
    } else {
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'danger',
        message:
          'User already have device(s) from other organization/departments',
        sticky: false
      });
    }
  }

  getDuration(CONNECTION_ATTEMPT_TIME, DISCONNECTION_TIME) {
    const connectionAttemptTime = new Date(CONNECTION_ATTEMPT_TIME);
    const disconnectionTime = new Date(DISCONNECTION_TIME);
    const durationMilliseconds =
      disconnectionTime.getTime() - connectionAttemptTime.getTime();
    const hours = Math.floor(durationMilliseconds / 3600000);
    const minutes = Math.floor((durationMilliseconds % 3600000) / 60000);
    const seconds = Math.floor((durationMilliseconds % 60000) / 1000);
    let durationString = '';
    if (hours > 0) {
      durationString += `${hours} Hours `;
    }
    if (minutes > 0) {
      durationString += `${minutes} Minutes `;
    }
    if (hours > 0 || minutes > 0 || seconds > 1) {
      durationString += `${seconds} Seconds`;
    } else {
      durationString += `${seconds} Second`;
    }
    return durationString;
  }

  getValidDomains(isEditing: boolean) {
    const domainPagination = new Pagination();
    domainPagination.itemsPerPage = 99999;
    this.validDomainService
      .getValidDomains(domainPagination)
      .subscribe((data: any) => {
        this.domains = data.data.map(d => {
          return {
            ...d,
            domain: '@' + d.domain
          };
        });
        this.reachableDomains = data.data.map(d => {
          return {
            ...d,
            reachDomain: '@' + d.domain
          };
        });
        if (this.domains.length !== 0 && !isEditing) {
          this.domainName = this.domains[0].domain;
          this.reachableDomainName = this.reachableDomains[0].domain;
        } else if (isEditing) {
          if(this.localUser.identity_provider == 'Active Directory'){
            const reachableSplittedEmail = this.localUser.reachable_email.split('@');
            this.localUser.reachable_email = reachableSplittedEmail[0];
            this.reachableDomainName = '@' + reachableSplittedEmail[1];
            const reachableDomiansExist = this.reachableDomains.map(domain => domain.reachDomain);          
            if (reachableDomiansExist.indexOf(this.reachableDomainName) === -1) {
              this.reachableDomainName = null;
            }
          }else{
            const splittedEmail = this.localUser.email.split('@');
            this.localUser.email = splittedEmail[0];
            this.domainName = '@' + splittedEmail[1];
            const domiansExist = this.domains.map(domain => domain.domain);
            if (domiansExist.indexOf(this.domainName) === -1) {
              this.domainName = null;
            }
          }
          if (this.domains.length === 0) {
            this.pushAlert();
          }
          if (this.reachableDomains.length === 0) {
            this.pushAlert();
          }
        } else {
          this.pushAlert();
        }
      });
  }

  pushAlert() {
    let warningAlert = {
      closed: false,
      type: 'danger',
      message:
        'First add the domains in portal setting tab to add/edit local user.',
      sticky: true,
      name: 'validDomainWarningAlert'
    };
    this.globalMessageService.pushAlert(warningAlert);
  }

  ngOnDestroy() {
    this.globalMessageService.removeAlertByName('validDomainWarningAlert');
  }

  emailWithoutDomainHandler(event) {
    const isOauthService = JSON.parse(localStorage.getItem('isOauthService'))
    if(isOauthService){
      if (event.target.value.includes('@') || event.target.value.includes('+')) {
        this.OAuthEmailValid = false;
      } else {
        this.OAuthEmailValid = true;
      }
    }else {
      if (event.target.value.includes('@')) {
        this.emailValid = false;
      } else {
        this.emailValid = true;
      }
    }
  }

  saveLocalUser() {
    if (this.invalidDeviceTypeCombo) {
      this.globalMessageService.pushAlert({
        closed: false,
        type: 'danger',
        message: 'Invalid device type combination detected.',
        sticky: false
      });
      return;
    }
    this.loading = true;
    this.disableSaveButton = true;
    if (this.editing && this.localUser.password) {
      let regexPattern = new RegExp(
        /^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[^A-z0-9]).{8,100}$/
      );
      if (!regexPattern.test(this.localUser.password)) {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message:
            'Use between 8 to 100 characters with a mix of letters, numbers and symbols e.g. 1nvisilyR@cks',
          sticky: false
        });
        return;
      }
    }

    this.localUserService
      .addLocalUser(this.localUser, this.editing, this.domainName,this.reachableDomainName)
      .subscribe(
        user => {
          this.loading = false;
          if (!this.editing) {
            this.router.navigate(['/users']);
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: 'Local User created successfully',
              sticky: false
            });
          } else {
            this.router.navigate(['/users']);
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: `${
                this.localUser.identity_provider === IdentityProvierEnum.Local
                  ? 'Local User'
                  : 'Active Directory User'
              } updated successfully`,
              sticky: false
            });
          }
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
        }
      );
  }

  cancelHandler() {
    this.router.navigate(['/users']);
  }

  onClick() {
    this.modalService.show(ModalComponent, {
      initialState: sharedContent.passwordSuggestion,
      class: 'modal-dialog-centered'
    });
  }

  generateRandomPassword() {
    this.localUser.password =
      Math.random()
        .toString(36)
        .toUpperCase()
        .replace(/[0-9O]/g, '')
        .substring(1, 3) +
      Math.random()
        .toString(36)
        .toLowerCase()
        .replace(/[0-9O]/g, '')
        .substring(1, 3) +
      Math.floor(Math.random() * (999 - 100 + 1) + 100) +
      '!@#$&*()<>.,?'.substr(Math.floor(13 * Math.random()), 1);
    this.confirmPasswordField = this.localUser.password;
  }

  showPassword() {
    const passwordField = $('#password');
    passwordField.prop(
      'type',
      passwordField.prop('type') === 'password' ? 'text' : 'password'
    );
  }

  async organizationChange(orgName) {
    if (!orgName) {
      this.availableDevices = [];
      this.departments = [{ deptId: null, deptName: null }];
      this.displayedDept = null;
      return;
    }
    // this.departments = [];
    const org = (await this.organizationService
      .getOrganizationByName(orgName)
      .toPromise()) as any;
    const departments = (await this.organizationService
      .getAllDepartments(org.id)
      .toPromise()) as any;
    this.organizationId = org.id;
    this.departments = [];
    for (let j = 0; j < departments.data.length; j++) {
      this.departments.push({
        deptId: departments.data[j].id,
        deptName: departments.data[j].name
      });
    }
    if (this.initDone) this.displayedDept = null;
    this.initDone = true;
  }

  departmentChange(deptId) {
    if (!deptId) {
      this.availableDevices = [];
      return;
    }
    this.localUser.department = deptId;
    if (this.editing) {
      this.getAvailableDevices();
    }
  }

  subscribeToSearch() {
    this.$searchDevices
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.loading = true;
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.deviceService.searchDevicesForUser(
            search.searchWord,
            5,
            this.pagination.currentPage,
            'device_name',
            'asc',
            this.userId
          );
        })
      )
      .subscribe((devices: any) => {
        this.devices = devices;
        this.pagination.totalItems = devices.recordsTotal;
        this.loading = false;
      });
  }

  getDevices(id) {
    this.loading = true;
    this.pagination.itemsPerPage = 5;
    this.deviceService
      .getDevicesForUser(
        this.pagination,
        'device_name',
        'asc',
        this.searchWord,
        id
      )
      .subscribe(
        (response: any) => {
          this.loading = false;
          this.serverSideDeviceFound = false;
          response.data = response.data.map(_device => {
            if (
              [
                DeviceTypeEnum.PhysicalServer,
                DeviceTypeEnum.VirtualServer
              ].includes(_device.device_type)
            ) {
              this.serverSideDeviceFound = true;
            }
            if (
              [DeviceTypeEnum.Smartphone, DeviceTypeEnum.Tablet].includes(
                _device.device_type
              )
            ) {
              this.mobileDeviceFound = true;
            }
            return { ..._device, isChecked: false };
          });
          this.devices = response;
          this.pagination.totalItems = response.recordsTotal;
          this.invalidDeviceTypeCombo =
            this.serverSideDeviceFound &&
            this.mobileDeviceFound &&
            response.data.length > 2;
          this.getAvailableDevices();
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting devices',
            sticky: false
          });
        }
      );
  }

  deleteDevice(device) {
    this.selectedDevice = device;
    this.showConfirm = true;
    this.devicePolicyLoading = true;
    this.localUserService
      .getUserDeviceRelatedPolicies(this.localUser.id, device.id)
      .subscribe((response: any) => {
        this.relatedPoliciesCount = response.data;
        this.devicePolicyLoading = false;
      });
  }

  deleteDeviceFromUser() {
    this.cancelConfirmation();
    this.localUserService
      .removeDeviceFromUser(this.localUser.id, this.selectedDevice.id)
      .subscribe(data => {
        this.getDevices(this.userId);
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'Device removed successfully',
          sticky: false
        });
      });
  }

  cancelConfirmation() {
    this.showConfirm = false;
  }

  updateDarkeningState(device) {
    this.deviceService
      .toggleDarkeningState(device.id, device.darkening_state)
      .subscribe(data => {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'Darkening State updated successfully',
          sticky: false
        });
      });
  }

  changeClientMfa() {
    // this.client.client_mfa = !this.client.client_mfa;
    if (this.localUser.client_mfa === false) {
      this.localUser.biometric_state = false;
      this.localUser.is_strong_mfa = false;
    }
  }

  changeBiometricMfa() {
    this.localUser.biometric_state = !this.localUser.biometric_state;
    if (!this.localUser.biometric_state) this.localUser.is_strong_mfa = false;
  }

  showOTPQrCode(device_id) {
    this.showOtpImage = false;
    this.clientService.generateOTP(device_id).subscribe(
      (data: any) => {
        this.OTPQcodeImageData = data.OTP_image_data;
        this.showOtpImage = true;
      },
      error => {
        this.alert = {
          closed: false,
          type: 'danger',
          message: 'Error in generating linOTP Token'
        };
        this.globalMessageService.pushAlert(this.alert);
      }
    );
  }

  cancelOTPQrCode() {
    this.showOtpImage = false;
  }

  sendOTPQrCode(device_id, user_id) {
    this.clientService
      .sendOTPQr(device_id, user_id)
      .pipe(
        tap((response: any) => {
          if (response.success) {
            const alert = {
              closed: false,
              type: 'success',
              message: 'Client will receive OTP Code within 2 minutes.'
            };
            this.globalMessageService.pushAlert(alert);
          }
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
            }
          } else {
            const alert = {
              closed: false,
              type: 'danger',
              message: 'Something went wrong while requesting OTP QR Code.'
            };
            this.globalMessageService.pushAlert(alert);
          }
        },
        complete: () => {}
      });
  }
}
