import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { ManageService } from '../manage-service/manage.service';
import { Pagination } from '../models/pagination';
import { ServiceGroup } from '../models/serviceGroup';
import { PolicyService } from '../policy/policy.service';
import { ServiceConnectionOptionsEnum } from '../shared/emuns/service.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-policy',
  templateUrl: './create-policy.component.html',
  styleUrls: ['./create-policy.component.css']
})
export class CreatePolicyComponent implements OnInit {
  policyName = 'Policy Name';
  Object = Object;
  // All Services variables
  allServices = [];
  allServicesPagination: Pagination = new Pagination();
  allServicesSort: string = 'createdAt';
  allServicesSortDir: string = 'desc';
  allServicesSearch: string = '';
  allSelectedServices = {};
  allServicesSelected: boolean = false;
  disableAddAllServices: boolean = true;
  servicesLoading: boolean = false;
  selectAllExcept = new Set();
  allPersistentFilter: boolean = false;

  // Current Services Variables
  currentServices = [];
  filteredCurrentServices = [];
  currentServicesPagination: Pagination = new Pagination();
  currentServicesSort: string = 'createdAt';
  currentServicesSortDir: string = 'desc';
  currentServicesSearch: string = '';
  currentSelectedServices = {};
  selectCurrentExcept = new Set();
  allCurrentServicesSelected: boolean = false;
  disableDeleteCurrentServices: boolean = true;
  currentLoading: boolean = false;
  currentPersistentFilter: boolean = false;
  selectedProtocol = null;
  selectedType = null;

  pageSize = [10, 25, 50, 100, 200];
  serviceGroup: ServiceGroup = new ServiceGroup();
  editingMode: boolean = false;
  currentSelectedProtocol = null;
  currentSelectedType = null;

  protocols = [];
  types = [];
  isSaving: boolean = false; // will keep the track either the Saving is in Process

  constructor(
    private manageServices: ManageService,
    private policyService: PolicyService,
    private router: Router,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {}

  ngOnInit() {
    const groupId = this.route.snapshot.params.id;
    if (groupId) {
      this.policyService
        .getGroupDetailById(groupId)
        .subscribe((serviceGroup: any) => {
          this.currentServices = serviceGroup.Services;
          this.serviceGroup = new ServiceGroup();
          this.serviceGroup.id = serviceGroup.id;
          this.serviceGroup.name = serviceGroup.name;
          this.serviceGroup.description = serviceGroup.description;

          this.getPaginatedCurrentServices();
        });
      this.editingMode = true;
    }
    if (this.editingMode) {
      this.titleService.setTitle('Infrastructure | Service Group | Edit Service Group');
    }
    else {
      this.titleService.setTitle('Infrastructure | Service Group | Add Service Group');
    }
    this.manageServices.getAllTypesAndProtocols().subscribe(response => {
      this.protocols = response.protocols;
      this.types = response.types;
    });

    this.getAllServicesWithPagination();
  }

  // All Services methods

  getAllServicesWithPagination() {
    this.servicesLoading = true;
    this.manageServices
      .getServices(
        this.allServicesPagination.currentPage,
        this.allServicesPagination.itemsPerPage,
        'isActive',
        this.allServicesSort,
        this.allServicesSortDir,
        this.allServicesSearch,
        this.allPersistentFilter,
        ServiceConnectionOptionsEnum.All,
        [],
        false,
        this.selectedType,
        this.selectedProtocol
      )
      .subscribe(
        response => {
          this.allServicesPagination.totalItems = response.recordsTotal;
          const self = this;
          this.allServices = response.data.map(service => {
            return {
              ...service,
              isChecked:
                self.allServicesSelected &&
                !self.selectAllExcept.has(service.id)
                  ? true
                  : self.allSelectedServices[service.id]
                  ? true
                  : false
            };
          });
          this.servicesLoading = false;
          this.checkAlreadyAddedServices();
        },
        err => {
          this.servicesLoading = false;
        }
      );
  }

  allPaginationPageChange(event) {
    this.allServicesPagination.currentPage = event;
    this.getAllServicesWithPagination();
    this.checkAlreadyAddedServices();
  }

  allServicesPageSizeChange(event) {
    this.allServicesPagination.itemsPerPage = event;
    this.getAllServicesWithPagination();
    this.checkAlreadyAddedServices();
  }

  handleAllServicesSearch() {
    this.clearAllSelections();
    this.allServicesPagination.currentPage = 1;
    this.getAllServicesWithPagination();
    this.checkAlreadyAddedServices();
  }

  sortAllServices(sort) {
    this.allServicesSort = sort;
    this.allServicesSortDir =
      this.allServicesSortDir == 'desc' ? 'asc' : 'desc';
    this.allServicesPagination.currentPage = 1;
    this.getAllServicesWithPagination();
    this.checkAlreadyAddedServices();
  }

  selectAllServices() {
    this.allServices = this.allServices.map(service => {
      return {
        ...service,
        isChecked: this.allServicesSelected
      };
    });
    this.selectAllExcept = new Set();
    this.allSelectedServices = {};
    this.updateSelectedCount();
  }

  changeAllServiceCheckBox(service) {
    if (this.allServicesSelected) {
      if (this.selectAllExcept.has(service.id)) {
        this.allServices = this.allServices.map(myService => {
          return {
            ...myService,
            isChecked: service.id === myService.id ? true : myService.isChecked
          };
        });
        this.selectAllExcept.delete(service.id);
      } else {
        this.allServices = this.allServices.map(myService => {
          return {
            ...myService,
            isChecked: service.id === myService.id ? false : myService.isChecked
          };
        });
        this.selectAllExcept.add(service.id);
      }

      // this.allServicesSelected = false
      // this.allServices = this.allServices.map(myService => {
      //   return {
      //     ...myService,
      //     isChecked: service.id === myService.id
      //   }
      // })
      // this.allSelectedServices[service.id] = service
    } else {
      if (service.isChecked) {
        this.allSelectedServices[service.id] = service;
      } else {
        delete this.allSelectedServices[service.id];
      }
    }
    this.updateSelectedCount();
  }

  clearAllSelections() {
    this.selectAllExcept.clear();
    this.allServicesSelected = false;
  }

  updateSelectedCount() {
    let disable = Object.keys(this.allSelectedServices).length < 1;
    if (this.allServicesSelected) {
      disable = false;
    }
    this.disableAddAllServices = disable;
  }

  checkAlreadyAddedServices() {
    const currentIds = this.currentServices.map(service => service.id);
    this.allServices = this.allServices.map(service => {
      return {
        ...service,
        alreadySelected: currentIds.includes(service.id)
      };
    });
  }

  isMixOfPersistent(data) {
    if (data.length > 1) {
      const currentPersistentState = data[0].is_persistent;
      return data.some(
        service => service.is_persistent !== currentPersistentState
      );
    }
    return false;
  }

  addSelectedServices() {
    if (this.allServicesSelected) {
      this.manageServices
        .getServices(
          this.allServicesPagination.currentPage,
          this.allServicesPagination.itemsPerPage,
          'isActive',
          this.allServicesSort,
          this.allServicesSortDir,
          this.allServicesSearch,
          this.allPersistentFilter,
          ServiceConnectionOptionsEnum.All,
          [],
          true,
          this.selectedType,
          this.selectedProtocol,
          Array.from(this.selectAllExcept)
        )
        .subscribe(response => {
          const currentIds = this.currentServices.map(service => service.id);
          let data = response.data.filter(
            service => !currentIds.includes(service.id)
          );
          let tempData = [...this.currentServices, ...data];
          if (this.isMixOfPersistent(tempData)) {
            this.globalMessageService.pushAlert({
              type: 'danger',
              message:
                'Mix of persistent and non persistent services not allowed'
            });
            this.servicesLoading = false;
            return;
          }
          this.currentServices = tempData;
          this.allServices = this.allServices.map(service => {
            return {
              ...service,
              isChecked: false
            };
          });
          this.allServicesSelected = false;
          this.allSelectedServices = {};
          this.updateCurrentServicesCount();
          this.servicesLoading = false;
          this.getPaginatedCurrentServices();
          this.checkAlreadyAddedServices();
        });
    } else {
      this.servicesLoading = true;
      const currentIds = this.currentServices.map(service => service.id);
      let data = Object.values(this.allSelectedServices).map((service: any) => {
        return {
          ...service,
          isChecked: false
        };
      });
      data = data.filter(service => !currentIds.includes(service.id));
      let tempData = [...this.currentServices, ...data];
      if (this.isMixOfPersistent(tempData)) {
        this.globalMessageService.pushAlert({
          type: 'danger',
          message: 'Mix of persistent and non persistent services not allowed'
        });
        this.servicesLoading = false;
        return;
      }
      this.allServices = this.allServices.map(service => {
        return {
          ...service,
          isChecked: false
        };
      });
      this.currentServices = tempData;
      this.allServicesSelected = false;
      this.allSelectedServices = {};
      this.updateCurrentServicesCount();
      this.servicesLoading = false;
      this.getPaginatedCurrentServices();
      this.checkAlreadyAddedServices();
    }
    this.selectAllExcept = new Set();
  }

  changeAllPersistenceFlag(persistent) {
    this.allPersistentFilter = persistent;
    this.allServicesPagination.currentPage = 1;
    this.clearAllSelections();
    this.getAllServicesWithPagination();
    this.updateSelectedCount();
  }

  allServicesFilterChange() {
    this.allServicesPagination.currentPage = 1;
    this.clearAllSelections();
    this.getAllServicesWithPagination();
  }

  // All Current Services Pagination

  // getAllCurrentServicesWithPagination() {
  //   this.currentLoading = true
  //   this.manageServices.getCurrentSelectedServices(
  //     this.currentServicesPagination.currentPage,
  //     this.currentServicesPagination.itemsPerPage,
  //     this.currentServicesSort,
  //     this.currentServicesSortDir,
  //     this.currentServicesSearch,
  //     this.allServicesSelected
  //   ).subscribe(response => {
  //     this.allServicesPagination.totalItems = response.recordsTotal
  //     const self = this
  //     this.allServices = response.data.map(service => {
  //       return {
  //         ...service,
  //         isChecked: self.allSelectedServices[service.id]? true: false
  //       }
  //     })
  //     this.allServicesSelected = false
  //     this.servicesLoading = false
  //     this.checkAlreadyAddedServices()
  //   }, err  => {
  //     this.servicesLoading = false
  //   })
  // }

  currentPaginationPageChange(event) {
    this.currentServicesPagination.currentPage = event;
    this.getPaginatedCurrentServices();
  }

  clearCurrentSelection() {
    this.selectCurrentExcept.clear();
    this.allCurrentServicesSelected = false;
  }

  currentServicesPageSizeChange(event) {
    if (event) {
      this.currentServicesPagination.itemsPerPage = event;
      this.getPaginatedCurrentServices();
      this.checkAlreadyAddedServices();
    }
  }

  sortCurrentServices(sort) {
    this.currentServicesSort = sort;
    this.currentServicesSortDir =
      this.currentServicesSortDir == 'desc' ? 'asc' : 'desc';
    this.currentServicesPagination.currentPage = 1;
    this.currentServices = this.currentServices.sort((a, b) => {
      return this.currentServicesSortDir == 'desc'
        ? a[sort].toString().localeCompare(b[sort])
        : b[sort].toString().localeCompare(a[sort]);
    });
    this.getPaginatedCurrentServices();
  }

  selectAllCurrentServices() {
    // const paginatedIds = this.filteredCurrentServices.map(service => service.id)
    // this.currentServices = this.currentServices.map(service => {
    //   return paginatedIds.includes(service.id)?
    //     ({
    //       ...service, isChecked: this.allCurrentServicesSelected })
    //       : service
    // })

    // if(this.allCurrentServicesSelected)
    // {
    //   // saving all selected services in normalize form
    //   this.filteredCurrentServices.forEach(service => {
    //     this.currentSelectedServices[service.id] = service
    //   })
    // }
    // else{
    //   this.currentServices.forEach(service => {
    //     delete this.currentSelectedServices[service.id]
    //   })
    // }
    this.currentServices = this.currentServices.map(service => {
      return {
        ...service,
        isChecked: false
      };
    });
    this.currentSelectedServices = {};
    this.selectCurrentExcept.clear();
    this.getPaginatedCurrentServices();
    this.updateCurrentSelectedCount();
  }

  changeCurrentServiceCheckBox(service) {
    if (this.allCurrentServicesSelected) {
      if (this.selectCurrentExcept.has(service.id)) {
        this.selectCurrentExcept.delete(service.id);
      } else {
        this.selectCurrentExcept.add(service.id);
      }
    } else {
      if (service.isChecked) {
        this.currentSelectedServices[service.id] = service;
      } else {
        delete this.currentSelectedServices[service.id];
      }
    }
    // this.allCurrentServicesSelected = false
    this.updateCurrentSelectedCount();
  }

  updateCurrentSelectedCount() {
    let disable = Object.keys(this.currentSelectedServices).length < 1;
    if (this.allCurrentServicesSelected) {
      disable = false;
    }
    this.disableDeleteCurrentServices = disable;
  }

  updateCurrentServicesCount() {
    this.currentServicesPagination.totalItems = this.currentServices.length;
  }

  getFilteredCurrentServices() {
    let services = [...this.currentServices];

    if (this.currentSelectedProtocol) {
      services = services.filter(
        service => service.protocol === this.currentSelectedProtocol
      );
    }
    if (this.currentSelectedType) {
      services = services.filter(
        services => services.type === this.currentSelectedType
      );
    }
    if (this.currentPersistentFilter) {
      services = services.filter(service => service.is_persistent);
    }
    return services.filter(
      service =>
        service.Name.toLowerCase().indexOf(
          this.currentServicesSearch.toLowerCase()
        ) > -1
    );
  }

  getPaginatedCurrentServices() {
    this.currentLoading = true;
    const offset =
      this.currentServicesPagination.itemsPerPage *
      (this.currentServicesPagination.currentPage - 1);
    let services = this.getFilteredCurrentServices();

    this.currentServicesPagination.totalItems = services.length;
    this.filteredCurrentServices = services.slice(
      offset,
      offset + this.currentServicesPagination.itemsPerPage
    );
    if (this.allCurrentServicesSelected) {
      this.filteredCurrentServices = this.filteredCurrentServices.map(
        service => {
          return {
            ...service,
            isChecked: !this.selectCurrentExcept.has(service.id)
          };
        }
      );
    }
    // if(this.filteredCurrentServices.every(service => service.isChecked) && this.filteredCurrentServices.length > 0)
    // {
    //   this.allCurrentServicesSelected = true
    // }
    // else{
    //   this.allCurrentServicesSelected = false
    // }
    this.currentLoading = false;
    this.checkAlreadyAddedServices();
    this.updateSelectedCount();
    this.updateCurrentSelectedCount();
    // this.updateCurrentServicesCount()
  }

  deleteSelectedServices() {
    this.currentLoading = true;
    let temp = [];

    if (!this.allCurrentServicesSelected) {
      this.currentServices.forEach((service, i) => {
        if (service.isChecked) {
          delete this.currentSelectedServices[service.id];
        }
        if (!service.isChecked) {
          temp.push(service);
        }
      });
      this.currentServicesPagination.totalItems = temp.length;
    }

    if (this.allCurrentServicesSelected) {
      let servicesToDelete = this.getFilteredCurrentServices();
      servicesToDelete = servicesToDelete.filter(
        service => !this.selectCurrentExcept.has(service.id)
      );
      let servicesIds = servicesToDelete.map(service => service.id);
      temp = this.currentServices.filter(
        service => !servicesIds.includes(service.id)
      );
    }

    setTimeout(() => {
      this.currentServices = [...temp];
      this.allCurrentServicesSelected = false;
      const offset = 0;
      this.updateCurrentSelectedCount();
      this.currentServicesPagination.currentPage = 1;
      this.getPaginatedCurrentServices();
      this.currentLoading = false;
      this.checkAlreadyAddedServices();
    }, 200);
    this.selectCurrentExcept = new Set();
  }

  saveHandler() {
    this.isSaving = true;

    if (this.isMixOfPersistent(this.currentServices)) {
      this.globalMessageService.pushAlert({
        type: 'danger',
        message: 'Mix of persistent and non persistent services not allowed'
      });
      this.isSaving = false;
      return;
    }
    const serviceIds = this.currentServices.map(service => service.id);
    if (this.editingMode) {
      this.policyService
        .editGroup(
          this.serviceGroup.id,
          this.serviceGroup.name,
          this.serviceGroup.description,
          serviceIds
        )
        .subscribe(
          response => {
            this.globalMessageService.pushAlert({
              type: 'success',
              message: response
            });
            this.cancelHandler();
            this.isSaving = false;
          },
          err => {
            this.isSaving = false;
          }
        );
    } else {
      this.policyService
        .addNewGroup(
          this.serviceGroup.name,
          this.serviceGroup.description,
          serviceIds
        )
        .subscribe(
          (response: any) => {
            this.globalMessageService.pushAlert({
              type: 'success',
              message: response.message
            });
            this.cancelHandler();
            this.isSaving = false;
          },
          err => {
            this.isSaving = false;
          }
        );
    }
  }

  cancelHandler() {
    this.router.navigate(['/serviceGroups']);
  }

  currentServicesFilterChange() {
    this.clearCurrentSelection();
    this.currentServicesPagination.currentPage = 1;
    this.getPaginatedCurrentServices();
  }

  changeCurrentPersistenceFlag(persistent: boolean) {
    this.clearCurrentSelection();
    this.currentPersistentFilter = persistent;
    this.getPaginatedCurrentServices();
  }
}
