import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { Subject } from 'rxjs';
import { debounceTime, distinctUntilChanged, switchMap } from 'rxjs/operators';
import { LocalUserService } from '../create-local-user/local-user.service';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { IotService } from '../iot.service';
import { Pagination } from '../models/pagination';
import { DeviceOSEnum } from '../shared/emuns/client.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-rbi-bridge',
  templateUrl: './create-rbi-bridge.component.html',
  styleUrls: ['./create-rbi-bridge.component.css']
})

export class CreateRbiBridgeComponent implements OnInit {
  type: string = 'rbi-bridge';
  clients: any = [];
  selectedClientIds;
  clientOffset = 0;
  currentClientSearch = '';
  $clientSearch = new Subject();
  isClientSearching = false;
  disableSaveButton = false;

  // localUser = [];
  // selectedUserId;
  // userOffset = 0;
  // currentUserSearch = '';
  // $physicalUserSearch = new Subject();
  // isPhysicalDeviceSearching = false;

  users = [];
  selectedUserId;
  userOffset = 0;
  currentUserSearch = '';
  $userSearch = new Subject();
  isUserSearching = false;

  selectedUsers = [];

  selectedUserIds = [];
  title: string;
  isEditing = false;
  currentEditingId;
  bridgeName = '';
  portalURL = '';
  portalPort = '';
  minVncPort = '';
  maxVncPort = '';
  // deviceIdsMatch s= false;

  loading: Boolean = false;

  pane: number = 0;
  addUserIds;

  constructor(
    public router: Router,
    private activatedRoute: ActivatedRoute,
    private iotService: IotService,
    private localUserService: LocalUserService,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {
      this.titleService.setTitle('Infrastructure | RBI Bridges | Add RBI Bridge');
    // this.type = router.url.replace('/create/', '');
    // this.type = this.type.replace('/edit/', '');
    // this.type = this.type.replace(/[0-9]/g, '');
    // this.type = this.type.replace('/', '');
    // switch(this.type){
    //   case 'iot-bridge':
    //     this.typeTitle = 'IoT Bridge';
    //     break;
    //   case 'client-bridge':
    //     this.typeTitle = 'Thin Client Bridge';
    //     break;
    //   default:
    //     this.typeTitle = 'Server Bridge';
    //     break;
    // }
  }

  ngOnInit() {
    this.disableSaveButton = false;
    this.activatedRoute.params.subscribe(data => {
      // this.getPhysicalDevices();
      this.getUsers();
      if (!data.id) {
        //create
        this.title = 'Add';
        this.getServerClients();
      } else {
        //Edit
        // this.deviceIdsMatch = true;
        this.isEditing = true;
        this.title = 'Edit';
        this.currentEditingId = data.id;
        this.getSinglePolicy(data.id);
      }
    });
    this.subscribeClientSearch();
  }

  getServerClients() {
    this.loading = true;
    this.iotService
      .getAllDropDownData(
        this.clientOffset,
        this.currentClientSearch,
        this.type
      )
      .subscribe(
        (data: any) => {
          this.clients = [...this.clients, ...data];
          this.loading = false;
        },
        err => {
          this.loading = false;
        }
      );
  }

  // getPhysicalDevices(){
  //   this.loading = true;
  //   this.iotService.getPhysicalDevices(
  //     this.physicalDeviceOffset,
  //     this.currentPhysicalDeviceSearch,
  //     this.type
  //   ).subscribe((data) => {
  //     this.physicalDevices = [...this.physicalDevices, ...data];
  //     this.loading = false;
  //   },
  //   err => {
  //     this.loading = false;
  //   })
  // }

  getUsers() {
    this.loading = true;
    this.localUserService
      .getLocalUsers(
        new Pagination('', 20, this.userOffset + 1),
        null,
        null,
        this.currentUserSearch
      )
      .subscribe(
        (data: any) => {
          this.users = [...this.users, ...data.data];
          this.loading = false;
        },
        err => {
          this.loading = false;
        }
      );
  }

  private subscribeClientSearch() {
    this.$clientSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isClientSearching = true;
          this.currentClientSearch = term;
          this.clientOffset = 0;
          return this.iotService.getAllDropDownData(
            this.clientOffset,
            this.currentClientSearch,
            this.type
          );
        })
      )
      .subscribe(data => {
        this.isClientSearching = false;
        this.clients = data;
      });

    // this.$physicalDeviceSearch
    // .pipe(
    //   debounceTime(500),
    //   distinctUntilChanged(),
    //   switchMap((term: string) => {
    //     this.isPhysicalDeviceSearching = true;
    //     this.currentPhysicalDeviceSearch = term;
    //     this.physicalDeviceOffset = 0;
    //     return this.iotService.getPhysicalDevices(this.physicalDeviceOffset,this.currentPhysicalDeviceSearch, this.type);
    //   })
    // )
    // .subscribe(data => {
    //   this.isPhysicalDeviceSearching = false;
    //   this.physicalDevices = data;
    // });

    this.$userSearch
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((term: string) => {
          this.isUserSearching = true;
          if (term !== null) {
            this.currentUserSearch = term;
          } else {
            this.currentUserSearch = ''
          }
          this.userOffset = 0;
          return this.localUserService.getLocalUsers(
            new Pagination('', 20, this.userOffset + 1),
            null,
            null,
            this.currentUserSearch
          );
        })
      )
      .subscribe((data: any) => {
        this.isUserSearching = false;
        if (this.currentUserSearch === '') {
          const combined = [...this.users, ...data.data];
          const uniqueSet = new Set(combined.map(item => JSON.stringify(item)));
          this.users = Array.from(uniqueSet).map(item => JSON.parse(item));
        }
        else {
          this.users = data.data;
        }
      });
  }

  fetchMoreClients() {
    this.clientOffset += 1;
    this.getServerClients();
  }

  // fetchMorePhysicalDevices() {
  //   this.physicalDeviceOffset += 1;
  //   this.getPhysicalDevices();
  // }

  fetchMoreUsers() {
    this.userOffset += 1;
    this.getUsers();
  }

  private getSinglePolicy(id) {
    this.loading = true;
    this.iotService.getSinglePolicy(id).subscribe(
      (data: any) => {
        this.loading = false;
        const {
          name,
          serverBridgeIds,
          serverBridgeUserIds,
          Client,
          clientId,
          device_id
        } = data;

        // const {
        //   portalUrl,
        //   portalPort,
        //   minVncPort,
        //   maxVncPort
        // } = data.B2bPortal

        this.clients = [Client];
        this.selectedClientIds = clientId;
        this.bridgeName = name;
        // this.portalURL = portalUrl;
        // this.portalPort = portalPort;
        // this.minVncPort = minVncPort;
        // this.maxVncPort = maxVncPort;

        // this.selectedDeviceIds = serverBridgeIds.map(device => device.deviceId);
        // this.selectedPhysicalDeviceId = device_id;
        this.selectedUsers =
          serverBridgeUserIds && serverBridgeUserIds.map(s => s.LocalUser);
      },
      err => {
        this.loading = false;
      }
    );
  }

  savePolicyHandler(event) {
    this.loading = true;
    this.disableSaveButton = true;
    event.stopPropagation();
    const users = this.selectedUsers.map(x => x.id);

    if (!this.isEditing) {
      this.iotService
        .createRBIBridge(
          this.bridgeName,
          this.selectedClientIds,
          users,
          this.portalURL,
          this.portalPort,
          this.minVncPort,
          this.maxVncPort
        )
        .subscribe(
          data => {
            this.loading = false;
            this.showAlert('success', 'RBI Bridge Created', false);
            this.router.navigate([`rbi-bridge`]);
          },
          error => {
            this.loading = false;
            this.disableSaveButton = false;
            // this.showAlert('danger', JSON.parse(error._body).message, false);
          }
        );
    } else {
      this.iotService
        .updateRBIBridge(
          this.currentEditingId,
          this.bridgeName,
          this.selectedClientIds,
          users,
          this.portalURL,
          this.portalPort,
          this.minVncPort,
          this.maxVncPort
        )
        .subscribe(
          data => {
            this.loading = false;
            this.showAlert('success', 'RBI Bridge Updated', false);
            this.router.navigate([`/rbi-bridge`]);
          },
          error => {
            this.loading = false;
            this.disableSaveButton = false;
            // this.showAlert('danger', JSON.parse(error._body).message, false);
          }
        );
    }
  }

  cancelHandler() {
    this.router.navigate([`/rbi-bridge`]);
  }

  private showAlert(type: string, message: string, sticky: boolean) {
    const alert = {
      closed: false,
      type: type,
      message: message,
      sticky: sticky
    };
    this.globalMessageService.pushAlert(alert);
  }

  deviceIdSelected() {
    // this.deviceIdsMatch = true;
    // const regex = /^([0-9A-Fa-f]{2}[:]){5}([0-9A-Fa-f]{2})$/;
    // this.selectedDeviceIds.forEach(id => {
    //   if (!regex.test(id)) {
    //     this.deviceIdsMatch = false;
    //   }
    // });
  }

  addUser() {
    if (this.selectedUserId && !this.isAlreadyExist(this.selectedUserId)) {
      this.selectedUsers.push(
        this.users.filter(x => x.id === this.selectedUserId)[0]
      );
    }
  }

  isAlreadyExist(id) {
    for (let i = 0; i < this.selectedUsers.length; i++) {
      if (this.selectedUsers[i].id == id) {
        return true;
      }
    }
    return false;
  }

  removeUser(id) {
    this.selectedUsers = this.selectedUsers.filter(x => x.id !== id);
  }

  appendrightOs(os) {
    switch (os) {
      case DeviceOSEnum.Ubuntu18:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer18:
        return 'Ubuntu Server';
      case DeviceOSEnum.Macintosh:
        return 'Macintosh';
      case DeviceOSEnum.Windows:
        return 'Windows';
      case DeviceOSEnum.Ubuntu:
        return 'Ubuntu Desktop';
      case DeviceOSEnum.UbuntuServer:
        return 'Ubuntu Server';
      case DeviceOSEnum.WindowsServer:
        return 'Windows Server';
      case DeviceOSEnum.IOS:
        return 'iOS';
      case DeviceOSEnum.Android:
        return 'Android';
      case DeviceOSEnum.UbuntuServer20armv8:
        return 'Ubuntu Server (ARMv8)';
      case DeviceOSEnum.CentosServer:
        return 'CentOS Server';
      case DeviceOSEnum.Unknown:
        return 'Unknown';
      default:
        return os;
    }
  }
}
