import { Directive, HostListener, ElementRef } from '@angular/core';
import { KeyboardKeyCodeEnum } from '../shared/emuns/global.enum';

@Directive({
  selector: '[digitOnly]'
})
export class DigitOnlyDirective {

  constructor(private inputElement: ElementRef) {}

  @HostListener('keydown', ['$event'])
  onKeyDown(e: KeyboardEvent) {
    if (
      // Allow: Delete, Backspace, Tab, Escape, Enter
      [46, 8, 9, 27, 13].indexOf(e.keyCode) !== -1 ||
      (e.keyCode === KeyboardKeyCodeEnum.CtrlA && e.ctrlKey === true) || // Allow: Ctrl+A
      (e.keyCode === KeyboardKeyCodeEnum.CtrlC && e.ctrlKey === true) || // Allow: Ctrl+C
      (e.keyCode === KeyboardKeyCodeEnum.CtrlV && e.ctrlKey === true) || // Allow: Ctrl+V
      (e.keyCode === KeyboardKeyCodeEnum.CtrlX && e.ctrlKey === true) || // Allow: Ctrl+X
      (e.keyCode === KeyboardKeyCodeEnum.CmdA && e.metaKey === true) || // Cmd+A (Mac)
      (e.keyCode === KeyboardKeyCodeEnum.CmdC && e.metaKey === true) || // Cmd+C (Mac)
      (e.keyCode === KeyboardKeyCodeEnum.CmdV && e.metaKey === true) || // Cmd+V (Mac)
      (e.keyCode === KeyboardKeyCodeEnum.CmdX && e.metaKey === true) || // Cmd+X (Mac)
      (e.keyCode >= 35 && e.keyCode <= 39) // Home, End, Left, Right
    ) {
      return; // let it happen, don't do anything
    }
    // Ensure that it is a number and stop the keypress
    if (
      (e.shiftKey || (e.keyCode < 48 || e.keyCode > 57)) &&
      (e.keyCode < 96 || e.keyCode > 105)
    ) {
      e.preventDefault();
    }
  }

  @HostListener('paste', ['$event'])
  onPaste(event: ClipboardEvent) {
    event.preventDefault();
    const pastedInput: string = event.clipboardData
      .getData('text/plain')
      .replace(/\D/g, ''); // get a digit-only string
    document.execCommand('insertText', false, pastedInput);
  }
  @HostListener('drop', ['$event'])
  onDrop(event: DragEvent) {
    event.preventDefault();
    const textData = event.dataTransfer.getData('text').replace(/\D/g, '');
    const inputElement = this.inputElement as any;
    inputElement.focus();
    document.execCommand('insertText', false, textData);
  }
}
