import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { GatewayService } from '../gateway.service';
import { Gateway } from '../../models/gateway';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import * as _ from 'lodash';
import * as moment from 'moment';
import { Observable, Subscription, Subject, timer } from 'rxjs';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Popup } from '../../Popup';
import { ControllerService } from '../../controller/controller.service';
import { Controller } from '../../models/controller';
import { AuthenticationService } from '../../login/authentication.service';
import {
  debounceTime,
  distinctUntilChanged,
  first,
  switchMap,
  tap
} from 'rxjs/operators';
import { GatewayGroupsService } from '../../gateway-groups/gateway-groups.service';
import { InstallerStatusEnum } from '../../shared/emuns/global.enum';
import { LicenseErrorEnum } from '../../shared/emuns/license.enum';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { ConfirmModalComponent } from '../../../app/shared/confirm-modal.component';
import { MatDialog } from '@angular/material/dialog';
import { Title } from '@angular/platform-browser';
import { AccessPolicyService } from '../../create-access-policy/access-policy.service';
import { OtpAuthenticationService } from '../../otp-verification/otp-authentication.service';
import { RolesAndPermissionService } from '../../roles-and-permission/roles-and-permission.service';

@Component({
  selector: 'app-gateway-list',
  templateUrl: './gateway-list.component.html',
  styleUrls: ['./gateway-list.component.css']
})
export class GatewayListComponent implements OnInit {
  showHelpText: boolean = false;
  public downloadInitialtedFor = new Array();
  public totalCount: any = 0;
  public alert: GlobalMessageClass;
  gateways: any = {};
  private gatewaysList = new Array();
  private selectedGateway: Gateway;
  selectedGatewayId: number;
  private activeStatus = true;
  pagination: Pagination = new Pagination();
  installerStatus: Popup = new Popup();
  private hearbeatTimeThresholdInSeconds: number = 120;
  public controllers: Controller[];
  private timerSubscription: Subscription;
  showPasswordConfimation = false;
  currentGatewayHealth;
  showHealth = false;
  isSearched: Boolean = false;

  preserveSearch = new PreserveSearch();
  navigationToEdit: boolean = false;

  loading: Boolean = true;

  pageSize = [10, 25, 50, 100, 200];

  sort: string;
  sortDir: string;

  $searchGateway = new Subject();
  searchKeyword;

  gatewayGroups = [];
  selectedGatewayGroups = [];

  selectAll: boolean = false;
  selectedGateways = new Set();
  selectAllExcept = new Set();
  normalizedSelectedGateways = {};
  showQuickGroup: boolean = false;

  gatwayOS = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    centosserver: 'CentOS Server',
    windows: 'Windows',
    macintosh: 'Macintosh'
  };

  InstallerStatus = InstallerStatusEnum;
  isDarkMode: boolean;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  constructor(
    private router: Router,
    private gatewayService: GatewayService,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private controllerService: ControllerService,
    private authenticationService: AuthenticationService,
    private gatewayGroupsServices: GatewayGroupsService,
    public dialog: MatDialog,
    private titleService: Title,
    private accessService: AccessPolicyService,
    private OtpAuthenticationService: OtpAuthenticationService,
    private rolesPermissionService : RolesAndPermissionService
  ) {
    this.controllers = [];
  }

  subscibeToSearch() {
    this.$searchGateway
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          data.length === 0
            ? (this.isSearched = false)
            : (this.isSearched = true);
          this.loading = true;
          this.searchKeyword = data;
          this.pagination.currentPage = 1;
          return this.gatewayService.getGateways(
            this.pagination.currentPage,
            this.pagination.itemsPerPage,
            'all',
            this.sort,
            this.sortDir,
            this.searchKeyword
          );
        })
      )
      .subscribe({
        next: (gateways: any) => {
          this.loading = false;
          const { recordsTotal } = gateways;
          this.gateways = gateways;
          this.pagination.totalItems = recordsTotal;
          this.selectAll = false;
          this.selectAllExcept = new Set();
          this.changeCheckStatusOfAllClients();
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  cancelStatusPopup() {
    this.installerStatus.show = false;
  }

  statusClicked(gateway) {
    this.currentGatewayHealth = gateway.GatewayHealth
      ? gateway.GatewayHealth
      : {};

    this.selectedGatewayId = gateway.id;
    this.showHealth = true;
  }

  updateCurrentGateway() {
    this.gateways.data.map((gateway: any) => {
      const { id, GatewayHealth } = gateway;
      if (this.currentGatewayHealth.gatewayId === id) {
        this.currentGatewayHealth = GatewayHealth ? GatewayHealth : {};
      }
    });
    this.changeCheckStatusOfAllClients();
  }

  cancelHealth(event) {
    this.showHealth = false;
  }

  private onStatus(gateway: Gateway): void {
    // this.selectedGateway = gateway;
    this.installerStatus.heading = 'Installer Status';
    this.installerStatus.body = gateway.installer_status;
    this.installerStatus.action = 'Confirm';
    this.installerStatus.show = true;
    setTimeout(() => {
      this.cancelStatusPopup();
    }, 2000);
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'));
    this.getCRUDdata(25);
    this.titleService.setTitle('Infrastructure | Gateways');
    let gatewaySearch = this.preserveSearch.getSearch('gatewaySearch');
    if (gatewaySearch) {
      this.pagination = gatewaySearch.pagination;
      this.searchKeyword = gatewaySearch.searchKeyword;
    }
    this.refreshData();
    this.subscibeToSearch();
    // this.getGatewayGroups()
  }
  
  getCRUDdata(privId){
    this.rolesPermissionService.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false
      this.readPermission = res?.read_permission === 1 ? true : false
      this.updatePermission = res?.update_permission === 1 ? true : false
      this.deletePermission = res?.delete_permission === 1 ? true : false
    });
  }

  getGatewayGroups() {
    this.loading = true;
    this.gatewayGroupsServices.getAvailableGatewaysGroups().subscribe(
      result => {
        let gatewayParams: any = this.route.snapshot.queryParamMap.getAll(
          'gatewayGroup'
        );
        let parameters = [];
        if (gatewayParams && gatewayParams.length > 0) {
          gatewayParams.forEach(param => {
            if (param.startsWith('[') && param.endsWith(']')) {
              parameters = JSON.parse(param);
            } else {
              parameters.push(parseInt(param));
            }
          });
        }
        this.gatewayGroups = result;
        this.selectedGatewayGroups = parameters.filter(selected => {
          return (
            this.gatewayGroups.findIndex(group => group.id === selected) > -1
          );
        });
      },
      null,
      () => {
        this.getPage(this.pagination.currentPage);
      }
    );
  }

  clearfun() {
    this.searchKeyword = '';
    this.getGatewayGroups();
  }

  handleGatewayChange() {
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  public ngOnDestroy(): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    if (!this.navigationToEdit) {
      this.preserveSearch.saveSearch('gatewaySearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  private onDownload(gateway: Gateway): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to request a new Gateway installer and download it?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedGateway = gateway;
        this.gatewayService.download(gateway);
      }
    });
  }

  private subscribeToData(): void {
    this.timerSubscription = timer(10000)
      .pipe(first())
      .subscribe(() => this.refreshData());
  }

  public refreshData(number = null): void {
    if (this.timerSubscription) {
      this.timerSubscription.unsubscribe();
    }
    if (number != null) {
      this.pagination.currentPage = number;
    }
    this.getPage(this.pagination.currentPage);
    this.subscribeToData();
  }

  private initiateDownload(gateways) {
    var index = -1;
    gateways.forEach(gateway => {
      if (this.downloadInitialtedFor.includes(gateway.id)) {
        if (
          gateway.installer_status === InstallerStatusEnum.New &&
          gateway.isDownloadPackage === true &&
          gateway.packageDownloaded === false
        ) {
          index = this.downloadInitialtedFor.indexOf(gateway.id);
          if (index > -1) {
            this.downloadInitialtedFor.splice(index, 1);
          }
          window.location.assign(gateway.onPrenDownloadLink);
        }
      }
    });
  }

  getPage(page: number) {
    this.loading = true;
    this.controllerService
      .getAllControllers()
      .pipe(
        tap((res: any) => {
          this.controllers = res.data;
        })
      )
      .subscribe();
    this.pagination.currentPage = page;
    this.gatewayService
      .getGateways(
        this.pagination.currentPage,
        this.pagination.itemsPerPage,
        'all',
        this.sort,
        this.sortDir,
        this.searchKeyword,
        this.selectedGatewayGroups,
        '',
        '',
        false,
        false
      )
      .pipe(
        tap((res: any) => {
          this.loading = false;
          this.computeOnlineStatus(res.data || []);
          this.initiateDownload(res.data || []);
          this.gateways = res;
          this.changeCheckStatusOfAllClients();
          this.totalCount = res.recordsTotal;
          this.pagination.totalItems = res.recordsTotal;
          this.pagination.currentPage = page;
          this.updateCurrentGateway();
        })
      )
      .subscribe({
        next: next => {},
        error: err => {
          this.loading = false;
          if (err.status === ErrorCodeEnum.Forbidden) {
            if (err._body == LicenseErrorEnum.Invalid) {
              this.authenticationService.showLicenseOnly(true);
              this.router.navigate(['/license']);
            } else {
              this.alert = {
                closed: false,
                type: 'danger',
                message: 'You are not authorized to access this page.'
              };
              this.globalMessageService.pushAlert(this.alert);
            }
          }
        }
      });
  }

  private computeOnlineStatus(gateways) {
    let heartBeatSentAt;
    let allowedTime: moment.Moment = moment().subtract(
      this.hearbeatTimeThresholdInSeconds,
      'seconds'
    );
    let diffInSeconds: number;
    let heartBeat;

    gateways.forEach(gateway => {
      heartBeat = gateway.heart_beat;
      heartBeatSentAt = moment(gateway.heart_beat_sent_at);
      diffInSeconds = heartBeatSentAt.diff(allowedTime, 'second');

      if (heartBeat && gateway.is_unique) {
        if (diffInSeconds >= 0) {
          gateway.is_online = true;
        } else {
          gateway.is_online = false;
        }
      } else {
        gateway.is_online = false;
      }
    });
  }

  private onSelect(gateway: Gateway): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'edit';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
        (res: any) => {
          if (res == true) {
            let gatewaySearch = {
              pagination: this.pagination,
              searchKeyword: this.searchKeyword
            };
            this.preserveSearch.saveSearch('gatewaySearch', gatewaySearch);
            this.navigationToEdit = true;
            this.selectedGateway = gateway;
            this.router.navigate(['/gateways', this.selectedGateway.id]);
            this.OtpAuthenticationService.setOtpStatusValueEdit(false);
          }
        }
      );
    } else {
      let gatewaySearch = {
        pagination: this.pagination,
        searchKeyword: this.searchKeyword
      };
      this.preserveSearch.saveSearch('gatewaySearch', gatewaySearch);
      this.navigationToEdit = true;
      this.selectedGateway = gateway;
      this.router.navigate(['/gateways', this.selectedGateway.id]);
    }
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  submitVerificationPassword(event) {
    this.accessService
      .verifyPolicies('', '', [], '', this.selectedGateway.id)
      .subscribe(res => {
        if (res) {
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Remove gateway from Entitlement(s) first',
            sticky: false
          });
        } else {
          this.gatewayService
            .delete_gateway(this.selectedGateway.id, event)
            .subscribe(response => {
              this.showPasswordConfimation = false;
              if (response.success) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'Gateway has been deleted successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
                this.selectedGateways.clear();
              }
            });
        }
      });
  }

  private onDelete(gateway: Gateway): void {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'delete';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
        (res: any) => {
          if (res == true) {
            this.showPasswordConfimation = false;

            this.accessService
              .verifyPolicies('', '', [], '', gateway.id)
              .subscribe(res => {
                if (res) {
                  this.globalMessageService.pushAlert({
                    closed: false,
                    type: 'danger',
                    message: 'Remove gateway from Entitlement(s) first',
                    sticky: false
                  });
                } else {
                  this.gatewayService
                    .delete_gateway_by_otp(gateway.id)
                    .subscribe(response => {
                      this.showPasswordConfimation = false;
                      if (response.success) {
                        this.alert = {
                          closed: false,
                          type: 'success',
                          message: 'Gateway has been deleted successfully.'
                        };
                        this.globalMessageService.pushAlert(this.alert);
                        this.getPage(this.pagination.currentPage);
                        this.selectedGateways.clear();
                      }
                    });
                }
              });

            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueDel(false);
          }
        }
      );
    } else {
      this.selectedGateway = gateway;
      this.showPasswordConfimation = true;
    }
  }

  onClone(gateway: Gateway): void {
    this.gatewayService.clone_gateway(gateway).subscribe(response => {
      if (response.success) {
        this.alert = {
          closed: false,
          type: 'success',
          message: 'Gateway has been cloned successfully.'
        };
        this.globalMessageService.pushAlert(this.alert);
        this.getPage(this.pagination.currentPage);
      }
    });
  }

  add_gateway(): void {
    this.router.navigate(['/create-gateway']);
  }

  toggle(gateway: Gateway, index) {
    this.gatewayService.change_status(gateway).subscribe(response => {
      if (response.success) {
        this.gateways.data[index].is_active = !this.gateways.data[index]
          .is_active;
        if (this.gateways[index].is_active) {
          this.alert = {
            closed: false,
            type: 'warning',
            message:
              'Gateway activated successfully but you have to start it manually.'
          };
          this.globalMessageService.pushAlert(this.alert);
        } else {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'Gateway deactivated successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
        }
      }
    });
  }

  private onResend(gateway: Gateway): void {
    const dialogRef = this.dialog.open(ConfirmModalComponent, {
      panelClass: this.isDarkMode ? 'dark-theme' : '',
      data: {
        title: 'Confirmation',
        message: `Are you sure you want to request a new Gateway installer ?`
      }
    });

    dialogRef.afterClosed().subscribe(result => {
      if (result) {
        this.selectedGateway = gateway;

        this.gatewayService
          .generateGatewayInstaller(gateway.id, 0)
          .pipe(
            tap((response: any) => {
              if (response.status == true) {
                this.alert = {
                  closed: false,
                  type: 'success',
                  message: 'Gateway installer has been sent successfully.'
                };
                this.globalMessageService.pushAlert(this.alert);
                this.getPage(this.pagination.currentPage);
              } else {
                this.alert = {
                  closed: false,
                  type: 'danger',
                  message: 'Failed to send the Gateway Installer.'
                };
                this.globalMessageService.pushAlert(this.alert);
              }
            })
          )
          .subscribe({
            error: err => {
              this.loading = false;
            },
            complete: () => {}
          });
      }
    });
  }

  pageSizeChange(size) {
    this.pagination.itemsPerPage = +size;
    this.pagination.totalItems = 0;
    this.pagination.currentPage = 1;
    this.getPage(this.pagination.currentPage);
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getPage(this.pagination.currentPage);
  }

  goToDetailPage(gateway) {
    this.router.navigate([`/gateway/${gateway.id}`]);
  }

  changeCheckStatusOfAllClients() {
    this.gateways.data = this.gateways.data.map(gateway => {
      return {
        ...gateway,
        isChecked:
          (this.selectAll && !this.selectAllExcept.has(gateway.id)) ||
          this.selectedGateways.has(gateway.id)
      };
    });
  }

  changeCheckBox(client) {
    const { isChecked, id } = client;
    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
      } else {
        this.selectAllExcept.add(id);
      }
    } else {
      if (!isChecked) {
        delete this.normalizedSelectedGateways[id];
        this.selectedGateways.delete(id);
      } else {
        this.normalizedSelectedGateways[id] = client;
        this.selectedGateways.add(id);
      }
    }
  }

  selectAllAction() {
    this.normalizedSelectedGateways = {};
    this.selectAllExcept.clear();

    if (!this.selectAll) {
      this.selectAllExcept.clear();
      this.selectedGateways.clear();
    } else {
      this.selectedGateways.clear();
    }
    this.changeCheckStatusOfAllClients();
  }

  async handleAddToGroup() {
    if (
      this.selectAll &&
      (this.selectAllExcept.size < 1 || this.searchKeyword)
    ) {
      this.normalizedSelectedGateways = {};
    }
    if (
      this.selectAll &&
      (this.selectAllExcept.size > 0 || this.searchKeyword)
    ) {
      const response = await this.gatewayService
        .getQuickGateways(this.searchKeyword, Array.from(this.selectAllExcept))
        .toPromise();
      const gateways = response.data;
      if (gateways) {
        let normalize = {};
        gateways.forEach(gateway => {
          normalize[gateway.id] = gateway;
        });
        this.normalizedSelectedGateways = normalize;
      }
    }
    this.showQuickGroup = true;
  }

  closeQuickGroup() {
    this.showQuickGroup = false;
  }

  successCloseQuickGroup() {
    this.showQuickGroup = false;
    this.selectAll = false;
    this.selectedGateways = new Set();
    this.normalizedSelectedGateways = {};
    this.changeCheckStatusOfAllClients();
  }

  isFilterApplied() {
    return this.selectAllExcept.size > 0 || this.searchKeyword;
  }

  cancelOtpVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitOtpVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }

  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
}
