import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { PaginationResponse } from '../models/pagination-response';
import { Gateway } from '../models/gateway';
import { Observable } from 'rxjs';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { map } from 'rxjs/operators';

@Injectable()
export class GatewayService {
  private gatewaysUrl = 'http://localhost:3000/gateways';
  private gatewaysApiUrl = '/gateways';
  private serviceUrl = '/services'
  private gatewaysChangeStatusApiUrl = '/gateways/status';
  private gateways: Gateway[];
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };

  constructor(
    private authShortlivedCookie: AuthShortlivedCookieService,
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.gatewaysApiUrl = this.apiBaseUrl + this.gatewaysApiUrl;
    this.gatewaysChangeStatusApiUrl =
      this.apiBaseUrl + this.gatewaysChangeStatusApiUrl;
  }

  /** GET from the server */

  getGateways(
    page: number = 1,
    pageSize: number = 10,
    action: string = 'active',
    sort: string = 'id',
    dir: string = 'desc',
    searchKey = '',
    gatewayGroups = [],
    protocol = '',
    buildOs = '',
    all = false,
    unique = true,
    except = []
  ): any {
    const skip = JSON.stringify(except);
    let query = `?limit=${pageSize}&offset=${page -
      1}&action=${action}&sort=${sort}&dir=${dir}&searchKey=${searchKey}&gatewayGroups=${JSON.stringify(
      gatewayGroups
    )}&protocol=${protocol}&buildOs=${buildOs}&all=${all}&unique=${unique}&except=${skip}`;

    return this.authHttp
      .get(this.gatewaysApiUrl + query)
      .pipe(map(response => response));
  }

  getGatewayServices(id) {
    return this.authHttp
      .get(this.gatewaysApiUrl + `/${id}` + this.serviceUrl)
      // .pipe(map(response => response));
  }

  getQuickGateways(searchKey = '', except = []): any {
    const skip = JSON.stringify(except);
    let query = `?searchKey=${searchKey}&except=${skip}`;

    return this.authHttp
      .get(this.gatewaysApiUrl + '/quick' + query)
      .pipe(map(response => response));
  }

  getAvailabeGateways(force: boolean): any {
    return this.authHttp
      .get(this.gatewaysApiUrl + '/available?force=' + force)
      .pipe(map(response => response));
  }

  getcontrollerGraphs(gatewayId) {
    return this.authHttp
      .get(`${this.gatewaysApiUrl}/health/graph/${gatewayId}`)
      .pipe(map(response => response));
  }

  add_gateway(gateway: Gateway): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';

    if (gateway.id > 0 && typeof gateway.id != 'undefined') {
      return this.authHttp
        .put(
          this.gatewaysApiUrl + '/' + gateway.id,
          { gateway: gateway },
          options
        )
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      return this.authHttp
        .post(this.gatewaysApiUrl, { gateway: gateway }, options)
        .pipe(map(response => response));
    }
  }

  change_status(gateway: Gateway): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(this.gatewaysChangeStatusApiUrl, { gateway: gateway }, options)
      .pipe(map(response => response));
  }

  delete_gateway(id: number, verificationPassword): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(this.gatewaysApiUrl + queryString, this.httpOptions)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  delete_gateway_by_otp(id: number): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/otp/' + id;
    }
    return this.authHttp
      .delete(this.gatewaysApiUrl + queryString)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }


  clone_gateway(gateway: Gateway): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };

    return this.authHttp
      .post(this.gatewaysApiUrl + '/clone', { gateway: gateway }, options)
      .pipe(map((response: any) => response));
    //.catch(this.handleError);
  }

  getGatewayGraphs(gatewayId: number, interval: number = 1) {
    return this.authHttp
      .get(
        `${this.gatewaysApiUrl}/health/graph/${gatewayId}?interval=${interval}`
      )
      .pipe(map(response => response));
  }

  getgatewaydetail(id): Observable<Gateway | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }
    return this.authHttp
      .get(this.gatewaysApiUrl + queryString)
      .pipe(map(response => response));
  }

  getGatewayOsAndProtocols() {
    const url = '/os/protocol';
    return this.authHttp
      .get(this.gatewaysApiUrl + url)
      .pipe(map(response => response));
  }

  getGatewaysHealth(offset, limit) {
    const url = `/health/summary?offset=${offset}&limit=${limit}`;
    return this.authHttp
      .get(this.gatewaysApiUrl + url)
      .pipe(map(response => response));
  }

  generateGatewayInstaller(id: number, isDownload = 0): any {
    let queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    if (isDownload == 1) {
      queryString += '/' + 1;
    } else {
      queryString += '/' + 0;
    }
    return this.authHttp
      .get(`${this.gatewaysApiUrl}/generate/service${queryString}`)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  download(gateway) {
    const { id } = gateway;

    this.authShortlivedCookie.setFor(5);

    window.location.href = this.apiBaseUrl + `/gateways/download/service/${id}`;
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
