import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { GroupService } from '../group.service';
import { ClientStatusEnum, ClientTypeEnum, DeviceOSEnum } from '../../shared/emuns/client.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-group-detail',
  templateUrl: './group-detail.component.html',
  styleUrls: ['./group-detail.component.css']
})
export class GroupDetailComponent implements OnInit {

  pagination: Pagination = new Pagination()
  clients = []
  searchTerm = ''
  sort = 'Name'
  sortDir = 'asc'
  loading: boolean = false
  pageSize = [10,25,50,100,200];

  groupId;
  groupName;

  foundGroup: boolean = false;
  showConfirm: boolean = false;
  currentSelectedType;
  currentFilterOs;

  ClientStatus = ClientStatusEnum;
  ClientType = ClientTypeEnum;
  DeviceOS = DeviceOSEnum;

  types = [ClientTypeEnum.Server, ClientTypeEnum.Client]

  quickFiltersOs = [
    {
      isChecked: false,
      title: 'Ubuntu Desktop',
      value: DeviceOSEnum.Ubuntu
    },
    {
      isChecked: false,
      title: 'Ubuntu Server',
      value: DeviceOSEnum.UbuntuServer
    },
    {
      isChecked: false,
      title: 'Ubuntu 18 Desktop',
      value: DeviceOSEnum.Ubuntu18
    },
    {
      isChecked: false,
      title: 'Ubuntu 18 Server',
      value: DeviceOSEnum.UbuntuServer18
    },
    {
      isChecked: false,
      title: 'Ubuntu 20 Server (ARMv8)',
      value: DeviceOSEnum.UbuntuServer20armv8
    },
    {
      isChecked: false,
      title: 'Windows',
      value: DeviceOSEnum.Windows
    },
    {
      isChecked: false,
      title: 'Windows Server',
      value: DeviceOSEnum.WindowsServer
    },
    {
      isChecked: false,
      title: 'Android',
      value: DeviceOSEnum.Android
    },
    {
      isChecked: false,
      title: 'iOS',
      value: DeviceOSEnum.IOS
    },
    {
      isChecked: false,
      title: 'Macintosh',
      value: DeviceOSEnum.Macintosh
    }
  ];

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private groupService: GroupService,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) { }

  ngOnInit() {
    const groupId = this.route.snapshot.params.id;
    this.groupId = groupId
    if(groupId)
    {
      this.getGroupDetail()
    }
    this.titleService.setTitle('Client Management | Groups | ' + this.groupName);
  }

  handleSearch() {
    this.pagination.currentPage = 1;
    this.getGroupDetail()
  }

  pageSizeChange() {
    this.pagination.currentPage = 1;
    this.getGroupDetail()
  }

  pageChange(page) {
    this.pagination.currentPage  = page
    this.getGroupDetail()
  }

  sortClients(sort) {
    this.sort = sort
    this.sortDir = this.sortDir == 'desc' ? 'asc' : 'desc';
    this.pagination.currentPage = 1
    this.getGroupDetail()
  }
  
  goBack() {
    this.router.navigate(['/groups'])
  }

  editHandler() {
    this.router.navigate([`/create-group/${this.groupId}`])
  }

  showConfirmation() {
    this.showConfirm = true
  }

  cancelHandler() {
    this.showConfirm = false
  }

  deleteHandler() {
    this.showConfirm = false
    this.groupService.deleteGroup(this.groupId).subscribe(
      response => {
        this.globalMessageService.pushAlert({
          message: `Group deleted`,
          type: 'success'
        });
        this.goBack()
      },
      err => {
        this.showConfirm = false;
      }
    );

  }

  
  getGroupDetail() {
    this.loading = true
    this
    .groupService
    .getGroupClientsWithPaginatin(
      this.groupId, 
      this.pagination.currentPage -1,
      this.pagination.itemsPerPage,
      this.searchTerm,
      this.sort,
      this.sortDir,
      this.currentSelectedType? this.currentSelectedType: this.currentSelectedType,
      this.currentFilterOs
    ).subscribe(
      response => {
        this.clients = response.data
        this.pagination.totalItems = response.total
        this.groupName = response.name
        this.loading = false
        this.foundGroup = true
      },
      err => {
        this.loading = false
      }
    )
  }

  currentClientTypeChange() {
    this.pagination.currentPage = 1;
    this.getGroupDetail()
  }

  currentOsFilterChanged() {
    this.pagination.currentPage = 1;
    this.getGroupDetail()
  }

  navigate(groupName:string){}
}
