import { Component, OnInit } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Subscription } from 'rxjs';
import { IdpService } from '../idp.service';
import { ServiceProvider } from '../../models/service_provider';
import { DragulaService } from 'ng2-dragula';
import { Pagination } from '../../models/pagination';
import { PreserveSearch } from '../../models/preserve-search';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { ContextHelpEnum } from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-idp',
  templateUrl: './create-idp.component.html',
  styleUrls: ['./create-idp.component.css']
})
export class CreateIdpComponent implements OnInit {
  public alert: GlobalMessageClass;
  idp = new ServiceProvider();
  pagination: Pagination = new Pagination();
  private Search: PreserveSearch = new PreserveSearch();
  public pageTitle: string = 'Add Service Provider';

  // //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  // //Contextual Help
  loading: Boolean = false;
  disableSaveButton = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private idpService: IdpService,
    private dragulaService: DragulaService,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) {
    this.contextualHelp = ['', ''];
    const bag: any = this.dragulaService.find('service-bag');
    if (bag !== undefined) this.dragulaService.destroy('service-bag');
    dragulaService.drop.subscribe(value => {
      // this.onDrop(value.slice(1));
    });
    dragulaService.setOptions('service-bag', {
      moves: function(el: any, container: any, handle: any): any {
        if (el.classList.contains('disableListItem')) {
          return false;
        }
        return true;
      }
    });
    // dragulaService.drop().subscribe(({ name, el, target, source, sibling }) => {
    //   //this.onDrop([el, target, source, sibling]);
    // });
    // dragulaService.createGroup('service-bag', {
    //   moves: function(el: any, container: any, handle: any): any {
    //     if (el.classList.contains('disableListItem')) {
    //       return false;
    //     }
    //     return true;
    //   }
    // });
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.Name) {
      this.contextualHelp = [
        'Name',
        'Name of the IDP. Please consult service provider.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.spId) {
      this.contextualHelp = ['spId', 'Service Provider ID.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.assertionUrl) {
      this.contextualHelp = ['assertionUrl', 'Assertion URL.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.eId) {
      this.contextualHelp = ['eId', 'Entity ID.'];
      this.showContext = true;
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  getPage(page: number) {
    let ref = this;
    this.pagination.currentPage = page;

    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.pageTitle = 'Edit IDP';
      this.loading = true;
      this.idpService.getIdpDetail(this.route.params['_value']['id']).subscribe(
        idp => {
          this.loading = false;
          if (!idp[0]) {
            this.alert = {
              closed: false,
              type: 'danger',
              message: 'No IDP found with the specified ID.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.router.navigate(['/idp']);
          } else {
            let g = idp[0];
            this.idp = g;
          }
        },
        err => {
          this.loading = false;
        }
      );
    } else {
      this.pageTitle = 'Add Service Provider';
    }
  }

  ngOnInit() {
    this.disableSaveButton = false;
    let idpSearch = this.Search.getSearch('idpSearch');
    if (idpSearch) {
      this.pagination.currentPage = idpSearch.current_page;
    }
    this.getPage(this.pagination.currentPage);
    this.titleService.setTitle('Infrastructure | IDP Management | ' + this.pageTitle);
  }

  onsubmit() {
    this.loading = true;
    this.disableSaveButton = true;
    this.idpService.addIdp(this.idp).subscribe(
      response => {
        this.loading = false;
        if (response.success) {
          this.alert = {
            closed: false,
            type: 'success',
            message: 'IDP has been added/updated successfully.'
          };
          this.globalMessageService.pushAlert(this.alert);
          this.router.navigate(['/idp']);
        } else {
          // this.alert = {
          //   closed: false,
          //   type: 'danger',
          //   message: 'Failed to add/update IDP.'
          // };
          // this.globalMessageService.pushAlert(this.alert);
        }
      },
      error => {
        this.disableSaveButton = false;
        this.loading = false;
        // console.log(error);
        // this.alert = {
        //   closed: false,
        //   type: 'danger',
        //   message:
        //     error._body || 'Something went wrong in adding/editing of idp.'
        // };
        // this.globalMessageService.pushAlert(this.alert);
      }
    );
  }

  onCancel(): void {
    this.router.navigate(['/idp']);
  }
}
