import { Component, OnInit } from '@angular/core';
import {InfrastructureReportsService} from './reporting.service';
import { Pagination } from '../models/pagination';
import * as moment from 'moment';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-services-reports',
  templateUrl: './infrastructure-reports.component.html',
  styleUrls: ['./infrastructure-reports.component.css']
})
export class InfrastructureReportsComponent implements OnInit {
  today: Date;
  ReportsOfAllControllers:any = [];
  controllersActive: number;
  gatewaysActive: number;
  ReportsOfAllGateways:any = [];
  servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  gatewaysDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceSortDir: string = 'asc'
  serviceSort: string;
  private hearbeatTimeThresholdInSeconds: number = 120;
  gatewaysSortDir: string = 'asc'
  gatewaysSort: string;
  last_sync_time: string;
  directories_count: number;

  summaryOfAllServicesPagination: Pagination = new Pagination();
  summaryOfAllGatewaysPagination: Pagination = new Pagination();
  reportsOfAllControllersLoading:Boolean = true;
  reportsOfAllGatewaysLoading:Boolean = true;
  pageSizeSummaryOfServices = [10,25,50,100,200];
  pageSizeSummaryOfGateways = [10,25,50,100,200];

  constructor (
    private InfrastructureReportsService:InfrastructureReportsService,
    private titleService: Title
    ) {
    this.today = new Date();
  }

  ngOnInit() {
    this.titleService.setTitle('Reports | Infrastructure Reports');
    this.summaryOfAllServicesPagination.id='controllers_reports';
    this.summaryOfAllGatewaysPagination.id='gateways_reports';
    this.getReportsOfAllControllers(this.summaryOfAllServicesPagination);
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination);
  }
  
  ngAfterViewInit() {
  }

  ngOnDestroy() {
  }


  convertBytesToMiB(bytes: number): string {
    const MiB = bytes / (1024 * 1024);
    return MiB.toFixed(2) + ' MiB';
  }

  getReportsOfAllControllers(pagination: Pagination){
    this.reportsOfAllControllersLoading = true;
    this.InfrastructureReportsService.
    getControllersReports(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.serviceSortDir,
      this.serviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.result.recordsTotal;
        let allowedTime: moment.Moment = moment().subtract(
          this.hearbeatTimeThresholdInSeconds,
          'seconds'
        );
        this.ReportsOfAllControllers = summary.result.data && summary.result.data.map(controller => {
          let heartBeat = controller.heart_beat;
          let heartBeatSentAt = moment(controller.heart_beat_sent_at);
          let diffInSeconds = heartBeatSentAt.diff(allowedTime, 'second');
          
          if (heartBeat) {
            if (diffInSeconds >= 0) {
              controller.is_online = true;
            } else {
              controller.is_online = false;
            }
          } else {
            controller.is_online = false;
          }

          controller.ControllerHealth = this.convertBytesToMiB(
            controller.ControllerHealth || 0
          );
          return controller
        });
        this.controllersActive = summary.controllersActive;
        this.reportsOfAllControllersLoading = false;
      },
      error: err=>{
        this.reportsOfAllControllersLoading = false;
      }
    })
  }

  getReportsOfAllGateways(pagination: Pagination){
    this.reportsOfAllGatewaysLoading = true;
    this.InfrastructureReportsService.
    getGatewaysReports(
      pagination,
      this.gatewaysDaterangepicker[0],
      this.gatewaysDaterangepicker[1],
      this.gatewaysSortDir,
      this.gatewaysSort
    )
    .subscribe({
      next: summary => {
        let heartBeatSentAt;
        let allowedTime: moment.Moment = moment().subtract(
          this.hearbeatTimeThresholdInSeconds,
          'seconds'
        );
        let diffInSeconds: number;
        let heartBeat;

        this.summaryOfAllGatewaysPagination.totalItems = summary.result.recordsTotal;
        this.ReportsOfAllGateways = summary.result;
        this.last_sync_time = summary.last_sync_time.last_ldap_sync_end_time;
        this.directories_count = summary.directories_count;
        this.ReportsOfAllGateways.data = this.ReportsOfAllGateways && this.ReportsOfAllGateways.data && this.ReportsOfAllGateways.data.map(gateway => {
          heartBeat = gateway.heart_beat;
          heartBeatSentAt = moment(gateway.heart_beat_sent_at);
          diffInSeconds = heartBeatSentAt.diff(allowedTime, 'second');

          if (heartBeat && gateway.is_unique) {
            if (diffInSeconds >= 0) {
              gateway.is_online = true;
            } else {
              gateway.is_online = false;
            }
          } else {
            gateway.is_online = false;
          }
          gateway.GatewayHealth = this.convertBytesToMiB(
            gateway.GatewayHealth || 0
          );
          return gateway;
        });
        this.gatewaysActive = summary.gatewaysActive;
        this.reportsOfAllGatewaysLoading = false;
      },
      error: err=>{
        this.reportsOfAllGatewaysLoading = false;
      }
    })
  }


  clearDate (type) {
    if(type === APIDataTypeEnum.Service) {
      this.servicesDaterangepicker = [];
      this.serviceDateChange();
    }else if (type === 'gateways') {
      this.gatewaysDaterangepicker = [];
      this.gatewaysDateChange();
    }
  }

  serviceDateChange(){
    if(!this.servicesDaterangepicker)
    this.servicesDaterangepicker = [];
    this.summaryOfAllServicesPagination.currentPage = 1;
    this.getReportsOfAllControllers(this.summaryOfAllServicesPagination);
  }

  gatewaysDateChange(){
    if(!this.gatewaysDaterangepicker)
    this.gatewaysDaterangepicker = [];
    this.summaryOfAllGatewaysPagination.currentPage = 1;
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination);
  }

  pageChangeSummaryOfAllServices(page:any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getReportsOfAllControllers(this.summaryOfAllServicesPagination);
  }

  pageChangeSummaryOfAllGateways(page:any) {
    this.summaryOfAllGatewaysPagination.currentPage = page;
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination);
  }


  sortServiceRecord(term: string){
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;

    this.getReportsOfAllControllers(this.summaryOfAllServicesPagination)
  }

  sortGatewaysRecord(term: string){
    this.gatewaysSortDir = this.gatewaysSortDir == 'asc' ? 'desc' : 'asc';
    this.gatewaysSort = term;
    
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination)
  }

  exportCSV(value){
    this.InfrastructureReportsService.exportCSV(value)
  }

}




