import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { ConfigService } from '../services/config.service';
import * as moment from 'moment';
import { AuthShortlivedCookieService } from '../auth-shortlived-cookie.service';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

interface date {
  year: String;
  month: String;
  day: string;
}
@Injectable()
export class InfrastructureReportsService {
  private apiBaseUrl: string;
  private reportingApiUrl: string = '/reports';

  constructor(
    private authHttp: AuthHttp,
    private congigService: ConfigService,
    private authShortlivedCookie: AuthShortlivedCookieService
  ) {
    this.apiBaseUrl = this.congigService.getConfiguration().apiBaseUrl;
    this.reportingApiUrl = this.apiBaseUrl + this.reportingApiUrl;
  }

  getControllersReports(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _startDate = '';
    let _endDate = '';

    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/infrastructure/controllers/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map(response => response));
  }

  getGatewaysReports(
    pagination: Pagination,
    startDate: Date,
    endDate: Date,
    dir: string,
    sort: string
  ): any {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _startDate = '';
    let _endDate = '';

    if (startDate) {
      _startDate = moment(startDate).format('YYYY-MM-DD');
    }
    if (endDate) {
      _endDate = moment(endDate).format('YYYY-MM-DD');
    }

    return this.authHttp
      .get(
        this.reportingApiUrl +
          '/infrastructure/gateways/reports?offset=' +
          currentPage +
          '&limit=' +
          itemsPerPage +
          '&startDate=' +
          _startDate +
          '&endDate=' +
          _endDate +
          '&dir=' +
          dir +
          '&sort=' +
          sort
      )
      .pipe(map(response => response));
  }

  exportCSV(value) {
    this.authShortlivedCookie.setFor(5);
    window.location.href = `${this.reportingApiUrl}/infrastructure/export-csv?value=${value}`;
  }
}
