import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';

import { AuthenticationService } from '../login/authentication.service';
import { PaginationResponse } from '../models/pagination-response';
import { Ldap } from '../models/ldap';
import { LoginResponse } from '../models/login_response';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';

@Injectable()
export class LdapService {
  private ldapsUrl = 'http://localhost:3000/ldaps';
  private ldapsApiUrl = '/ldaps';
  private ldapsTestUrl = '/ldapmanagement/test/';
  private checkvalidity = '/ldaps/duplicationCheck';
  private ldaps: Ldap[];
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.ldapsApiUrl = this.apiBaseUrl + this.ldapsApiUrl;
    this.checkvalidity = this.apiBaseUrl + this.checkvalidity;
    this.ldapsTestUrl = this.apiBaseUrl + this.ldapsTestUrl;
  }

  /** GET ldaps from the server */

  getLdaps(page: number = 1, pageSize: number = 10, searchKey = ''): any {
    let query = `?limit=${pageSize}&offset=${page - 1}&searchKey=${searchKey}`;
    return this.authHttp
      .get(this.ldapsApiUrl + query)
      .pipe(map(response => response));
  }

  add_user(ldap: Ldap): Observable<LoginResponse | any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';
    if (ldap.id > 0 && typeof ldap.id != 'undefined') {
      queryString = '/' + ldap.id;
      return this.authHttp
        .put(this.ldapsApiUrl + queryString, { ldap: ldap }, options)
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      return this.authHttp
        .post(this.ldapsApiUrl, { ldap: ldap }, options)
        .pipe(map(response => response));
    }
  }

  test_settings(ldap: Ldap): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';
    return this.authHttp
      .post(this.ldapsTestUrl, { ldap: ldap }, options)
      .pipe(map(response => response));
  }

  delete_ldap(
    id: number,
    verificationPassword
  ): Observable<LoginResponse | any> {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(this.ldapsApiUrl + queryString, this.httpOptions)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  delete_ldap_by_otp(id: number): Observable<LoginResponse | any> {
    var queryString = '';

    if (id > 0) {
      queryString = '/otp/' + id;
    }

    return this.authHttp
      .delete(this.ldapsApiUrl + queryString)
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  updateldap(ldap: Ldap): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(this.ldapsUrl, ldap, options)
      .pipe(map(response => response));
  }

  getldapdetail(id): Observable<Ldap | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }

    return this.authHttp
      .get(this.ldapsApiUrl + queryString)
      .pipe(map(response => response));
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
