import {
  AfterViewInit,
  Component,
  ElementRef,
  OnInit,
  Renderer2,
  ViewChild
} from '@angular/core';
import { Router } from '@angular/router';
import { Pagination } from '../models/pagination';
import { LocalUserService } from '../create-local-user/local-user.service';
import { OrganizationService } from '../organization-list-view/organization.service';
import { LocalUser } from '../models/LocalUser';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { BehaviorSubject, Observable, Subject, from } from 'rxjs';
import { PreserveSearch } from '../models/preserve-search';
import {
  catchError,
  debounceTime,
  distinctUntilChanged,
  map,
  startWith,
  switchMap
} from 'rxjs/operators';
import { IdentityProvierEnum } from '../shared/emuns/user.enum';
import { Title } from '@angular/platform-browser';
import { FormControl, NgForm } from '@angular/forms';
import { AccessPolicyService } from '../create-access-policy/access-policy.service';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
import { Subscription } from 'rxjs';
import { OtpAuthenticationService } from '../otp-verification/otp-authentication.service';

export interface User {
  name: string;
}

@Component({
  selector: 'app-local-users-list',
  templateUrl: './local-users-list.component.html',
  styleUrls: ['./local-users-list.component.css']
})
export class LocalUsersListComponent implements OnInit, AfterViewInit {
  @ViewChild('input') inputText!: NgForm;
  @ViewChild('event') searchInputEvent: ElementRef;
  myControl = new FormControl();
  hiddenCtrl = new FormControl();
  $searchUsersJQL = new BehaviorSubject<any>({});
  lastValue: string;
  currentText: string;
  isExpression: boolean = false;
  isAnOptionSelected: boolean = false;
  isSearchExecuted: boolean = false;
  isAlreadyClear: boolean = true;
  showDynamicGroupButton: boolean = false;
  rawQuery: string = '';

  departmentsData: string[] = [];
  organizationsData: string[] = [];
  userNamesData: string[] = [];
  namesData: string[] = [];
  emailsData: string[] = [];

  private defaultSelection: string = Action.Field;
  private currentEvent: string;
  private response: ApiResponse[] = [];

  public fieldList: SuggestionDetails = {
    Name: Action.Field,
    Value: [],
    Valid: ['string']
  };
  private operatorList: SuggestionDetails = {
    Name: Action.Operator,
    Value: ['=', '!=', '>', '<', '<=', '>=', '(', ')'],
    Valid: ['string']
  };
  private valueList: SuggestionDetails = {
    Name: Action.Value,
    Value: [],
    Valid: ['string']
  };
  private expressionList: SuggestionDetails = {
    Name: Action.Expression,
    Value: ['And', 'Or'],
    Valid: ['string']
  };

  private operator: string[] = this.operatorList.Value;
  private value: string[] = this.valueList.Value;
  private expression: string[] = this.expressionList.Value;
  isUploading: boolean;
  fileData: { data: string | ArrayBuffer; name: any; type: any; };
  isFileValid: boolean;
  failedUsers: [];
  showFailedUpload: boolean;

  private get field(): string[] {
    return this.fieldList.Value;
  }

  filteredOptions: Observable<string[]>;
  private searchList: SelectedOption[] = [];

  private get selectionList(): SelectionDict[] {
    return [
      { Name: Action.Field, Value: this.field, NextSelection: Action.Operator },
      {
        Name: Action.Operator,
        Value: this.operator,
        NextSelection: Action.Value
      },
      {
        Name: Action.Value,
        Value: this.value,
        NextSelection: Action.Expression
      },
      {
        Name: Action.Expression,
        Value: this.expression,
        NextSelection: Action.Field
      }
    ];
  }

  selectAllExcept: any = new Set();
  selectedUsers: any = new Set();
  adUserSelectedCount: number = 0;
  IdentityProvier = IdentityProvierEnum;
  declarativeSearch: boolean = true;
  simpleSearch: boolean = false;

  // Ended Dynamic Search

  showHelpText: boolean = false;
  pagination = new Pagination();
  localUsers: any = {};
  allLocalUsers: any = {};
  public actionsDropdown: number = 0;
  $searchUsers = new Subject();
  showPasswordConfimation = false;
  showPasswordConfimationBulk = false;
  userTobeDeleted: LocalUser;
  isNameSort: boolean = false;
  searchWord;
  preserveSearch = new PreserveSearch();
  navigationToEditLocalUser: boolean = false;
  isLdapSyncing: boolean = false;
  loading: Boolean = true;

  groupName: string;
  groupDescription: string;

  pageSize = [10, 25, 50, 100, 200];

  identityProvider = ['Active Directory', 'Local User'];
  organizations = [];
  departments = [{ deptId: null, deptName: null }];
  removePagination: boolean = false;

  selectedIdentityProvider: string = '';
  selectedOrganization: string = '';
  selectedDepartment: string = '';

  handleModal = false;
  toggleRuleViewStyle;

  sort: string;
  sortDir: string;
  searchType: string;

  selectAll = false;

  showPasswordConfimationResetPassword: Boolean = false;

  createPermission: boolean = true;
  readPermission: boolean = true;
  updatePermission: boolean = true;
  deletePermission: boolean = true;

  selectedIdsForDeleteCurrent: any = [];
  deleteAllTotal: number = 0;
  deleteCurrentTotal: number = 0;

  isDeleteCurrent: boolean = false;
  isDeleteAll: boolean = false;

  showOtpDialog: boolean = false;
  otptoken: string = '';
  option: string;
  private otpStatusSubscription: Subscription;

  uploadCsvFile = false;

  constructor(
    private router: Router,
    private localUserService: LocalUserService,
    private organizationService: OrganizationService,
    private globalMessageService: GlobalMessageService,
    private titleService: Title,
    private accessService: AccessPolicyService,
    private renderer: Renderer2,
    private roles: RolesAndPermissionService,
    private OtpAuthenticationService: OtpAuthenticationService
  ) {}

  sortByNameHandler() {
    this.isNameSort = !this.isNameSort;
    if (this.searchWord) {
      this.$searchUsers.next({ searchWord: this.searchWord, typing: false });
    } else {
      this.getLocalUsersFromDropDown();
    }
  }

  uploadHandler() {
    this.uploadCsvFile = !this.uploadCsvFile;
  }

  handleFileUpload(event) {
    if (event.target.files && event.target.files.length > 0) {
      let file = event.target.files[0];
      let name = file.name;
      let type = file.type;
      let localName = name.split('.');
      let extension = localName[localName.length - 1];
      if (extension === 'csv' || type === 'text/csv') {
        let fileRedaer = new FileReader();
        fileRedaer.readAsDataURL(file);
        fileRedaer.onload = () => {
          let data = fileRedaer.result;
          this.fileData = {
            data: data,
            name: name,
            type: type
          };
          this.isFileValid = true;
        };
      } else {
        this.isFileValid = false;
        alert('Please select a valid file type, these are allowed types `csv`');
      }
    }
  }

  uploadCsvFileHandler() {
    this.isUploading = true;
    this.localUserService.uploadCsvFile(this.fileData).subscribe(
      (data: any) => {
        
        if (data.errors && data.errors.length > 0) {
          this.failedUsers = data.errors;
          this.showFailedUpload = true;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: data.errors.join(', '),
            sticky: false
          });
        } else {
          this.failedUsers = [];
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: data.message,
            sticky: false
          });
        }
        this.uploadCsvFile = false;
        this.isUploading = false;
        this.getLocalUsers();
      },
      error => {
        console.error('Error:', error);
        this.uploadCsvFile = false;
        this.isUploading = false;
        let errorMessage = error.error ? error.error : 'Error in uploading CSV file';
        
        if (typeof errorMessage === 'object') {
          errorMessage = JSON.stringify(errorMessage);
        }
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: errorMessage,
          sticky: false
        });
      }
    );
  }

  ngOnInit() {
    this.titleService.setTitle('IAM | Users');
    this.getCRUDdata(37);
    this.searchType = sessionStorage.getItem('userPreference');

    this.searchType
      ? this.searchType
      : (sessionStorage.setItem('userPreference', 'declarative'),
        (this.searchType = 'declarative'));

    this.searchOption(this.searchType);

    const _saveSearch = this.preserveSearch.getSearch('localUserSearch');
    if (!_saveSearch) {
      this.subscribeToJqlSearch();
    } else {
      this.searchWord = _saveSearch.searchKey;
      this.pagination = _saveSearch.pagination;
      this.selectedIdentityProvider = _saveSearch.selectedIdentityProvider;
      this.selectedOrganization = _saveSearch.selectedOrganization;
      this.selectedDepartment = _saveSearch.selectedDepartment;
      if (this.selectedOrganization) {
        this.fetchOrganizationDepartments();
      } else {
        this.selectedDepartment = undefined;
      }
      this.subscribeToJqlSearch();
    }
    this.organizationService.getOrganizationNames().subscribe((orgs: any) => {
      this.organizations = orgs;
    });
    this.subscribeToSearch();
  }

  ngAfterViewInit(): void {}

  addEventListeners(): void {
    if (sessionStorage.getItem('userPreference') == 'declarative') {
      this.renderer.listen(
        this.searchInputEvent.nativeElement,
        'keydown',
        event => {
          if (event.key === 'Backspace' && this.inputText.value === '') {
            this.clearForm('');
          }
        }
      );

      this.renderer.listen(
        this.searchInputEvent.nativeElement,
        'keyup',
        event => {
          if (event.ctrlKey && event.key.toLowerCase() === 'x') {
            this.clearForm('');
          }
          if (event.key === 'Backspace' && this.inputText.value === '') {
            this.clearForm('');
          }
        }
      );
    }
  }

  getCRUDdata(privId) {
    this.roles.getCrudData(privId).subscribe((res: any) => {
      this.createPermission = res?.create_permission === 1 ? true : false;
      this.readPermission = res?.read_permission === 1 ? true : false;
      this.updatePermission = res?.update_permission === 1 ? true : false;
      this.deletePermission = res?.delete_permission === 1 ? true : false;
    });
  }

  ngOnDestroy() {
    if (!this.navigationToEditLocalUser) {
      this.preserveSearch.saveSearch('localUserSearch', '');
    }

    if (this.otpStatusSubscription) {
      this.otpStatusSubscription.unsubscribe();
    }
  }

  selectAllAction() {
    this.deleteAllTotal = this.localUsers.recordsTotal;
    this.deleteCurrentTotal = this.localUsers.data.length;

    if (!this.selectAll) {
      this.selectedIdsForDeleteCurrent = [];
      this.selectAllExcept.clear();
      this.selectedUsers.clear();
      this.deleteAllTotal = 0;
      this.deleteCurrentTotal = 0;
    } else {
      this.selectedUsers.clear();
      this.localUsers.data.forEach(_user => {
        this.selectedIdsForDeleteCurrent.push(_user.id);
      });
    }

    this.changeCheckStatusOfAllClients(this.selectAll);
    this.adUserSelectedCount = 0;
    if (this.selectAll) {
      this.localUsers.data.map(_user => {
        if (
          _user.identity_provider == IdentityProvierEnum.AD &&
          _user.isChecked
        ) {
          this.adUserSelectedCount += 1;
        }
      });
    }
  }

  private changeCheckStatusOfAllClients(check) {
    this.localUsers.data = this.localUsers.data.map(_user => {
      return {
        ..._user,
        isChecked:
          this.isNotGWAdmin(_user) &&
          ((!this.selectAllExcept.has(_user.id) && this.selectAll) ||
            (this.selectedUsers.has(_user.id) && !this.selectAll))
      };
    });
  }

  isNotGWAdmin(user) {
    if (user.userName) {
      return user.userName.indexOf('gwadmin') == -1;
    }
    return true;
  }

  subscribeToSearch() {
    this.$searchUsers
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.clearSelection();
          this.loading = true;
          if (search.typing) {
            this.pagination.currentPage = 1;
          }
          return this.localUserService.searchUsers(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort,
            this.sortDir,
            this.selectedIdentityProvider,
            this.selectedOrganization,
            this.selectedDepartment
          );
        })
      )
      .subscribe((users: any) => {
        this.localUsers = users;
        this.isLdapSyncing =
          users.isLdapSyncing == null ? false : users.isLdapSyncing;
        this.pagination.totalItems = users.recordsTotal;
        this.loading = false;
      });
  }

  createLocalUser() {
    this.router.navigate(['/create-user']);
  }

  getLocalUsersFromDropDown(size = '') {
    let allUser;
    if (!size) allUser = true;
    if (this.simpleSearch) {
      this.loading = true;
      this.localUserService
        .getLocalUsersFromDropDown(
          this.pagination,
          this.sort,
          this.sortDir,
          this.searchWord,
          this.selectedIdentityProvider || '',
          this.selectedOrganization || '',
          this.selectedDepartment || '',
          allUser
        )
        .subscribe(
          (response: any) => {
            this.loading = false;
            response.data = response.data.map(_user => {
              return { ..._user, isChecked: false };
            });

            this.localUsers = response;
            setTimeout(() => {
              this.changeCheckStatusOfAllClients(this.selectAll);
            }, 10);
            this.isLdapSyncing =
              response.isLdapSyncing == null ? false : response.isLdapSyncing;
            this.pagination.totalItems = response.recordsTotal;
            this.allLocalUsers = this.localUsers;
          },
          error => {
            this.loading = false;
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'danger',
              message: 'Error in getting local users',
              sticky: false
            });
          }
        );
      this.preserveCurrentSearch();
    }
  }

  getLocalUsers() {
    this.loading = true;
    this.localUserService
      .getLocalUsers(
        this.pagination,
        this.sort,
        this.sortDir,
        this.searchWord,
        this.selectedIdentityProvider,
        this.selectedOrganization,
        this.selectedDepartment
      )
      .subscribe(
        (response: any) => {
          this.loading = false;
          response.data = response.data.map(_user => {
            return { ..._user, isChecked: false };
          });

          this.localUsers = response;
          setTimeout(() => {
            this.changeCheckStatusOfAllClients(this.selectAll);
          }, 10);
          this.isLdapSyncing =
            response.isLdapSyncing == null ? false : response.isLdapSyncing;
          this.pagination.totalItems = response.recordsTotal;
          this.allLocalUsers = this.localUsers;
        },
        error => {
          this.loading = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'danger',
            message: 'Error in getting local users',
            sticky: false
          });
        }
      );
    this.preserveCurrentSearch();
  }

  clearfun() {
    this.searchWord = '';
    this.subscribeToJqlSearch();
    this.deleteAllTotal = 0;
    this.deleteCurrentTotal = 0;
    this.selectedIdsForDeleteCurrent = [];
  }

  changeCheckBox(user) {
    const { isChecked, id, identity_provider } = user;

    if (identity_provider == IdentityProvierEnum.AD) {
      if (isChecked) {
        this.adUserSelectedCount += 1;
      } else {
        this.adUserSelectedCount -= 1;
      }
    }

    if (this.selectAll) {
      if (isChecked) {
        this.selectAllExcept.delete(id);
        this.selectedIdsForDeleteCurrent.push(id);
        this.deleteAllTotal = this.deleteAllTotal + 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal + 1;
      } else {
        this.selectAllExcept.add(id);
        this.selectedIdsForDeleteCurrent = this.selectedIdsForDeleteCurrent.filter(
          selectedId => selectedId !== id
        );
        this.deleteAllTotal = this.deleteAllTotal - 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal - 1;
      }
    } else {
      if (!isChecked) {
        this.selectedIdsForDeleteCurrent = this.selectedIdsForDeleteCurrent.filter(
          selectedId => selectedId !== id
        );
        this.selectedUsers.delete(id);
        this.deleteAllTotal = this.deleteAllTotal - 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal - 1;
      } else {
        this.selectedUsers.add(id);
        this.selectedIdsForDeleteCurrent.push(id);
        this.deleteAllTotal = this.deleteAllTotal + 1;
        this.deleteCurrentTotal = this.deleteCurrentTotal + 1;
      }
    }
  }

  preserveCurrentSearch() {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord,
      selectedIdentityProvider: this.selectedIdentityProvider,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment
    };
    this.preserveSearch.saveSearch('localUserSearch', save);
  }

  editUser(user) {
    this.preserveCurrentSearch();
    this.navigationToEditLocalUser = true;
    this.router.navigate(['/user', user.id]);
  }

  viewUser(user) {
    const save = {
      pagination: this.pagination,
      searchKey: this.searchWord,
      selectedIdentityProvider: this.selectedIdentityProvider,
      selectedOrganization: this.selectedOrganization,
      selectedDepartment: this.selectedDepartment
    };
    this.preserveSearch.saveSearch('localUserSearch', save);
    this.navigationToEditLocalUser = true;
    this.router.navigate(['/view-user', user.id]);
  }

  cancelVerificationPassword(event) {
    this.showPasswordConfimation = false;
  }

  cancelVerificationPasswordBulk(event) {
    this.showPasswordConfimationBulk = false;
  }

  submitVerificationPassword(event) {
    if (this.isDeleteCurrent === true) {
      const selectedIds = this.selectAll
        ? this.selectAllExcept
        : this.selectedUsers;

      if (!(selectedIds instanceof Set)) {
        return;
      }

      if (!this.selectAll) {
        this.selectedIdsForDeleteCurrent = [];
        this.selectedIdsForDeleteCurrent.push(...selectedIds);
      } else {
        let temp = this.selectedIdsForDeleteCurrent.filter(selectedId => {
          return ![...selectedIds].some(id => id === selectedId);
        });

        this.selectedIdsForDeleteCurrent = temp;
      }

      this.localUserService
        .deleteFromCurrentPage(this.selectedIdsForDeleteCurrent, event)
        .subscribe({
          next: (res: any) => {
            this.getLocalUsersFromDropDown();
            this.subscribeToJqlSearch();
            this.selectAllExcept.clear();
            this.selectedUsers.clear();
            this.selectAll = false;
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: res.message,
              sticky: false
            });
            this.deleteAllTotal = 0;
            this.deleteCurrentTotal = 0;
            this.selectedIdsForDeleteCurrent = [];
            this.showPasswordConfimation = false;
          },
          error: err => {
            this.deleteAllTotal = 0;
            this.deleteCurrentTotal = 0;
            this.loading = false;
          }
        });
    } else if (this.isDeleteAll === true) {
      const selectedIds = this.selectAll
        ? this.selectAllExcept
        : this.selectedUsers;
      const searchText = this.searchWord;
      const department = this.selectedDepartment;
      const idp = this.selectedIdentityProvider;
      this.showPasswordConfimationBulk = false;
      this.localUserService
        .bulkActions(
          'deleteUser',
          false,
          this.selectAll,
          selectedIds,
          this.selectedOrganization,
          department,
          searchText,
          idp,
          event,
          false
        )
        .subscribe({
          next: (res: any) => {
            this.getLocalUsersFromDropDown();
            this.subscribeToJqlSearch();
            this.selectAllExcept.clear();
            this.selectedUsers.clear();
            this.selectAll = false;
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: res.message,
              sticky: false
            });
            this.showPasswordConfimation = false;
            this.deleteAllTotal = 0;
            this.deleteCurrentTotal = 0;
          },
          error: err => {
            this.loading = false;
            this.deleteAllTotal = 0;
            this.deleteCurrentTotal = 0;
          }
        });
    } else {
      this.accessService
        .verifyPolicies(this.userTobeDeleted.id, '', [])
        .subscribe(res => {
          if (res) {
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'danger',
              message: 'Remove user from Entitlement(s) first',
              sticky: false
            });
          } else {
            this.localUserService
              .deleteUser(
                this.userTobeDeleted.id,
                this.userTobeDeleted.userName,
                event
              )
              .subscribe(
                response => {
                  this.showPasswordConfimation = false;
                  this.getLocalUsersFromDropDown();
                  this.subscribeToJqlSearch();
                  this.globalMessageService.pushAlert({
                    closed: false,
                    type: 'success',
                    message: 'User has been deleted successfully.',
                    sticky: false
                  });
                },
                error => {
                  this.loading = false;
                }
              );
          }
        });
    }
  }

  deleteUser(user) {
    this.userTobeDeleted = user;

    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      // this.option = 'delete';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheck$.subscribe(
        (res: any) => {
          if (res == true) {
            this.accessService
              .verifyPolicies(this.userTobeDeleted.id, '', [])
              .subscribe(res => {
                if (res) {
                  this.globalMessageService.pushAlert({
                    closed: false,
                    type: 'danger',
                    message: 'Remove user from Entitlement(s) first',
                    sticky: false
                  });
                } else {
                  this.localUserService
                    .deleteUserOtp(
                      this.userTobeDeleted.id,
                      this.userTobeDeleted.userName
                    )
                    .subscribe(
                      response => {
                        this.showPasswordConfimation = false;
                        this.getLocalUsersFromDropDown();
                        this.subscribeToJqlSearch();
                        this.globalMessageService.pushAlert({
                          closed: false,
                          type: 'success',
                          message: 'User has been deleted successfully.',
                          sticky: false
                        });
                      },
                      error => {
                        this.loading = false;
                      }
                    );
                }
              });
            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValue(false);
          }
          this.otpStatusSubscription.unsubscribe();
        }
      );
    } else {
      this.showPasswordConfimation = true;
    }
  }

  deleteUserBulk() {
    this.showPasswordConfimationBulk = true;
  }

  showConfirmationResetPassword(user) {
    this.showPasswordConfimationResetPassword = true;
  }

  hideConfirmationResetPassword(user) {
    this.showPasswordConfimationResetPassword = false;
  }

  pageChange(page) {
    this.pagination.currentPage = page;

    if (this.searchWord) {
      this.$searchUsers.next({ searchWord: this.searchWord, typing: false });
    } else if (!this.simpleSearch) {
      this.subscribeToJqlSearch();
    } else if (this.simpleSearch) {
      this.getLocalUsersFromDropDown();
    }
  }

  sendPasswordReset(user) {
    this.actionsDropdown = 0;
    this.localUserService.sendResetPasswordLink(user).subscribe(
      response => {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message:
            'The user will receive an email with reset password within two minutes.',
          sticky: false
        });
      },
      error => {
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'danger',
          message: 'Error in sending reset email',
          sticky: false
        });
      }
    );
  }

  pageSizeChange(size) {
    if (size == 'All' && !this.simpleSearch) {
      this.loading = true;
      this.localUserService
        .searchUsersJQL(
          this.rawQuery,
          0,
          this.pagination.currentPage,
          this.sort
        )
        .subscribe(res => {
          this.loading = false;
          this.localUsers = res;
        });
    } else if (!this.simpleSearch) {
      this.pagination.itemsPerPage = +size;
      this.pagination.totalItems = 0;
      this.pagination.currentPage = 1;
      this.localUserService
        .searchUsersJQL(
          this.rawQuery,
          this.pagination.itemsPerPage,
          this.pagination.currentPage,
          this.sort
        )
        .subscribe((res: any) => {
          this.loading = false;
          this.localUsers = res;
          this.pagination.totalItems = res.recordsTotal;
        });
    } else if (this.simpleSearch) {
      this.getLocalUsersFromDropDown(size);
    }
  }

  clearSelection() {
    this.selectAll = false;
    this.selectAllExcept.clear();
  }

  identityProviderChange() {
    this.clearSelection();
    if (
      this.selectedIdentityProvider ||
      this.selectedOrganization ||
      this.selectedDepartment
    ) {
      this.getLocalUsersFromDropDown();
    } else {
      this.selectedIdentityProvider = undefined;
      this.subscribeToJqlSearch();
    }
  }

  async fetchOrganizationDepartments() {
    this.pagination.currentPage = 1;
    if (!this.selectedOrganization) {
      this.selectedOrganization = undefined;
    } else {
      const org = (await this.organizationService
        .getOrganizationByName(this.selectedOrganization)
        .toPromise()) as any;
      const departments = (await this.organizationService
        .getAllDepartments(org.id)
        .toPromise()) as any;
      for (let j = 0; j < departments.data.length; j++) {
        this.departments.push({
          deptId: departments.data[j].id,
          deptName: departments.data[j].name
        });
      }
      this.departments = [...this.departments];
    }
  }

  async organizationChange() {
    this.clearSelection();
    this.departments = [];
    this.selectedDepartment = undefined;
    this.deleteAllTotal = 0;
    this.deleteCurrentTotal = 0;
    this.selectedIdsForDeleteCurrent = [];

    if (
      this.selectedIdentityProvider ||
      this.selectedOrganization ||
      this.selectedDepartment
    ) {
      this.fetchOrganizationDepartments();
      this.getLocalUsersFromDropDown();
    } else {
      this.selectedOrganization = undefined;
      this.subscribeToJqlSearch();
    }
  }

  departmentChange() {
    this.clearSelection();
    this.deleteAllTotal = 0;
    this.deleteCurrentTotal = 0;
    this.selectedIdsForDeleteCurrent = [];
    if (
      this.selectedIdentityProvider ||
      this.selectedOrganization ||
      this.selectedDepartment
    ) {
      this.getLocalUsersFromDropDown();
    } else {
      this.selectedDepartment = undefined;
      this.subscribeToJqlSearch();
    }
  }

  sortRecord(term: string) {
    this.sortDir = this.sortDir == 'asc' ? 'desc' : 'asc';
    this.sort = term;
    this.getLocalUsersFromDropDown();
  }

  enforcePasswordChange(event) {
    this.loading = true;
    this.localUserService
      .bulkPasswordChange(
        this.selectedUsers,
        this.selectAllExcept,
        this.selectAll,
        this.searchWord,
        event
      )
      .subscribe({
        next: (res: any) => {
          this.loading = false;
          this.showPasswordConfimationResetPassword = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: 'Password change process initiated.',
            sticky: false
          });
        },
        error: err => {
          this.loading = false;
        }
      });
  }

  bulkDeleteUser(event) {
    const selectedIds = this.selectAll
      ? this.selectAllExcept
      : this.selectedUsers;
    const idp = this.selectedIdentityProvider;
    this.showPasswordConfimationBulk = false;
    this.localUserService
      .bulkDeleteUsers(this.selectAll, selectedIds, idp, event)
      .subscribe({
        next: (res: any) => {
          this.getLocalUsersFromDropDown();
          this.selectAllExcept.clear();
          this.selectedUsers.clear();
          this.selectAll = false;
          this.globalMessageService.pushAlert({
            closed: false,
            type: 'success',
            message: res.message,
            sticky: false
          });
          this.deleteAllTotal = 0;
          this.deleteCurrentTotal = 0;
        },
        error: err => {
          this.loading = false;
          this.deleteAllTotal = 0;
          this.deleteCurrentTotal = 0;
        }
      });
  }

  openModal() {
    this.handleModal = !this.handleModal;
    this.toggleRuleViewStyle = 'block';
  }

  handleCloseModal() {
    this.toggleRuleViewStyle = 'none';
    let localUsersId = this.localUsers.data.map((data: any) => data.id);

    this.localUserService
      .addCustomUserGroups({
        groupName: this.groupName,
        groupDesc: this.groupDescription,
        sqlQuery: '',
        rawQuery: this.currentText || '',
        selectedIds:
          !this.searchWord &&
          !this.selectedIdentityProvider &&
          !this.selectedOrganization &&
          !this.selectedDepartment
            ? [...this.selectedUsers]
            : [...localUsersId],
        allUser: !this.searchWord && this.selectAll ? true : false
      })
      .subscribe(response => {
        this.clearForm('');
        this.router.navigate(['/customUserGroups']);
        this.globalMessageService.pushAlert({
          closed: false,
          type: 'success',
          message: 'Custom user group added',
          sticky: false
        });
      });
  }

  handleCancelModal() {
    this.toggleRuleViewStyle = 'none';
  }

  bulkAction(type, status) {
    if (type == 'deleteUser') {
      let mfa = localStorage.getItem('adminMfaValue');
      if (mfa === 'true') {
        this.showOtpDialog = true;
        this.option = 'edit';
        this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckEdit$.subscribe(
          (res: any) => {
            if (res == true) {
              const selectedIds = this.selectAll
                ? this.selectAllExcept
                : this.selectedUsers;
              const searchText = this.searchWord;
              const department = this.selectedDepartment;
              const idp = this.selectedIdentityProvider;
              this.showPasswordConfimationBulk = false;
              this.localUserService
                .bulkActions(
                  'deleteUser',
                  false,
                  this.selectAll,
                  selectedIds,
                  this.selectedOrganization,
                  department,
                  searchText,
                  idp,
                  event,
                  mfa
                )
                .subscribe({
                  next: (res: any) => {
                    this.getLocalUsersFromDropDown();
                    this.subscribeToJqlSearch();
                    this.selectAllExcept.clear();
                    this.selectedUsers.clear();
                    this.selectAll = false;
                    this.globalMessageService.pushAlert({
                      closed: false,
                      type: 'success',
                      message: res.message,
                      sticky: false
                    });
                    this.deleteAllTotal = 0;
                    this.deleteCurrentTotal = 0;
                    this.showPasswordConfimation = false;
                  },
                  error: err => {
                    this.loading = false;
                    this.deleteAllTotal = 0;
                    this.deleteCurrentTotal = 0;
                  }
                });
              this.showOtpDialog = false;
              this.OtpAuthenticationService.setOtpStatusValueEdit(false);
            }
            this.otpStatusSubscription.unsubscribe();
          }
        );
      } else {
        this.showPasswordConfimation = true;
        this.isDeleteAll = true;
      }
    } else {
      const selectedIds = this.selectAll
        ? this.selectAllExcept
        : this.selectedUsers;
      const searchText = this.searchWord;
      const department = this.selectedDepartment;
      const idp = this.selectedIdentityProvider;
      this.showPasswordConfimationBulk = false;
      console.log('selectedIds :>> ', selectedIds);
      console.log('TYy :>> ',!idp && !department && !searchText && !this.selectAll ? true : false );
      console.log('selectAllExcept :>> ', this.selectAllExcept);
      console.log('selectedUsers :>> ', this.selectedUsers);

        if (!(selectedIds instanceof Set)) {
          return;
        }

        if (!this.selectAll) {
          this.selectedIdsForDeleteCurrent = [];
          this.selectedIdsForDeleteCurrent.push(...selectedIds);
        } else {
          let temp = this.selectedIdsForDeleteCurrent.filter(selectedId => {
            return ![...selectedIds].some(id => id === selectedId);
          });

          this.selectedIdsForDeleteCurrent = temp;
        }

        console.log('this.selectedIdsForDeleteCurrent :>> ', this.selectedIdsForDeleteCurrent);
      this.localUserService
        .bulkActions(
          type,
          status,
          this.selectAll,
          this.selectedIdsForDeleteCurrent,
          this.selectedOrganization,
          department,
          searchText,
          idp,
          '',
          false
        )
        .subscribe({
          next: (res: any) => {
            this.getLocalUsersFromDropDown();
            this.selectAllExcept.clear();
            this.selectedUsers.clear();
            this.selectAll = false;
            this.globalMessageService.pushAlert({
              closed: false,
              type: 'success',
              message: res.message,
              sticky: false
            });
               this.deleteAllTotal = 0;
               this.deleteCurrentTotal = 0;
               this.selectedIdsForDeleteCurrent = [];
          },
          error: err => {
            this.loading = false;
          }
        });
    }
  }

  deleteFromCurrent() {
    let mfa = localStorage.getItem('adminMfaValue');
    if (mfa === 'true') {
      this.showOtpDialog = true;
      this.option = 'delete';
      this.otpStatusSubscription = this.OtpAuthenticationService.otpStatusCheckDel$.subscribe(
        (res: any) => {
          if (res == true) {
            const selectedIds = this.selectAll
              ? this.selectAllExcept
              : this.selectedUsers;

            if (!(selectedIds instanceof Set)) {
              return;
            }

            if (!this.selectAll) {
              this.selectedIdsForDeleteCurrent = [];
              this.selectedIdsForDeleteCurrent.push(...selectedIds);
            } else {
              let temp = this.selectedIdsForDeleteCurrent.filter(selectedId => {
                return ![...selectedIds].some(id => id === selectedId);
              });

              this.selectedIdsForDeleteCurrent = temp;
            }

            this.localUserService
              .deleteFromCurrentPageOtp(this.selectedIdsForDeleteCurrent)
              .subscribe({
                next: (res: any) => {
                  this.getLocalUsersFromDropDown();
                  this.subscribeToJqlSearch();
                  this.selectAllExcept.clear();
                  this.selectedUsers.clear();
                  this.selectAll = false;
                  this.globalMessageService.pushAlert({
                    closed: false,
                    type: 'success',
                    message: res.message,
                    sticky: false
                  });
                  this.deleteAllTotal = 0;
                  this.deleteCurrentTotal = 0;
                  this.selectedIdsForDeleteCurrent = [];
                  this.showPasswordConfimation = false;
                },
                error: err => {
                  this.loading = false;
                  this.deleteAllTotal = 0;
                  this.deleteCurrentTotal = 0;
                }
              });
            this.showOtpDialog = false;
            this.OtpAuthenticationService.setOtpStatusValueDel(false);
          }
          this.otpStatusSubscription.unsubscribe();
        }
      );
    } else {
      this.isDeleteCurrent = true;
      this.showPasswordConfimation = true;
    }
  }

  initializeSearch() {
    // Declarative Search
    this.localUserService.getCustomGroupData().subscribe((response: any) => {
      this.departmentsData.push(response['departments']);
      this.organizationsData.push(response['organization']);
      this.userNamesData.push(response['userNames']);
      this.emailsData.push(response['emails']);
      this.namesData.push(response['names']);

      this.getSearchObject();
      this.filteredOptions = this.inputText.valueChanges.pipe(
        startWith(''),
        map(value => this._filter(value))
      );
      this.addEventListeners();
    });
  }

  clearForm(data) {
    this.searchList = [];
    this.displayFn('');
    if (this.inputText) {
      this.inputText.reset('');
    }
    this.showDynamicGroupButton = false;
    this.isAlreadyClear = true;
    this.isSearchExecuted = false;
  }

  getSearchObject(stop = false): void {
    let response: Observable<any>;
    response = from([
      {
        Result: {
          DisplayName: 'Identity Provider',
          SearchType: 'Field',
          AutoCompleteValues: ['Local User', 'Active Directory']
        }
      },
      {
        Result: {
          DisplayName: 'Name',
          SearchType: 'Field',
          AutoCompleteValues: this.namesData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Email',
          SearchType: 'Field',
          AutoCompleteValues: this.emailsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Username',
          SearchType: 'Field',
          AutoCompleteValues: this.userNamesData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Organization',
          SearchType: 'Field',
          AutoCompleteValues: this.organizationsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'Department',
          SearchType: 'Field',
          AutoCompleteValues: this.departmentsData[0]
        }
      },
      {
        Result: {
          DisplayName: 'OrderBy',
          SearchType: 'Field',
          AutoCompleteValues: ['Ascending', 'Descending']
        }
      }
    ]);

    response.subscribe(val => {
      this.response.push(val.Result);
      this.fieldList.Value = this.response
        .filter(r => r.SearchType == Action.Field)
        .map<string>(r => r.DisplayName.toString());
      this.myControl.setValue(''); // trigger the autocomplete to populate new values
    });
  }

  // autocomplete material ui events
  _filter(value: string): string[] {
    if (value) {
      this.isSearchExecuted = true;
      this.isAlreadyClear = false;
    }
    this.currentText = this.searchList.map(s => s.Value).join(' ');
    this.getSearchText(value);
    if (!/Ascending|Descending/i.test(this.currentText)) {
      this.lastValue = '';
    }

    if (this.isExpression) {
      this.$searchUsersJQL.next({ searchWord: this.currentText, typing: true });
    }

    if (!(this.lastValue === 'Ascending' || this.lastValue === 'Descending')) {
      let optionListToBePopulated: string[] = this.getOptionList();
      var searchText = this.getSearchText(value);
      return optionListToBePopulated.filter(
        option =>
          option.toLowerCase().indexOf(searchText.toLowerCase().trim()) != -1
      );
    }
  }

  displayFn(value: string): string {
    if (!!value) {
      if (this.lastValue === 'Ascending' || this.lastValue === 'Descending') {
        this.getSearchObject(true);
      } else {
        this.searchList.push(
          new SelectedOption(
            value,
            this.currentEvent,
            this.getNextEvent(this.currentEvent)
          )
        );
      }
      this.lastValue = value;
    }
    return this.searchList.length > 0
      ? this.searchList.map(s => s.Value).join(' ')
      : '';
  }

  // private functions
  // ------------- Get Autocomplete List START --------------------
  private getOptionList(): string[] {
    if (
      this.searchList == null ||
      this.searchList == undefined ||
      this.searchList.length === 0
    ) {
      this.currentEvent = this.defaultSelection;
      return this.field;
    }

    let lastElement: SelectedOption = <SelectedOption>(
      this.searchList.slice(-1).pop()
    );
    let currentList = this.selectionList.find(
      s => s.Name.toLowerCase() === lastElement.Next.toLowerCase()
    );
    this.currentEvent = currentList ? currentList.Name : this.defaultSelection;
    return currentList ? this.getValues(currentList) : this.field;
  }

  private getValues(currentList: SelectionDict): string[] {
    if (currentList.NextSelection == Action.Expression) {
      this.showDynamicGroupButton = false;
    }

    if (this.currentEvent.toLowerCase() != 'value') return currentList.Value;
    var selectedField = this.getlastField();
    var selectedValue = selectedField ? selectedField.Value : '';
    var filteredResponse = this.response.find(
      r => r.DisplayName === selectedValue
    );
    return filteredResponse ? filteredResponse.AutoCompleteValues : [];
  }

  // ------------- Get Autocomplete List END --------------------

  // --------------- START : Get the search text based on which the autocomplete will populate --------

  private getSearchText(value: string): string {
    var oldText = this.searchList.map(s => s.Value).join(' ');
    this.handleBackspace(value);
    return value.trim().replace(oldText, '');
  }

  private handleBackspace(searchValue: string): void {
    var oldText = this.searchList.map(s => s.Value).join(' ');
    var previousListName =
      this.searchList.length != 0
        ? this.searchList[this.searchList.length - 1].PopulatedFrom
        : '';
    var prevList = this.selectionList.find(
      s => s.Name.toLowerCase() === previousListName.toLowerCase()
    );
    var prevListValue = prevList ? prevList.Value : [];

    if (previousListName == Action.Value) {
      var lastField = this.getlastField();
      var lastFieldValue = lastField ? lastField.Value : '';
      var filteredResponse = this.response.find(
        r => r.DisplayName === lastFieldValue
      );
      prevListValue = filteredResponse
        ? filteredResponse.AutoCompleteValues
        : [];
    }

    if (
      (prevListValue ? prevListValue.indexOf(searchValue) === -1 : false) &&
      oldText.trim().length > searchValue.trim().length
    )
      this.searchList.pop();
  }

  // --------------- END : Get the search text based on which the autocomplete will populate --------

  private getNextEvent(currentEvent: string): string {
    if (!(this.lastValue === 'Ascending' || this.lastValue === 'Descending')) {
      var currentList = this.selectionList.find(
        s => s.Name.toLowerCase() === currentEvent.toLowerCase()
      );
      if (
        (currentList ? currentList.NextSelection : this.defaultSelection) ==
        Action.Expression
      ) {
        this.isExpression = true;
        this.showDynamicGroupButton = true;
      } else {
        this.isExpression = false;
        this.showDynamicGroupButton = false;
      }
      return currentList ? currentList.NextSelection : this.defaultSelection;
    }
  }

  private getlastField(): SelectedOption | undefined {
    if (this.searchList.length === 0) return undefined;
    let i: number = this.searchList.length - 1;
    for (i; i >= 0; i--) {
      if (this.searchList[i].PopulatedFrom == Action.Field)
        return this.searchList[i];
    }
    return undefined;
  }

  subscribeToJqlSearch() {
    this.$searchUsersJQL
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((search: { searchWord: string; typing: boolean }) => {
          this.clearSelection();
          this.loading = true;
          if (search.typing && !this.rawQuery) {
            this.pagination.currentPage = 1;
          }
          this.rawQuery = search.searchWord;
          return this.localUserService.searchUsersJQL(
            search.searchWord,
            this.pagination.itemsPerPage,
            this.pagination.currentPage,
            this.sort
          );
        })
      )
      .subscribe((users: any) => {
        this.localUsers = users;
        this.isLdapSyncing =
          users.isLdapSyncing == null ? false : users.isLdapSyncing;
        this.pagination.totalItems = users.recordsTotal;
        this.loading = false;
      });
  }

  cancelOtpVerificationPassword(event: any) {
    this.showOtpDialog = false;
    this.OtpAuthenticationService.setOtpStatusValueEdit(false);
    this.OtpAuthenticationService.setOtpStatusValueDel(false);
  }

  submitOtpVerificationPassword(event: any) {
    this.otptoken = event;
    // this.otptoken = event
  }
  getClosePopup(event: any) {
    this.showOtpDialog = false;
  }
  searchOption(option: String) {
    this.clearForm('');
    this.clearSelection();
    this.selectedIdentityProvider = undefined;
    this.selectedOrganization = undefined;
    this.organizationChange();
    this.clearfun();

    if (option === 'declarative') {
      sessionStorage.setItem('userPreference', 'declarative');
      this.searchType = 'declarative';
      this.simpleSearch = false;
      this.declarativeSearch = true;
      this.initializeSearch();
      this.selectAll = false;
      this.selectAllExcept.clear();
      this.selectedUsers.clear();

      if (this.rawQuery.length > 0) {
        this.showDynamicGroupButton = true;
      }
    } else if (option === 'simple') {
      sessionStorage.setItem('userPreference', 'simple');
      this.searchType = 'simple';
      this.simpleSearch = true;
      this.declarativeSearch = false;
      this.showDynamicGroupButton = false;
      this.selectAll = false;
      this.selectAllExcept.clear();
      this.selectedUsers.clear();
    }
  }
}

class SelectedOption {
  public Value: string;
  public PopulatedFrom: string;
  public Next: string;

  constructor(value: string, populatedFrom: string, next: string) {
    this.Value = value;
    this.PopulatedFrom = populatedFrom;
    this.Next = next;
  }
}

class SuggestionDetails {
  public Name: string;
  public Valid: string[];
  public Value: string[];
}

class SelectionDict {
  public Name: string;
  public Value: string[];
  public NextSelection: string;
}

// Server response
class ApiResponse {
  public DisplayName: string;
  public SearchType: string;
  public AutoCompleteValues: string[];
}

enum Action {
  Field = 'Field',
  Operator = 'Operator',
  Value = 'Value',
  Expression = 'Expression'
}
