import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';

import { AuthenticationService } from '../login/authentication.service';
import { PaginationResponse } from '../models/pagination-response';
import { Service } from '../models/service';
import { Assign } from '../models/assign';
import { Observable } from 'rxjs';
import { map } from 'rxjs/operators';
import { ServiceConnectionOptionsEnum } from '../shared/emuns/service.enum';

@Injectable()
export class ManageService {
  private servicesUrl = 'http://localhost:3000/services';
  private AssociatedservicesApiUrl = '/services/associated';
  private servicesApiUrl = '/services';
  private assignServiceApiUrl = '/assign';
  private assignServiceToAll = '/services/assignToAll';
  private unassignServiceToAll = '/services/unassignToAll';
  private ldapApiUrl = '/ldapmanagement';
  private localUsersApiUrl = '/localUsers';
  private services: Service[];
  private apiBaseUrl: string;
  httpOptions = {
    headers: new HttpHeaders({
      'Content-Type': 'application/json'
    })
  };
  private ConnectionOptions = ServiceConnectionOptionsEnum;

  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.servicesApiUrl = this.apiBaseUrl + this.servicesApiUrl;
    this.AssociatedservicesApiUrl =
      this.apiBaseUrl + this.AssociatedservicesApiUrl;
    //this.checkvalidity = this.apiBaseUrl + this.checkvalidity;
    this.ldapApiUrl = this.apiBaseUrl + this.ldapApiUrl;
    this.localUsersApiUrl = this.apiBaseUrl + this.localUsersApiUrl;
    this.assignServiceApiUrl = this.apiBaseUrl + this.assignServiceApiUrl;
    this.assignServiceToAll = this.apiBaseUrl + this.assignServiceToAll;
    this.unassignServiceToAll = this.apiBaseUrl + this.unassignServiceToAll;
  }

  /** GET from the server */

  getServices(
    page: number = 1,
    pageSize: number = 10,
    active: string = 'isactive',
    sort: string = 'createdAt',
    dir: string = 'desc',
    searchKeyword: string = '',
    is_persistent = false,
    selectedConnection = this.ConnectionOptions.All,
    gatewayGroups = [],
    all = false,
    type = '',
    protocol = '',
    except = []
  ): any {
    const urlEncoded = encodeURIComponent(searchKeyword);
    const skip = JSON.stringify(except);
    var queryString = `?limit=${pageSize}&offset=${page -
      1}&active=${active}&sort=${sort}&dir=${dir}&search=${urlEncoded}&is_persistent=${is_persistent}&gatewayGroups=${JSON.stringify(
      gatewayGroups
    )}&type=${type}&protocol=${protocol}&all=${all}&except=${skip}&selectedConnection=${selectedConnection}`;
    const token = localStorage.getItem('token');
    if(token)
      return this.authHttp
        .get(`${this.servicesApiUrl}/all${queryString}`)
        .pipe(map(response => response));
  }

  getCurrentSelectedServices(
    page: number = 1,
    pageSize: number = 10,
    sort: string = 'createdAt',
    dir: string = 'desc',
    searchKeyword: string = '',
    allServicesSelected = false
  ): any {
    const urlEncoded = encodeURIComponent(searchKeyword);
    var queryString = `?limit=${pageSize}&offset=${page -
      1}&sort=${sort}&dir=${dir}&search=${urlEncoded}&allServicesSelected=${allServicesSelected}`;
    return this.authHttp
      .get(`${this.servicesApiUrl}/services${queryString}`)
      .pipe(map(response => response));
  }

  getAllTypesAndProtocols(): any {
    return this.authHttp
      .get(`${this.servicesApiUrl}/type/protocol`)
      .pipe(map(response => response));
  }

  getAvailableServices(force: boolean): any {
    return this.authHttp
      .get(`${this.servicesApiUrl}/available?force=${force}`)
      .pipe(map(response => response));
  }

  getLdapUsers(term = null, nextPage = null): any {
    return this.authHttp
      .post(`${this.ldapApiUrl}/${btoa(term)}`, { nextPage })
      .pipe(map(response => response));
  }

  getLocalUsers(term) {
    return this.authHttp
      .post(
        `${this.localUsersApiUrl}/searchClient?search=${encodeURIComponent(
          term
        )}`,
        {}
      )
      .pipe(map(response => response));
  }

  add_user(service: Service): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    let queryString = '';
    if (service.id > 0 && typeof service.id != 'undefined') {
      queryString = '/' + service.id;
      return this.authHttp
        .put(this.servicesApiUrl + queryString, { service: service }, options)
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    } else {
      return this.authHttp
        .post(this.servicesApiUrl, { service: service }, options)
        .pipe(map(response => response));
    }
  }

  assignService(assignedService: Assign): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .post(
        this.assignServiceApiUrl,
        { assignedService: assignedService },
        options
      )
      .pipe(map(response => response));
  }

  assignServiceToAllClient(id: number): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .post(this.assignServiceToAll, { serviceId: id }, options)
      .pipe(map(response => response));
  }

  unassignServiceToAllClient(id: number): any {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .post(this.unassignServiceToAll, { serviceId: id }, options)
      .pipe(map(response => response));
  }

  getAssignedServices(userName: string): any {
    var queryString = '';

    if (userName != '') {
      queryString = '/' + userName;
    }
    return this.authHttp
      .get(this.assignServiceApiUrl + queryString)
      .pipe(map(response => response));
  }

  delete_service(id: number, verificationPassword): any {
    var queryString = '';

    if (id > 0) {
      queryString = '/' + id;
    }
    this.httpOptions['body'] = { verificationPassword };
    return this.authHttp
      .delete(this.servicesApiUrl + queryString, this.httpOptions)
      .pipe(map(this.extractData));
  }

  unassignService(assignedService: Assign): any {
    var queryString = '';

    if (assignedService.serviceid > 0 && assignedService.ldapUserName != '') {
      queryString =
        '/' + assignedService.serviceid + '/' + assignedService.ldapUserName;
    }
    return this.authHttp
      .delete(this.assignServiceApiUrl + queryString)
      .pipe(map(this.extractData));
    // .catch(this.handleError);
  }

  updatemanage(service: Service): Observable<any> {
    let headers = new HttpHeaders({ 'Content-Type': 'application/json' });
    let options = { headers: headers };
    return this.authHttp
      .put(this.servicesUrl, service, options)
      .pipe(map(response => response));
  }
  getmanagedetail(id): Observable<Service | any> {
    var queryString = '';

    if (id != '') {
      queryString = '/' + id;
    }

    return this.authHttp
      .get(this.servicesApiUrl + queryString)
      .pipe(map(response => response));
  }

  getAllAvailableDevices(): Observable<any> {
    return this.authHttp
      .get(this.servicesApiUrl + '/devices')
      .pipe(map((response: any) => response.devices));
  }

  getAllServices(): any {
    return this.authHttp
      .get(this.servicesApiUrl)
      .pipe(map(response => response));
  }

  getSingleServiceDetails(id): any {
    return this.authHttp
      .get(`${this.servicesApiUrl}/detail/${id}`)
      .pipe(map(response => response));
  }

  getQuickGateways(searchKey = '', is_persistent = false, except = []): any {
    const skip = JSON.stringify(except);
    let query = `?searchKey=${searchKey}&is_persistent=${is_persistent}&except=${skip}`;

    return this.authHttp
      .get(this.servicesApiUrl + '/quick' + query)
      .pipe(map(response => response));
  }

  getAllServicesForGateway(): any {
    return this.authHttp
      .get(this.servicesApiUrl + '/getallforgateway/')
      .pipe(map(response => response));
  }

  getAllAssociatedServices(): any {
    return this.authHttp
      .get(this.AssociatedservicesApiUrl)
      .pipe(map(response => response));
  }

  deleteBulkServices(
    selectAll,
    selectedIds,
    search = '',
    is_persistent = false,
    password
  ) {
    this.httpOptions['body'] = {
      selectAll,
      selectedIds: Array.from(selectedIds),
      verificationPassword: password,
      search,
      is_persistent
    };
    return this.authHttp.delete(
      this.servicesApiUrl + `/deleteBulkServices`,
      this.httpOptions
    );
  }

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }
}
