import { Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { Pagination } from '../models/pagination';
import { map } from 'rxjs/operators';

@Injectable()
export class MicrosegmentService {
  private microsegmentationApiUrl: string;
  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    const baseUrl = configService.getConfiguration().apiBaseUrl;
    this.microsegmentationApiUrl = baseUrl + '/microsegments';
  }

  getWorkload(id) {
    return this.authHttp
      .get(`${this.microsegmentationApiUrl}/workload/${id}`)
      .pipe(map(response => response));
  }

  getMicrosegment(id, deviceId) {
    return this.authHttp
      .get(`${this.microsegmentationApiUrl}/microsegment/${id}/${deviceId}`)
      .pipe(map(response => response));
  }

  getWorkloadMicrosegments(id) {
    return this.authHttp
      .get(`${this.microsegmentationApiUrl}/workloadMicrosegments/${id}`)
      .pipe(map(response => response));
  }

  getMicrosegmentDestinations(microsegmentId, destination) {
    // .post(`${this.microsegmentationApiUrl}/ports/${microsegmentId}`,destination)
    if (destination) {
      return this.authHttp
        .get(
          `${this.microsegmentationApiUrl}/ports/${microsegmentId}/${destination}`
        )
        .pipe(map(response => response));
    }
  }

  editMicrosegmentDestinations(microsegmentId, destination) {
    if (destination) {
      return this.authHttp
        .post(
          `${this.microsegmentationApiUrl}/ports/${microsegmentId}`,
          destination
        )
        .pipe(map(response => response));
    }
  }

  createDestinationPort(destinationPort) {
    return this.authHttp
      .post(
        `${this.microsegmentationApiUrl}/destination-port/create`,
        destinationPort
      )
      .pipe(map(response => response));
  }

  createMicrosegment(microsegmentObj) {
    return this.authHttp
      .post(`${this.microsegmentationApiUrl}/create`, microsegmentObj)
      .pipe(map(response => response));
  }
  getAllMicrosegmentDestinations(
    microsegmentId,
    pagination: Pagination,
    name: String = ''
  ) {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _name: String = '';
    if (name.trim()) {
      _name = name.trim();
    }
    return this.authHttp
      .get(
        this.microsegmentationApiUrl +
          '/microsegmentDestinations' +
          `?microsegmentId=${microsegmentId}&offset=${currentPage}&limit=${itemsPerPage}&searchValue=${_name}`
      )
      .pipe(map(response => response));
  }
  getAllMicrosegments(workloadId, pagination: Pagination, name: String = '') {
    const currentPage = pagination.currentPage - 1 || 0;
    const itemsPerPage = pagination.itemsPerPage || 10;
    let _name: String = '';
    if (name.trim()) {
      _name = name.trim();
    }
    return this.authHttp
      .get(
        this.microsegmentationApiUrl +
          `?workloadId=${workloadId}&offset=${currentPage}&limit=${itemsPerPage}&searchValue=${_name}`
      )
      .pipe(map(response => response));
  }

  searchMicrosegments(
    workloadId,
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?workloadId=${workloadId}` +
      `&limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.microsegmentationApiUrl}${query}`)
      .pipe(map(response => response));
  }
  searchMicrosegmentDestination(
    microsegmentId,
    searchQuery: string,
    pageSize: number = 10,
    page: number = 1,
    sort: string = 'id',
    dir: string = 'desc'
  ) {
    const encodedUrlComponent = encodeURIComponent(searchQuery);
    let query =
      `?microsegmentId=${microsegmentId}` +
      `&limit=${pageSize}` +
      `&offset=${page - 1}` +
      `&sort=${sort}` +
      `&dir=${dir}` +
      `&searchTerm=${encodedUrlComponent}`;

    return this.authHttp
      .get(`${this.microsegmentationApiUrl}/microsegmentDestinations${query}`)
      .pipe(map(response => response));
  }

  deleteMicrosegment(id) {
    return this.authHttp
      .delete(this.microsegmentationApiUrl + `/${id}`)
      .pipe(map(response => response));
  }

  deleteMicrosegmentDestination(id, destination) {
    return this.authHttp
      .post(
        this.microsegmentationApiUrl + '/destinationDevice' + `/${id}`,
        destination
      )
      .pipe(map(response => response));
  }

  discoverMicrosegments(workloadId) {
    return this.authHttp
      .get(this.microsegmentationApiUrl + '/discover' + `/${workloadId}`)
      .pipe(map(response => response));
  }

  discoverAllMicrosegments() {
    return this.authHttp
      .get(this.microsegmentationApiUrl + '/discover')
      .pipe(map(response => response));
  }
}
