import {
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  EventEmitter,
  Output
} from '@angular/core';
import { Location } from '@angular/common';
import { AuthenticationService } from '../login/authentication.service';
import { NgbDropdownConfig } from '@ng-bootstrap/ng-bootstrap';
import * as $ from 'jquery';
import { Router, ActivatedRoute, NavigationStart } from '@angular/router';
import { EventManager } from '@angular/platform-browser';
import { ConfigService } from '../services/config.service';
import { takeUntil, tap } from 'rxjs/operators';
import * as OverlayScrollbars from 'overlayscrollbars';
import { RoleEnum, TitleEnum } from '../shared/emuns/global.enum';
import { LicenseService } from '../license/license.service';
import { ErrorCodeEnum } from '../shared/emuns/error.enum';
import { GlobalMessageClass } from '../global-messages/global-messages';
import { GlobalMessageService } from '../global-messages/global-messages.service';
import { RolesData } from '../shared/roles/roles';
import { Subject } from 'rxjs';
import { RolesAndPermissionService } from '../roles-and-permission/roles-and-permission.service';
import { CategoryService } from '../topbar/nav.service';
import { map, take } from 'rxjs/operators';

//import { environment } from '../../environments/environment';
@Component({
  selector: 'app-navbar',
  templateUrl: './navbar.component.html',
  styleUrls: ['./navbar.component.css'],
  changeDetection: ChangeDetectionStrategy.OnPush
})
export class NavbarComponent {
  toggleSubmenu = false;
  userMenuToggle = false;
  S2SMenuToggle = false;
  clientMenuToggle = false;
  infrastructureMenuToggle = false;
  deviceMenuToggle = false;
  reportsMenuToggle = false;
  softwareMenuToggle = false;
  accessMenuToggle = false;
  settingMenuToggle = false;
  databaseMenuToggle = false;
  policyMenuToggle = false;
  accessPolicySubToggle = false;
  devicePolicySubToggle = false;
  alertsToggle = false;
  clientBridgeToggle = false;
  iotbridgeToggle = false;
  softwaresListMenuToggle = false;
  isS2sEnabled = false;

  title = 'Invisily';
  showNavBar: boolean = false;
  showLicenseOnly: boolean = false;
  //showReport: boolean = false;
  isadmin: boolean = true;
  public currentLocation = '';
  public littleShortVar: boolean = false;
  public userName: string = '';
  public userrole: string = '';
  assignedUserTemplates: any;
  navigationData = [];

  Role = RoleEnum;

  deviceBridge86Const = 'Device Bridge (x86)';
  deviceBridgeArmConst = 'Device Bridge (ARM)';

  @Output() navbarStatus = new EventEmitter<boolean>();
  public alert: GlobalMessageClass;
  private destroy$ = new Subject<void>();

  constructor(
    location: Location,
    private router: Router,
    activatedRoute: ActivatedRoute,
    private authenticationManager: AuthenticationService,
    config: NgbDropdownConfig,
    private licenseService: LicenseService,
    private globalMessageService: GlobalMessageService,
    private rolesPermissionService: RolesAndPermissionService,
    private changeDetectorRef: ChangeDetectorRef,
    private categoryService: CategoryService
  ) {
    this.setUserRole();
    router.events.subscribe(val => {
      this.currentLocation = location.path();
    });
    var currentUser = JSON.parse(localStorage.getItem('currentUser'));
    if (typeof currentUser != 'undefined' && currentUser != null) {
      this.userName = currentUser.userFirstName;
    }
    config.autoClose = false;

    this.authenticationManager.showNavBarEmitter
      .pipe(
        tap(mode => {
          var currentUser = JSON.parse(localStorage.getItem('currentUser'));

          if (typeof currentUser != 'undefined' && currentUser != null) {
            if (mode !== null || currentUser.token) {
              this.showNavBar = true;
            } else {
              this.showNavBar = false;
            }
          }
        })
      )
      .subscribe({
        next: () => {
          var currentUser = JSON.parse(localStorage.getItem('currentUser'));
          if (typeof currentUser == 'undefined' || currentUser == null) {
            this.showNavBar = false;
            this.changeDetectorRef.detectChanges();
          }
        },
        error: err => {
          console.log(`Oops... ${err}`);
        },
        complete: () => {
          console.log('error');
        }
      });

    this.authenticationManager.showLicenseOnlyEmitter.subscribe(mode => {
      this.showLicenseOnly = mode;
    });

    this.authenticationManager.authStateChange.subscribe(state => {
      this.setUserRole();
    });

    this.findLittleShort();
    this.subscriberRouters(router);
  }

  ngOnInit() {
    this.getLicense();
    this.rolesPermissionService.assignedUserTemplates$.subscribe({
      next: data => {
        this.assignedUserTemplates = data;
        if (this.assignedUserTemplates) {
          const assignedData = this.assignedUserTemplates;
          this.navigationData = RolesData.map(roleObj => {
            if (assignedData && assignedData.length > 0) {
              const node = assignedData?.find(
                assignedObj => assignedObj.privilege_id === roleObj.privilege_id
              );
              if (node) {
                return {
                  name: roleObj.name,
                  privilege_id: roleObj.privilege_id,
                  create_permission: Boolean(node.create_permission),
                  read_permission: Boolean(node.read_permission),
                  update_permission: Boolean(node.update_permission),
                  delete_permission: Boolean(node.delete_permission)
                };
              }
              return null;
            }
          }).filter(
            updatedNode =>
              updatedNode &&
              (updatedNode.create_permission ||
                updatedNode.read_permission ||
                updatedNode.update_permission ||
                updatedNode.delete_permission)
          );
        }
        this.changeDetectorRef.detectChanges();
      }
    });
    this.categoryService.selectedCategory$.subscribe(category => {
      if (category) {
        if (category === 'Dashboard') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.accessMenuToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'All Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Devices Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Services Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Policy Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Users Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Infrastructure Reports') {
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Onboarding Reports'){
          this.reportsMenuToggle = true;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Alerts'){
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Entitlements') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = true;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'User Entitlements') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = true;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Device Entitlements') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = true;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Security Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = true;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'OS Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = true;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Software Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = true;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Process Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = true;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Data Volume Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = true;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Routing Policy') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = true;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Services') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Service Groups') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Controllers') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Gateways') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Gateways Groups') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'LDAP Configuration') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'IDP Management') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Server Bridges') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Device Bridge (x86)') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Device Bridge (ARM)') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'RDI Bridges' || category === 'RBI Bridges') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'SaaS Bridges') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = true;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Admin Users') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = true;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Roles And Permissions') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = true;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Users') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = true;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'User Groups') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = true;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Devices') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = true;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Workload Devices') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = true;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Microsegments') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = true;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Workloads') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = true;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'End User Clients') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = true;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Server Side Client') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = true;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Groups') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = true;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Organizations') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Software Master List') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = true;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Unauthorized Software') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = true;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Process Binding') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = true;
          this.accessMenuToggle = false;
          this.settingMenuToggle = false;
        } else if (category === 'Client') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = true;
          this.settingMenuToggle = false;
        } else if (category === 'Admin') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = true;
          this.settingMenuToggle = false;
        } else if (category === 'Portal Settings') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        } else if (category === 'Client Settings') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        } else if (category === 'Core Settings') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        } else if (category === 'License Management') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        } else if (category === 'System Backup') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        } else if (category === 'Software Update') {
          this.reportsMenuToggle = false;
          this.policyMenuToggle = false;
          this.accessPolicySubToggle = false;
          this.devicePolicySubToggle = false;
          this.infrastructureMenuToggle = false;
          this.userMenuToggle = false;
          this.deviceMenuToggle = false;
          this.S2SMenuToggle = false;
          this.clientMenuToggle = false;
          this.softwaresListMenuToggle = false;
          this.accessMenuToggle = false;
          this.settingMenuToggle = true;
        }
      }
    });
  }

  findRoleByName(name) {
    const allowed = this.navigationData.find(role => role.name === name);
    return allowed ? true : false;
  }

  ngAfterViewInit() {
    $('body').on('click', '.sidebar-toggler', function(e) {
      var a = $('body'),
        t = $('.page-sidebar'),
        i = $('.page-sidebar-menu'),
        pageLogo = $('.page-logo'),
        v = $('.version');
      $('.sidebar-search', t).removeClass('open'),
        a.hasClass('page-sidebar-closed')
          ? (a.removeClass('page-sidebar-closed'),
            v.removeClass('page-sidebar-menu-closed'),
            i.removeClass('page-sidebar-menu-closed'),
            pageLogo.removeClass('justifyStart'),
            true)
          : (a.addClass('page-sidebar-closed'),
            i.addClass('page-sidebar-menu-closed'),
            v.addClass('page-sidebar-menu-closed'),
            a.hasClass('page-sidebar-fixed') && i.trigger('mouseleave'),
            pageLogo.addClass('justifyStart'),
            true),
        $(window).trigger('resize');

      if ($('body').hasClass('page-sidebar-closed')) {
        $('.page-logo').css('padding-left', '10px');
      } else {
        $('.page-logo').css('justify-content', 'space-between');
      }
    });

    $('body').hasClass('page-sidebar-fixed') &&
      $('.page-sidebar')
        .on('mouseenter', function() {
          if ($('body').hasClass('page-sidebar-closed')) {
            $(this)
              .find('.page-sidebar-menu')
              .removeClass('page-sidebar-menu-closed');
            $('.version').removeClass('page-sidebar-menu-closed');
          }
        })
        .on('mouseleave', function() {
          if ($('body').hasClass('page-sidebar-closed')) {
            $(this)
              .find('.page-sidebar-menu')
              .addClass('page-sidebar-menu-closed');
            $('.version').addClass('page-sidebar-menu-closed');
          }
        });
  }

  private subscriberRouters(router: Router) {
    router.events.subscribe((event: any) => {
      if (event instanceof NavigationStart) {
        if (event.url == '/restore' || event.url == '/backups') {
          this.toggleSubmenu = true;
        } else if (
          event.url == '/users' ||
          event.url == '/local-users' ||
          event.url == '/create-admin' ||
          event.url == '/create-user' ||
          event.url == '/view_roles_and_permissions' ||
          event.url == '/create_roles_and_permissions' ||
          event.url == '/edit_roles_and_permissions' ||
          event.url == '/view_additional_permissions' ||
          event.url == '/create_additional_permissions'
        ) {
          this.userMenuToggle = true;
        } else if (
          event.url == '/workloads' ||
          event.url == '/microsegmentation'
        ) {
          this.S2SMenuToggle = true;
        } else if (
          event.url == '/softwares' ||
          event.url == '/forbidden_softwares'
        ) {
          this.softwaresListMenuToggle = true;
        } else if (
          event.url == '/clients/Client' ||
          event.url == '/groups' ||
          event.url == '/clients/Server' ||
          event.url == '/create-client/Client' ||
          event.url == '/create-client/Server' ||
          event.url == '/create-group'
        ) {
          this.clientMenuToggle = true;
        } else if (
          event.url == '/services' ||
          event.url == '/controllers' ||
          event.url == '/gateways' ||
          event.url == '/create-manage' ||
          event.url == '/create-gateway' ||
          event.url == '/create-idp' ||
          event.url == '/ldaps' ||
          event.url == '/idp'
        ) {
          this.infrastructureMenuToggle = true;
        } else if (event.url == '/deviceInventory') {
          this.deviceMenuToggle = true;
        } else if (event.url == '/updates' || event.url == '/create-update') {
          this.softwareMenuToggle = true;
        } else if (
          event.url == '/admin-access-logs' ||
          event.url == '/clients-access-logs'
        ) {
          this.accessMenuToggle = true;
        }
      }
    });
  }

  public littleShort() {
    localStorage.setItem('littleShort', 'true');
    this.navbarStatus.emit(true);
    // document.getElementById('topbar').classList.add('little-short');
    // document
    //   .getElementById('top-nav-right')
    //   .classList.add('little-short-right');
    // document
    //   .getElementById('main-body-cont')
    //   .classList.add('little-short');
    this.findLittleShort();
  }

  public littleShortRevert() {
    localStorage.setItem('littleShort', 'false');
    this.navbarStatus.emit(false);
    // document.getElementById('topbar').classList.remove('little-short');
    // document
    //   .getElementById('top-nav-right')
    //   .classList.remove('little-short-right');
    // document
    //   .getElementById('main-body-cont')
    //   .classList.remove('little-short');
    this.findLittleShort();
  }

  public findLittleShort() {
    let isLittleShort = JSON.parse(localStorage.getItem('littleShort'));
    if (isLittleShort) {
      if (isLittleShort == true) {
        this.littleShortVar = true;
        this.navbarStatus.emit(true);
      }
    } else {
      if (isLittleShort == false) {
        this.littleShortVar = false;
        this.navbarStatus.emit(false);
      }
    }
  }

  openNewTab() {
    //let reportsUrl = environment.production ? "/web/reports/executiveSummaryReport" : "/reports/executiveSummaryReport";
    //window.open( reportsUrl );
    // this.router.navigate(['/reports/executiveSummaryReport']);
  }

  toggleDatabaseSubMenu() {
    this.toggleSubmenu = !this.toggleSubmenu;
  }

  toggleSubMenus(title) {
    if (title == 'users') {
      this.userMenuToggle = !this.userMenuToggle;
      this.S2SMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.databaseMenuToggle = false;
      this.policyMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Workloads) {
      this.S2SMenuToggle = !this.S2SMenuToggle;
      this.userMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.databaseMenuToggle = false;
      this.policyMenuToggle = false;
      this.alertsToggle = false;
      this.navItemsSelection(41);
    } else if (title == TitleEnum.Clients) {
      this.clientMenuToggle = !this.clientMenuToggle;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.infrastructureMenuToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Infrastructure) {
      this.infrastructureMenuToggle = !this.infrastructureMenuToggle;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.databaseMenuToggle = false;
      this.policyMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Device) {
      this.deviceMenuToggle = !this.deviceMenuToggle;
      this.softwaresListMenuToggle = false;
      this.reportsMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Reports) {
      this.reportsMenuToggle = !this.reportsMenuToggle;
      this.softwaresListMenuToggle = false;
      this.deviceMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Software) {
      this.softwareMenuToggle = !this.softwareMenuToggle;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.policyMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title == TitleEnum.Access) {
      this.accessMenuToggle = !this.accessMenuToggle;
      this.softwaresListMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.policyMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.settingMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.Settings) {
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.softwaresListMenuToggle = false;
      this.settingMenuToggle = !this.settingMenuToggle;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.policyMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.Database) {
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.databaseMenuToggle = !this.databaseMenuToggle;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.policyMenuToggle = false;
      this.settingMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.License) {
      this.accessPolicySubToggle = false;
      this.softwaresListMenuToggle = false;
      this.devicePolicySubToggle = false;
      this.databaseMenuToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.Dashboard) {
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.databaseMenuToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.PolicyMenu) {
      this.policyMenuToggle = !this.policyMenuToggle;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.AccessPolicySubMenu) {
      this.accessPolicySubToggle = !this.accessPolicySubToggle;
      this.devicePolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.DevicePolicySubMenu) {
      this.devicePolicySubToggle = !this.devicePolicySubToggle;
      this.accessPolicySubToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.alertsToggle = false;
    } else if (title === TitleEnum.AlertsMenu) {
      this.alertsToggle = !this.alertsToggle;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.policyMenuToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.softwaresListMenuToggle = false;
      this.clientMenuToggle = false;
      this.infrastructureMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
    } else if (title == TitleEnum.SoftwaresList) {
      this.softwaresListMenuToggle = !this.softwaresListMenuToggle;
      this.clientMenuToggle = false;
      this.accessPolicySubToggle = false;
      this.devicePolicySubToggle = false;
      this.infrastructureMenuToggle = false;
      this.userMenuToggle = false;
      this.S2SMenuToggle = false;
      this.deviceMenuToggle = false;
      this.reportsMenuToggle = false;
      this.softwareMenuToggle = false;
      this.accessMenuToggle = false;
      this.settingMenuToggle = false;
      this.policyMenuToggle = false;
      this.databaseMenuToggle = false;
      this.alertsToggle = false;
    }
  }

  setUserRole() {
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' && this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
  }

  getRolesData() {
    // this.assignedUserTemplates = localStorage.getItem('Roles');
  }

  getLicense() {
    this.licenseService.microSegLicense$.subscribe(value => {
      this.isS2sEnabled = value;
    });
  }

  ngOnDestroy(): void {
    this.authenticationManager.authStateChange.unsubscribe();
    this.destroy$.next();
    this.destroy$.complete();
  }

  navItemsSelection(requiredRole) {
    this.rolesPermissionService
      .getAssignedUserTemplates()
      .pipe(
        take(1), // Ensure the observable completes after emitting one value
        map((data: any) => {
          
          const permissions = data as any[];
          const filteredPermissions = permissions.filter(
            permission => permission.parent_id === requiredRole
          );
          
          if (filteredPermissions.length > 0) {
            // Get the first permission that matches the requiredRole
            const firstPermission = filteredPermissions[0];

            if (firstPermission.is_crud_allowed == 0) {
              const children = permissions.filter(
                permission =>
                  permission.parent_id === firstPermission.privilege_id
              );
                
              if (children.length > 0) {
                // Find the corresponding route_name in RolesData for the first child
                const routeName = RolesData.find(
                  roleData => roleData.privilege_id === children[0].privilege_id
                )?.route_name;

                if (routeName) {
                  // Navigate to the corresponding route for the first child
                  
                  this.router.navigate(['/' + routeName]);
                  this.devicePolicySubToggle = !this.devicePolicySubToggle;
                } else {
                  console.log(
                    'No route found for the matched privilege_id of the child'
                  );
                }
              } else {
                console.log('No children found for the matched privilege_id');
              }
            } else {
              // Find the corresponding route_name in RolesData
              const routeName = RolesData.find(
                roleData =>
                  roleData.privilege_id === firstPermission.privilege_id
              )?.route_name;
              
              if (routeName) {
                // Navigate to the corresponding route
                if (this.showLicenseOnly && routeName === 'services') {
                  const routeTo = 'controllers'
                  this.router.navigate(['/' + routeTo]);
                }
                
                else {
                  this.router.navigate(['/' + routeName]);
                }
                
                if(routeName == "access-policy"){
                  this.accessPolicySubToggle = true;
                }

              } else {
                console.log('No route found for the matched privilege_id');
              }
            }
          } else {
            // Handle the case when no permissions match the requiredRole
            // firstPermission check karna uska is crud, sirf policy managmenet main nahi hoga if entitlement na ho
            console.log('No permissions found for the required role');
          }
        })
      )
      .subscribe(
        () => {},
        error => {
          console.error('Error occurred:', error);
          // Handle error if necessary
        }
      );
  }
}
