import { DatePipe } from '@angular/common';
import { ChangeDetectorRef, Pipe, PipeTransform } from '@angular/core';
import { SettingsService } from '../settings/settings.service';
import { catchError, map } from 'rxjs';
import * as moment from 'moment-timezone';

@Pipe({
  name: 'dateTransform',
  pure: true
})
export class DateTransformPipe implements PipeTransform {
  private userTimeData: any = null;
  private loading = false;
  private userrole;

  constructor(
    private datePipe: DatePipe,
    private settingsService: SettingsService,
    private cdRef: ChangeDetectorRef
  ) {
    this.userTimeData =
      this.userTimeData || JSON.parse(localStorage.getItem('userTimeData'));
    this.userrole = localStorage.getItem('ur');
    if (typeof this.userrole != 'undefined' && this.userrole != null) {
      this.userrole = atob(this.userrole);
    }
    this.loading = true;
    if (!this.userTimeData && this.userrole == 'Admin') {
      this.settingsService
        .getTimeSettings()
        .pipe(map((data: any) => data?.timesetting))
        .subscribe({
          next: data => {
            this.userTimeData = data;
            this.loading = false;
            this.cdRef.markForCheck();
            localStorage.setItem('userTimeData', JSON.stringify(data));
          },
          error: err => catchError(err)
        });
    } else {
      this.loading = false;
    }
  }

  transform(utcDate: string, format: string = 'medium'): string {
    if (!this.loading) {
      if (utcDate) {
        if (this.userTimeData && this.userTimeData.time_type == 1) {
          return this.datePipe.transform(utcDate, format, 'UTC');
        } else if (this.userTimeData && this.userTimeData.time_type == 2) {
          const localDate = new Date(utcDate);
          return this.datePipe.transform(localDate, format);
        } else if (this.userTimeData && this.userTimeData.time_type == 3) {
          const matched = this.userTimeData.timezone.match(/(.*?)_GMT/);
          const extractedTimezone = matched ? matched[1] : null;
          const dateInTimezone = moment.tz(utcDate, extractedTimezone);
          return dateInTimezone.format('MMM D, YYYY, h:mm:ss A');
        }
      }
      return utcDate;
    }
  }
}
