import { Component, OnInit } from '@angular/core';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { Pagination } from '../../models/pagination';
import { PolicyService } from '../policy.service';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-service-group-detail',
  templateUrl: './service-group-detail.component.html',
  styleUrls: ['./service-group-detail.component.css']
})
export class ServiceGroupDetailComponent implements OnInit {

  services = []
  groupId;
  associatedServicesPagination: Pagination = new Pagination()
  associatedServicesSort = 'Name'
  associatedServicesSortDir = 'asc'
  servicesSearch = ''
  loading = false
  pageSize = [10, 25, 50, 100]
  showConfirm = false
  groupName;
  foundGroup: boolean = false
  
  

  constructor(
    private serviceGroupServices: PolicyService,
    private router: Router,
    private route: ActivatedRoute,
    private globalMessageService: GlobalMessageService,
    private titleService: Title
  ) { }

  ngOnInit() {
    this.groupId = this.route.snapshot.params.id
    this.getServiceByServiceGroupId()
  }


  getServiceByServiceGroupId() {
    this.loading = true
    this
      .serviceGroupServices
      .getGroupServicesWithPagination(
        this.groupId, 
        this.associatedServicesPagination.currentPage - 1, 
        this.associatedServicesPagination.itemsPerPage,
        this.associatedServicesSort,
        this.associatedServicesSortDir,
        this.servicesSearch
      ).subscribe(
        response => {
        this.associatedServicesPagination.totalItems = response.total
        this.services = response.data
        this.loading = false
        this.groupName = response.name
        this.foundGroup = true
        this.titleService.setTitle('Infrastructure | Service Groups | ' + this.groupName);
      },
      err => {
        this.loading = false
        this.foundGroup = false
      }
      
    )
  }

  servicesPaginationChange(page) {
    this.associatedServicesPagination.currentPage = page
    this.getServiceByServiceGroupId()
  }

  servicesPageSizeChange(total) {
    this.associatedServicesPagination.itemsPerPage = total
    this.associatedServicesPagination.currentPage = 1
    this.getServiceByServiceGroupId()
  }

  sortAssociatedServices(term) {
    this.associatedServicesSort = term;
    this.associatedServicesSortDir = this.associatedServicesSortDir == 'desc' ? 'asc' : 'desc';
    this.associatedServicesPagination.currentPage = 1
    this.getServiceByServiceGroupId()
  }

  handleServicesSearch() {
    this.associatedServicesPagination.currentPage = 1
    this.getServiceByServiceGroupId()
  }

  showConfirmation() {
    this.showConfirm = true
  }

  cancelHandler() {
    this.showConfirm = false
  }

  deleteHandler() {
    this.showConfirm = false
    this.serviceGroupServices.deleteGroup(this.groupId).subscribe(
      response => {
        this.globalMessageService.pushAlert({
          message: `'Gateway Group deleted`,
          type: 'success'
        });
        this.goBack()
      },
      err => {
        this.showConfirm = false;
      }
    );

  }

  goBack() {
    this.router.navigate(['/serviceGroups'])
  }

  editHandler() {
    this.router.navigate([`/create-service-group/${this.groupId}`])
  }


}
