import { Component, OnDestroy, OnInit } from '@angular/core';
import { AmChart, AmChartsService } from '@amcharts/amcharts3-angular';
import {ReportingService} from './reporting.service';
import * as moment from 'moment';
import { Pagination } from '../models/pagination';
import { APIDataTypeEnum } from '../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-reportings',
  templateUrl: './reportings.component.html',
  styleUrls: ['./reportings.component.css']
})
export class ReportingsComponent implements OnInit, OnDestroy {
  today: Date;
  firstChart: AmChart;
  secondChart: AmChart;
  servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  usersDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  serviceGraphDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  usersGraphDaterangepicker: Date = new Date();
  summaryOfAllUsers:any = [];
  summaryOfAllUsersGraph:any = [];
  summaryOfAllServices:any = [];
  summaryOfAllServicesGraph:any = [];
  ReportsOfAllUnassignedDevices:any =[];
  ReportsOfAllControllers:any = [];
  serviceSortDir: string = 'asc'
  serviceSort: string;
  deviceSortDir: string = 'asc'
  deviceSort: string;
  controllersActive: number;
  summaryOfAllControllerPagination: Pagination = new Pagination();
  summaryOfAllGatewaysPagination: Pagination = new Pagination();
  reportsOfAllControllersLoading:Boolean = true;
  reportsOfAllGatewaysLoading:Boolean = true;
  pageSizeSummaryOfController = [10,25,50,100,200];
  pageSizeSummaryOfGateways = [10,25,50,100,200];

  gatewaysActive: number;
  ReportsOfAllGateways:any = [];
  controllersDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  gatewaysDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  controllerSortDir: string = 'asc'
  controllerSort: string;
  gatewaysSortDir: string = 'asc'
  gatewaysSort: string;
  last_sync_time: string;
  directories_count: number;

  clientSortDir: string = 'asc'
  clientSort: string;


  selectedService: {
    serviceSummary: Array<any>,
    clientsSummary: Array<any>,
    serviceSummaryGraph: Array<any>,
    clientsSummaryGraph: Array<any>,
  } = {
    serviceSummary: null,
    clientsSummary: null,
    serviceSummaryGraph: null,
    clientsSummaryGraph: null
  };

  selectedClient: {
    serviceSummary: Array<any>,
    clientsSummary: Array<any>,
    serviceSummaryGraph: Array<any>,
    clientsSummaryGraph: Array<any>,
  } = {
    serviceSummary: null,
    clientsSummary: null,
    serviceSummaryGraph: null,
    clientsSummaryGraph: null
  };

  summaryOfAllServicesPagination: Pagination = new Pagination();
  summaryOfAllDevicesPagination: Pagination = new Pagination();
  summaryOfAllUsersPagination: Pagination = new Pagination();
  summaryOfAllControllersPagination: Pagination = new Pagination();
  services: any = [];
  clients: any = [];

  utilizationByServicesLoading:Boolean = true;
  hourlyDataConsumptionLoading:Boolean = true;
  summaryOfAllServicesLoading:Boolean = true;
  summaryOfAllClientsLoading:Boolean = true;
  reportsOfAllUnassignedDevicesLoading:Boolean = true;

  pageSizeSummaryOfUsers = [10,25,50,100,200];
  pageSizeSummaryOfServices = [10,25,50,100,200];
  pageSizeSummaryOfDevices = [10,25,50,100,200];

  ReportsOfAllUnusedPolicy:any = [];
  ReportsOfAllWithoutRoutingPolicy:any = [];
  ReportsOfAllUnassignedServicesPolicy:any = [];
  // servicesDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  unusedserviceSortDir: string = 'asc'
  unusedserviceSort: string;
  withoutroutingpolicySortDir: string = 'asc'
  withoutroutingpolicySort: string;
  UnassignedServicespolicySortDir: string = 'asc'
  UnassignedServicespolicySort: string;

  summaryOfAllUnusedServicesPagination: Pagination = new Pagination();
  summaryOfAllWithoutRoutingPolicyPagination: Pagination = new Pagination();
  summaryOfAllUnassignedServicesPolicyPagination: Pagination = new Pagination();
  reportsOfAllUnusedPolicyLoading:Boolean = true;
  reportsOfAllWithoutRoutingPolicyLoading:Boolean = true;
  reportsOfAllUnassignedServicesPolicyLoading:Boolean = true;
  pageSizeSummaryOfUnusedServices = [10,25,50,100,200];
  pageSizeSummaryOfWithoutRoutingPolicy = [10,25,50,100,200];
  pageSizeSummaryOfUnassignedServicesPolicy = [10,25,50,100,200];

  summaryOfAllCurrentUsers:any = [];
  summaryOfAllFailedUsers:any = [];

  currentuserSortDir: string = 'asc'
  currentuserSort: string;

  faileduserSortDir: string = 'asc'
  faileduserSort: string;

  summaryOfAllCurrentUsersPagination: Pagination = new Pagination();
  summaryOfAllFailedUsersPagination: Pagination = new Pagination();

  summaryOfAllCurrentUsersLoading:Boolean = true;
  summaryOfAllFailedUsersLoading:Boolean = true;

  pageSizeSummaryOfCurrentUsers = [10,25,50,100,200];
  pageSizeSummaryOfFailedUsers = [10,25,50,100,200];

  OShashMap = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    windows: 'Windows',
    windowsserver: 'Windows Server',
    centosserver: 'CentOS Server',
    macintosh: 'Macintosh',
    ios: 'iOS',
    android: 'Android',
    unknown: 'Unknown'
  };
  isDarkMode: any;
  constructor (
    private AmCharts: AmChartsService,
    private reportingService:ReportingService,
    private titleService: Title
  ) {
    this.today = new Date();
  }

  ngOnInit() {
    this.isDarkMode = JSON.parse(localStorage.getItem('darkMode'))

    this.titleService.setTitle('Reports | All Reports');
    this.summaryOfAllUsersPagination.id='users';
    this.summaryOfAllServicesPagination.id='services';
    this.summaryOfAllDevicesPagination.id='devices_reports';
    this.summaryOfAllControllersPagination.id='controllers_reports';
    this.summaryOfAllGatewaysPagination.id='gateways_reports';
    this.summaryOfAllUnusedServicesPagination.id='policy_unused_reports';
    this.summaryOfAllWithoutRoutingPolicyPagination.id='policy_withoutroutingpolicy_reports';
    this.summaryOfAllUnassignedServicesPolicyPagination.id='policy_unassignedservices_reports';
    this.summaryOfAllCurrentUsersPagination.id='current-users';
    this.summaryOfAllFailedUsersPagination.id='failed-users';
    this.setAmChartFont;
    this.getSummaryOfAllCurrentUsers(this.summaryOfAllCurrentUsersPagination);
    this.getSummaryOfAllFailedUser(this.summaryOfAllFailedUsersPagination);
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllUnusedServicesPagination);
    this.getReportsOfAllWithoutRoutingPolicy(this.summaryOfAllWithoutRoutingPolicyPagination);
    this.getReportsOfAllUnassignedServicesPolicy(this.summaryOfAllUnassignedServicesPolicyPagination);
    this.getReportsOfAllControllers(this.summaryOfAllControllersPagination);
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination);
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
    this.getReportsOfAllUnassignedDevices(this.summaryOfAllDevicesPagination);
    this.getAllServices();
    this.getAllClients();
  }

  setAmChartFont() {
    this.AmCharts.addInitHandler(function(chart) {
      // Set the default fontFamily for the chart
      chart.fontFamily = 'Poppins';
    }, ["serial", "pie", "xy"]);
  }

  ngAfterViewInit() {    
    this.firstChart = this.AmCharts.makeChart('firstChart', {
      type: "serial",
      theme: "light",
      dataProvider: [],
      hideCredits:true,
      valueAxes: [ {
        title: "Data consumed (MB)",
        gridColor: this.isDarkMode ? '#eee' : '#353d4a',
        gridAlpha: 0.2,
        dashLength: 0,
        color: this.isDarkMode ? '#eee' : '#353d4a',
        titleColor: this.isDarkMode ? '#eee' : '#353d4a',
      } ],
      gridAboveGraphs: true,
      startDuration: 1,
      graphs: [ {
        balloonText: "[[category]]: <b style='color:#111111;'>[[value]]</b>",
        fillColorsField: "color",
        fillAlphas: 0.8,
        lineAlpha: 0.2,
        type: "column",
         valueField: "value",
        color: this.isDarkMode ? '#eee' : '#353d4a',
      } ],
      chartCursor: {
        categoryBalloonEnabled: false,
        cursorAlpha: 0,
        zoomable: false
      },
      categoryField: "name",
      categoryAxis: {
        gridPosition: "start",
        gridAlpha: 0,
        tickPosition: "start",
        tickLength: 0,
        color: this.isDarkMode ? '#eee' : '#353d4a',
        labelFunction: function(label, item, axis) {

          var chart = axis.chart;
            if ( ( label.length > 8 ) ){
              return label.substr(0, 9) + '...';
            }
            if ( ( label.length > 11 ) ){
              return label.substr(0, 12) + '...';
            }
            return label;
          }
      },
      rotate: true
    });
    this.secondChart = this.AmCharts.makeChart('seondChart', {
      "type": "serial",
      "theme": "light",
      "dataProvider": [],
      "hideCredits":true,
      "valueAxes": [ {
        "title": "Data consumed (MB)",
        "gridColor":  this.isDarkMode ? '#eee' : '#353d4a',
        "gridAlpha": 0.2,
        "dashLength": 0,
        "color": this.isDarkMode ? '#eee' : '#353d4a',
        "titleColor": this.isDarkMode ? '#eee' : '#353d4a',
      } ],
      "gridAboveGraphs": true,
      "startDuration": 1,
      "graphs": [ {
        "balloonText": "[[category]]: <b style='color:#111111;'>[[value]]</b>",
        "fillColorsField": "color",
        "fillAlphas": 0.8,
        "lineAlpha": 0.2,
        "type": "column",
        "valueField": "total_volume",
        "color": this.isDarkMode ? '#eee' : '#353d4a',
      } ],
      "chartCursor": {
        "categoryBalloonEnabled": false,
        "cursorAlpha": 0,
        "zoomable": false
      },
      "categoryField": "hours",
      "categoryAxis": {
        "title": "Hours in a day",
        "gridPosition": "start",
        "gridAlpha": 0,
        "tickPosition": "start",
        "tickLength": 0,
        "color": this.isDarkMode ? '#eee' : '#353d4a',
        "titleColor": this.isDarkMode ? '#eee' : '#353d4a',
        "labelFunction": function(label, item, axis) {

          var chart = axis.chart;
            if ( ( label.length > 8 ) ){
              return label.substr(0, 9) + '...';
            }
            if ( ( label.length > 11 ) ){
              return label.substr(0, 12) + '...';
            }
            return label;
          }
      }
    });
    this.getSummaryOfAllServicesGraph();
    this.getSummaryOfAllUserGraph();
  }

  ngOnDestroy() {
    if (this.firstChart) {
      this.AmCharts.destroyChart(this.firstChart);
    }

    if (this.secondChart) {
      this.AmCharts.destroyChart(this.secondChart);
    }
  }

  getSummaryOfAllServicesGraph() {
    this.utilizationByServicesLoading = true;
    this.reportingService
    .getServicesSummaryGraph(
      this.serviceGraphDaterangepicker[0],
      this.serviceGraphDaterangepicker[1],
      this.selectedService.serviceSummaryGraph,
      this.selectedClient.serviceSummaryGraph,
    )
    .subscribe(data=>{
      if(data.length>0){
        data.map((d)=>{
          Object.assign(d, {color: "#39C6A5"});
        });
      }
      this.summaryOfAllServicesGraph = data;
      this.AmCharts.updateChart(this.firstChart, () => {
        this.firstChart.dataProvider = data;
        this.utilizationByServicesLoading = false;
      });
    });
  }


  getSummaryOfAllUserGraph() {
    this.hourlyDataConsumptionLoading = true;
    this.reportingService
    .getUsersSummaryGraph(
      this.usersGraphDaterangepicker,
      this.selectedService.clientsSummaryGraph,
      this.selectedClient.clientsSummaryGraph,
    )
    .subscribe(data=>{
      // let dt = [
      //   {"id":700,"total_volume":"0.0921","hours":7
      //   },
      //   {"id":761,"total_volume":"0.0081","hours":8
      //   },
      //   {"id":761,"total_volume":"0.0221","hours":9
      //   },
      //   {"id":761,"total_volume":"182.8824","hours":11
      //   },
      //   {"id":700,"total_volume":"0.1834","hours":12
      //   }];
      if(data.length>0){
        data.map((d)=>{
          Object.assign(d, {color: "#39C6A5"});
        });
      }
      this.summaryOfAllUsersGraph = data;
      this.AmCharts.updateChart(this.secondChart, () => {
        this.secondChart.dataProvider = data;
        this.hourlyDataConsumptionLoading = false;
      });
    });
  }


  getSummaryOfAllServices(pagination: Pagination){
    this.summaryOfAllServicesLoading = true;
    this.reportingService.
    getServicesSummary(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.selectedService.serviceSummary,
      this.selectedClient.serviceSummary,
      this.serviceSortDir,
      this.serviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllServices = summary;
        this.summaryOfAllServicesLoading = false;
      },
      error: err=>{
        this.summaryOfAllServicesLoading = false;
      }
    })
  }

  getSummaryOfAllUser(pagination: Pagination){
    this.summaryOfAllClientsLoading = true;
    this.reportingService.
    getUsersSummary(
      pagination,
      this.usersDaterangepicker[0],
      this.usersDaterangepicker[1],
      this.selectedService.clientsSummary,
      this.selectedClient.clientsSummary,
      this.clientSortDir,
      this.clientSort
    )
    .subscribe({
      next: summary=>{
        this.summaryOfAllClientsLoading = false;
        this.summaryOfAllUsersPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllUsers = summary;
      },
      error: err=>{
        this.summaryOfAllClientsLoading = false;
      }
    })
  }

  pageChangeSummaryOfAllServices(page:any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
  }


  pageChangeSummaryOfAllUsers(page:any) {
    this.summaryOfAllUsersPagination.currentPage = page;
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
  }

  clearDate (type) {
    if(type === APIDataTypeEnum.Service) {
      this.servicesDaterangepicker = [];
      this.serviceDateChange();
    } else if (type === APIDataTypeEnum.User) {
      this.usersDaterangepicker = [];
      this.userDateChange();
    } else if (type === APIDataTypeEnum.ServiceGraph) {
      this.serviceGraphDaterangepicker = [];
      this.onDateChange(type);
    } else if (type === APIDataTypeEnum.UserGraph) {
      this.usersGraphDaterangepicker = null;
      this.onDateChange(type);
    }
  }


  onDateChange(type) {
    if(type === APIDataTypeEnum.Service) {
      if(!this.servicesDaterangepicker)
        this.servicesDaterangepicker = [];
      this.summaryOfAllServicesPagination.currentPage = 1;
      // this.summaryOfAllServicesPagination.itemsPerPage = 10;
      this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
    } else if (type === APIDataTypeEnum.User) {
      if(!this.servicesDaterangepicker)
        this.servicesDaterangepicker = [];
      this.summaryOfAllUsersPagination.currentPage = 1;
      // this.summaryOfAllUsersPagination.itemsPerPage = 10;
      this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
    } else if (type === APIDataTypeEnum.ServiceGraph) {
      if(!this.serviceGraphDaterangepicker)
        this.serviceGraphDaterangepicker = [];
      this.getSummaryOfAllServicesGraph()
    } else if (type === APIDataTypeEnum.UserGraph) {
      if(!this.usersGraphDaterangepicker)
        this.usersGraphDaterangepicker = null;
      this.getSummaryOfAllUserGraph()
    }
  }

  userDateChange(){
    if(!this.servicesDaterangepicker)
      this.servicesDaterangepicker = [];
    this.summaryOfAllUsersPagination.currentPage = 1;
    // this.summaryOfAllUsersPagination.itemsPerPage = 10;
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
  }

  serviceDateChange(){
    if(!this.servicesDaterangepicker)
      this.servicesDaterangepicker = [];
    this.summaryOfAllServicesPagination.currentPage = 1;
    // this.summaryOfAllServicesPagination.itemsPerPage = 10;
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
  }

  onChangeService(type: string, value) {
    if(type === APIDataTypeEnum.Service) {
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.selectedService.serviceSummary = value;
      this.getSummaryOfAllServices(this.summaryOfAllServicesPagination)
    } else if (type === APIDataTypeEnum.User) {
      this.summaryOfAllUsersPagination.currentPage = 1;
      this.selectedService.clientsSummary = value;
      this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
    } else if (type === APIDataTypeEnum.ServiceGraph) {
      this.selectedService.serviceSummaryGraph = value;
      this.getSummaryOfAllServicesGraph();
    } else if (type === APIDataTypeEnum.UserGraph) {
      this.selectedService.clientsSummaryGraph = value;
      this.getSummaryOfAllUserGraph();
    }
  }

  onChangeClient(type: string, value) {
    if(type === APIDataTypeEnum.Service) {
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.selectedClient.serviceSummary = value;
      this.getSummaryOfAllServices(this.summaryOfAllServicesPagination)
    } else if (type === APIDataTypeEnum.User) {
      this.summaryOfAllUsersPagination.currentPage = 1;
      this.selectedClient.clientsSummary = value;
      this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
    } else if (type === APIDataTypeEnum.ServiceGraph) {
      this.selectedClient.serviceSummaryGraph = value;
      this.getSummaryOfAllServicesGraph();
    } else if (type === APIDataTypeEnum.UserGraph) {
      this.selectedClient.clientsSummaryGraph = value;
      this.getSummaryOfAllUserGraph();
    }
  }

  getAllServices(){
    this.reportingService.getAllServices().subscribe(services=>{
      this.services = services;
    });
  }

  getAllClients(){
    this.reportingService.getAllClients().subscribe(clients=>{
      this.clients = clients;
    });
  }

  getReportsOfAllUnassignedDevices(pagination: Pagination){
    this.reportsOfAllUnassignedDevicesLoading = true;
    this.reportingService.
    getDevicesReports(
      pagination,
      this.deviceSortDir,
      this.deviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllDevicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnassignedDevices = summary;
        this.reportsOfAllUnassignedDevicesLoading = false;
      },
      error: err=>{
        this.reportsOfAllUnassignedDevicesLoading = false;
      }
    })
  }

  pageChangeSummaryOfAllDevices(page:any) {
    this.summaryOfAllDevicesPagination.currentPage = page;
    this.getReportsOfAllUnassignedDevices(this.summaryOfAllDevicesPagination);
  }


  // sortDeviceRecord(term: string){
  //   this.deviceSortDir = this.deviceSortDir == 'asc' ? 'desc' : 'asc';
  //   this.deviceSort = term;
  //   this.getReportsOfAllUnassignedDevices(this.summaryOfAllDevicesPagination)
  // }


  getReportsOfAllControllers(pagination: Pagination){
    this.reportsOfAllControllersLoading = true;
    this.reportingService.
    getControllersReports(
      pagination,
      this.controllersDaterangepicker[0],
      this.controllersDaterangepicker[1],
      this.controllerSortDir,
      this.controllerSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllControllerPagination.totalItems = summary.result.recordsTotal;
        this.ReportsOfAllControllers = summary.result.data;
        this.controllersActive = summary.controllersActive;
        this.reportsOfAllControllersLoading = false;
      },
      error: err=>{
        this.reportsOfAllControllersLoading = false;
      }
    })
  }

  getReportsOfAllGateways(pagination: Pagination){
    this.reportsOfAllGatewaysLoading = true;
    this.reportingService.
    getGatewaysReports(
      pagination,
      this.gatewaysDaterangepicker[0],
      this.gatewaysDaterangepicker[1],
      this.gatewaysSortDir,
      this.gatewaysSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllGatewaysPagination.totalItems = summary.result.recordsTotal;
        this.ReportsOfAllGateways = summary.result;
        this.last_sync_time = summary.last_sync_time.last_ldap_sync_end_time;
        this.directories_count = summary.directories_count;
        this.gatewaysActive = summary.gatewaysActive;
        this.reportsOfAllGatewaysLoading = false;
      },
      error: err=>{
        this.reportsOfAllGatewaysLoading = false;
      }
    })
  }


  getReportsOfAllUnusedPolicy(pagination: Pagination){
    this.reportsOfAllUnusedPolicyLoading = true;
    this.reportingService.
    getUnusedPolicyReports(
      pagination,
      this.servicesDaterangepicker[0],
      this.servicesDaterangepicker[1],
      this.unusedserviceSortDir,
      this.unusedserviceSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllUnusedServicesPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnusedPolicy = summary;
        this.reportsOfAllUnusedPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllUnusedPolicyLoading = false;
      }
    })
  }


  getReportsOfAllWithoutRoutingPolicy(pagination: Pagination){
    this.reportsOfAllWithoutRoutingPolicyLoading = true;
    this.reportingService.
    getWithoutRoutingPolicyReports(
      pagination,
      this.withoutroutingpolicySortDir,
      this.withoutroutingpolicySort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllWithoutRoutingPolicyPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllWithoutRoutingPolicy = summary;
        this.reportsOfAllWithoutRoutingPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllWithoutRoutingPolicyLoading = false;
      }
    })
  }


  getReportsOfAllUnassignedServicesPolicy(pagination: Pagination){
    this.reportsOfAllUnassignedServicesPolicyLoading = true;
    this.reportingService.
    getUnassignedServicesPolicyReports(
      pagination,
      this.UnassignedServicespolicySortDir,
      this.UnassignedServicespolicySort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllUnassignedServicesPolicyPagination.totalItems = summary.recordsTotal;
        this.ReportsOfAllUnassignedServicesPolicy = summary;
        this.reportsOfAllUnassignedServicesPolicyLoading = false;
      },
      error: err=>{
        this.reportsOfAllUnassignedServicesPolicyLoading = false;
      }
    })
  }

  getSummaryOfAllCurrentUsers(pagination: Pagination){
    this.summaryOfAllCurrentUsersLoading = true;
    this.reportingService.
    getUserReports(
      pagination,
      this.currentuserSortDir,
      this.currentuserSort
    )
    .subscribe({
      next: summary => {
        this.summaryOfAllCurrentUsersPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllCurrentUsers = summary;
        this.summaryOfAllCurrentUsersLoading = false;
      },
      error: err=>{
        this.summaryOfAllCurrentUsersLoading = false;
      }
    })
  }

  getSummaryOfAllFailedUser(pagination: Pagination){
    this.summaryOfAllFailedUsersLoading = true;
    this.reportingService.
    getUserAuthenticationFailedReports(
      pagination,
      this.faileduserSortDir,
      this.faileduserSort
    )
    .subscribe({
      next: summary=>{
        this.summaryOfAllFailedUsersLoading = false;
        this.summaryOfAllFailedUsersPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllFailedUsers = summary;
      },
      error: err=>{
        this.summaryOfAllFailedUsersLoading = false;
      }
    })
  }

  pageChangeSummaryOfAllUnusedServices(page:any) {
    this.summaryOfAllUnusedServicesPagination.currentPage = page;
    this.getReportsOfAllUnusedPolicy(this.summaryOfAllUnusedServicesPagination);
  }

  pageChangeSummaryOfAllWithoutRoutingPolicy(page:any) {
    this.summaryOfAllWithoutRoutingPolicyPagination.currentPage = page;
    this.getReportsOfAllWithoutRoutingPolicy(this.summaryOfAllWithoutRoutingPolicyPagination);
  }

  pageChangeSummaryOfAllUnassignedServicesPolicy(page:any) {
    this.summaryOfAllUnassignedServicesPolicyPagination.currentPage = page;
    this.getReportsOfAllUnassignedServicesPolicy(this.summaryOfAllUnassignedServicesPolicyPagination);
  }

  pageChangeSummaryOfAllControllers(page:any) {
    this.summaryOfAllControllersPagination.currentPage = page;
    this.getReportsOfAllControllers(this.summaryOfAllControllersPagination);
  }

  pageChangeSummaryOfAllGateways(page:any) {
    this.summaryOfAllGatewaysPagination.currentPage = page;
    this.getReportsOfAllGateways(this.summaryOfAllGatewaysPagination);
  }

  pageChangeSummaryOfAllCurrentUsers(page:any) {
    this.summaryOfAllCurrentUsersPagination.currentPage = page;
    this.getSummaryOfAllCurrentUsers(this.summaryOfAllCurrentUsersPagination);
  }


  pageChangeSummaryOfAllFailedUsers(page:any) {
    this.summaryOfAllFailedUsersPagination.currentPage = page;
    this.getSummaryOfAllFailedUser(this.summaryOfAllFailedUsersPagination);
  }

  pageSizeChangeSummaryOfAllServices(size){
    this.summaryOfAllServicesPagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
    }, 0);

  }

  pageSizeChangeSummaryOfAllUsers(size){
    this.summaryOfAllUsersPagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.summaryOfAllUsersPagination.currentPage = 1;
      this.getSummaryOfAllServices(this.summaryOfAllUsersPagination);
    }, 0);

  }

  summaryOfAllUsersResultCount(){
    const pagination = this.summaryOfAllUsersPagination;
    let start = (pagination.currentPage*pagination.itemsPerPage)-(pagination.itemsPerPage-1);
    let end = pagination.currentPage*pagination.itemsPerPage;
    start = start > 0 ? start : 1;
    start = pagination.totalItems > 0 ? start : 0;
    end = pagination.totalItems > end ? end : pagination.totalItems;
    return `Showing ${start} to ${end} of ${pagination.totalItems} entries`;
  }


  summaryOfAllServicesResultCount(){
    const pagination = this.summaryOfAllServicesPagination;
    let start = (pagination.currentPage*pagination.itemsPerPage)-(pagination.itemsPerPage-1);
    let end = pagination.currentPage*pagination.itemsPerPage;
    start = start > 0 ? start : 1;
    start = pagination.totalItems > 0 ? start : 0;
    end = pagination.totalItems > end ? end : pagination.totalItems;
    return `Showing ${start} to ${end} of ${pagination.totalItems} entries`;
  }

  sortServiceRecord(term: string) {
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;  
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
  }

  sortClientRecord(term: string){
    this.clientSortDir = this.clientSortDir == 'asc' ? 'desc' : 'asc';
    this.clientSort = term;
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination)
  }

  getOSFullName(key){
    return this.OShashMap[key];
  }


}