import { FlatTreeControl } from '@angular/cdk/tree';
import { Component } from '@angular/core';
import { Validators, FormBuilder } from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatDialog } from '@angular/material/dialog';
import {
  MatTreeFlattener,
  MatTreeFlatDataSource
} from '@angular/material/tree';
import { Title } from '@angular/platform-browser';
import { ActivatedRoute, Router } from '@angular/router';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { RolesAndPermissionService } from '../roles-and-permission.service';

@Component({
  selector: 'app-edit-roles-and-permission',
  templateUrl: './edit-roles-and-permission.component.html',
  styleUrls: ['./edit-roles-and-permission.component.css']
})
export class EditRolesAndPermissionComponent {
  firstFormGroup = this._formBuilder.group({
    firstCtrl: ['', Validators.nullValidator]
  });
  secondFormGroup = this._formBuilder.group({
    secondCtrl: ['', Validators.nullValidator]
  });
  isEditable = true;
  name: string = '';
  description: string = '';

  private _transformer = (node: NavigationNode, level: number) => {
    return {
      expandable: !!node.children && node.children.length > 0,
      name: node.name,
      level: level,
      isSelected: node.isSelected || false,
      indeterminate: false,
      privilege_id: node.privilege_id,
      nodeId: node.nodeId
    };
  };

  treeControl = new FlatTreeControl<ExampleFlatNode>(
    node => node.level,
    node => node.expandable
  );

  treeFlattener = new MatTreeFlattener(
    this._transformer,
    node => node.level,
    node => node.expandable,
    node => node.children
  );

  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  UPDATED_TREE;

  templates: any = [];
  allPermissions: boolean = false;
  id: number;
  editedTemplate;

  constructor(
    private globalMessageService: GlobalMessageService,
    private rolesPermissionService: RolesAndPermissionService,
    private router: Router,
    private route: ActivatedRoute,
    public dialog: MatDialog,
    private titleService: Title,
    private _formBuilder: FormBuilder
  ) {}

  ngOnInit() {
    this.dataSource.data = TREE_DATA;
    this.titleService.setTitle('Edit Roles And Permissions');
    this.route.params.subscribe(params => {
      this.id = params['id'];
      if (this.id) {
        this.rolesPermissionService
          .getAssignedUserTemplates()
          .subscribe((templates: any) => {
            this.dataSource.data = this.filterNodes(
              templates,
              this.dataSource.data
            );
            this.setInitialExpansionState();

            this.rolesPermissionService
              .getAllTemplates()
              .subscribe((res: any) => {
                this.editedTemplate = res.filter(
                  template => template?.temp_id == this.id
                )[0];
                this.setExistingTemplate(this.editedTemplate);
              });
          });
      }
    });
  }

  private setInitialExpansionState() {
    // Assuming you want to expand all nodes by default
    this.treeControl.dataNodes.forEach(node => {
      this.treeControl.expand(node);
    });
  }

  ngOnDestroy() {}

  onChangeTemplate(template) {}

  hasChild = (_: number, node: ExampleFlatNode) => node.expandable;

  autoSelect(event: any, node: ExampleFlatNode) {
    node.isSelected = event.checked;
    node.indeterminate = false;
    node.createSelected = node.readSelected = node.updateSelected = node.deleteSelected = false;
    this.propagateSelection(node, event.checked);
    this.propagateChildrenSelection(node, event.checked);
    this.clearCreateSelectedForChildren(node);
    this.updateParentSelection(node);
  }

  clearCreateSelectedForChildren(node: ExampleFlatNode) {
    const stack = [...this.treeControl.getDescendants(node)];
    while (stack.length > 0) {
      const current = stack.pop();

      if (current) {
        current.createSelected = current.readSelected = current.updateSelected = current.deleteSelected = false;
        const descendants = this.treeControl.getDescendants(current);
        stack.push(...descendants);
      }
    }
  }

  handleCancelButton() {
    this.router.navigate(['/view_roles_and_permissions']);
  }

  filterNodes(nodes, dataNodes) {
    const nodeIds = nodes.map(node => node.privilege_id);

    return dataNodes.reduce((filteredResult, node) => {
      if (nodeIds.includes(node.privilege_id)) {
        const filteredNode = { ...node };
        if (node.children) {
          filteredNode.children = this.filterNodes(nodes, node.children);
        }
        filteredResult.push(filteredNode);
      } else if (node.children) {
        const selectedParentNodes = this.filterNodes(nodes, node.children);
        if (selectedParentNodes.length > 0) {
          const parentNode = { ...node, children: selectedParentNodes };
          filteredResult.push(parentNode);
        }
      }
      return filteredResult;
    }, [] as NavigationNode[]);
  }

  onParentCheckboxClick(event: any, node: ExampleFlatNode) {
    node.isSelected = event.checked;
    node.indeterminate = false;
    this.propagateSelection(node, event.checked);
    this.propagateChildrenSelection(node, event.checked);

    this.updateParentSelection(node);
  }

  handleDisable(dataSource) {
    const dataNodes = dataSource?._treeControl?.dataNodes;
    const selectedNodes = dataNodes.filter(
      (node: NavigationNode) => node?.isSelected
    );
  
    if (selectedNodes.length > 0) {
      const isEnabled = selectedNodes.every((node) =>
        node.createSelected || node.readSelected || node.deleteSelected || node.updateSelected
      );
      return !isEnabled;
    } else {
      return true;
    }
  }

  updateParentSelection(node: ExampleFlatNode) {
    let parent = this.findParent(node);

    while (parent) {
      const descendants = this.treeControl.getDescendants(parent);
      const allSelected = descendants.every(child => child.isSelected);
      const someSelected = descendants.some(child => child.isSelected);

      if (allSelected) {
        parent.isSelected = true;
        parent.indeterminate = false;
      } else if (someSelected) {
        parent.isSelected = false;
        parent.indeterminate = true;
      } else {
        parent.isSelected = false;
        parent.indeterminate = false;
      }

      parent = this.findParent(parent);
    }
  }

  getAllTemplates() {
    this.rolesPermissionService
      .getAllTemplates()
      .subscribe((templates: any) => {
        this.templates = templates.map(template => template?.name);
      });
  }

  onCheckboxChange(
    action: string,
    event: MatCheckboxChange,
    node: ExampleFlatNode
  ) {
    const property = `${action}Selected`;

    const stack = [...this.treeControl.getDescendants(node)];
    if (stack.length > 0) {
      node[property] = event.checked;
      while (stack.length > 0) {
        const current = stack.pop();

        if (current) {
          current[property] = event.checked;
          const descendants = this.treeControl.getDescendants(current);
          stack.push(...descendants);
        }
      }
    } else {
      node[property] = event.checked;
    }
  }

  handleData(dataSource) {
    const permissions = [];
    const dataNodes = dataSource?._treeControl?.dataNodes;
    const selectedNodes = dataNodes.filter(
      (node: NavigationNode) => node?.isSelected
    );
    if (selectedNodes) {
      selectedNodes.forEach(node => {
        const permission = {
          create_permission: node.createSelected ? 1 : 0,
          read_permission: node.readSelected ? 1 : 0,
          update_permission: node.updateSelected ? 1 : 0,
          delete_permission: node.deleteSelected ? 1 : 0,
          privilege_id: node.privilege_id
        };
        permissions.push(permission);
      });
    }

    this.rolesPermissionService
      .updateTemplate(this.id, {
        name: this.name,
        description: this.description,
        permissions: permissions
      })
      .subscribe(res => {
        if (res) {
          this.globalMessageService.pushAlert({
            message: `template updated successfully`,
            type: 'success'
          });
          this.router.navigate(['view_roles_and_permissions']);
        }
      });
  }

  handleBack() {
    this.router.navigate(['view_roles_and_permissions']);
  }

  getAllTemp() {
    this.rolesPermissionService.getAllTemplates().subscribe(res => {
      if (res) {
        this.globalMessageService.pushAlert({
          message: `template read successfully`,
          type: 'success'
        });
      }
    });
  }

  getTempById(id) {
    this.rolesPermissionService.getTemplatebyId(id).subscribe(res => {
      if (res) {
        this.globalMessageService.pushAlert({
          message: `template read by id successfully`,
          type: 'success'
        });
      }
    });
  }

  deleteTemplate(id) {
    this.rolesPermissionService.deleteTemplate(id).subscribe(res => {
      if (res) {
        this.globalMessageService.pushAlert({
          message: `template deleted successfully`,
          type: 'success'
        });
      }
    });
  }

  setExistingTemplate(templateData: any) {
    this.name = templateData?.name;
    this.description = templateData?.description;
    templateData.permissions.forEach(permission => {
      const node = this.findNodeByPrivilegeId(permission.privilege_id);
      if (node) {
        node.createSelected = permission.create_permission;
        node.readSelected = permission.read_permission;
        node.updateSelected = permission.update_permission;
        node.deleteSelected = permission.delete_permission;
        if (
          permission.create_permission == 1 ||
          permission.read_permission == 1 ||
          permission.update_permission == 1 ||
          permission.delete_permission == 1
        ) {
          node.isSelected = true;
        }
        this.updateParentSelection(node);
      }
    });
  }

  findNodeByPrivilegeId(privilegeId: number): ExampleFlatNode | null {
    return (
      this.treeControl.dataNodes.find(
        node => node.privilege_id === privilegeId
      ) || null
    );
  }

  findClosestAncestor(element: any, className: string): HTMLElement | null {
    let currentElement = element.parentElement;

    while (currentElement) {
      if (currentElement.classList.contains(className)) {
        return currentElement;
      }

      currentElement = currentElement.parentElement;
    }

    return null;
  }

  propagateSelection(node: ExampleFlatNode, isSelected: boolean) {
    const descendants = this.treeControl.getDescendants(node);
    descendants.forEach(child => {
      child.isSelected = isSelected;
      this.treeControl.dataNodes.forEach(dataNode => {
        if (dataNode === child) {
          dataNode.isSelected = isSelected;
        }
      });
    });
  }

  propagateParentSelection(node: ExampleFlatNode) {
    let parent = this.findParent(node);

    while (parent) {
      const descendants = this.treeControl.getDescendants(parent);
      const allSelected = descendants.every(child => child.isSelected);
      const someSelected = descendants.some(child => child.isSelected);

      if (allSelected) {
        parent.isSelected = true;
        this.treeControl.dataNodes.forEach(dataNode => {
          if (dataNode === parent) {
            dataNode.isSelected = true;
          }
        });
      } else if (someSelected) {
        parent.isSelected = false;
        this.treeControl.dataNodes.forEach(dataNode => {
          if (dataNode === parent) {
            dataNode.isSelected = false;
          }
        });
      } else {
        parent.isSelected = false;
      }

      parent = this.findParent(parent);
    }
  }

  propagateChildrenSelection(node: ExampleFlatNode, isSelected: boolean) {
    const stack = [...this.treeControl.getDescendants(node)];

    while (stack.length > 0) {
      const current = stack.pop();

      if (current) {
        current.isSelected = isSelected;
        const descendants = this.treeControl.getDescendants(current);
        stack.push(...descendants);
      }
    }
  }

  findParent(node: ExampleFlatNode): ExampleFlatNode | null {
    const currentLevel = node.level;

    for (let i = node.index - 1; i >= 0; i--) {
      if (this.treeControl.dataNodes[i].level < currentLevel) {
        return this.treeControl.dataNodes[i];
      }
    }

    return null;
  }
}

interface NavigationNode {
  privilege_id: number;
  nodeId: string;
  name: string;
  isSelected?: boolean;
  createSelected?: boolean;
  readSelected?: boolean;
  updateSelected?: boolean;
  deleteSelected?: boolean;
  createPermission?: string;
  children?: NavigationNode[];
}

interface ExampleFlatNode {
  expandable: boolean;
  name: string;
  level: number;
  isSelected: boolean;
  createSelected?: boolean;
  readSelected?: boolean;
  updateSelected?: boolean;
  deleteSelected?: boolean;
  index: number;
  children?: ExampleFlatNode[];
  indeterminate: boolean;
  privilege_id: number;
  nodeId: string;
  createPermission?: string;
}

const TREE_DATA: NavigationNode[] = [
  {
    privilege_id: 1,
    nodeId: '1',
    name: 'Dashboard',
    createPermission: '0',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: []
  },
  {
    privilege_id: 62,
    nodeId: '62',
    name: 'Infrastructure Dashboard',
    createPermission: '0',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: []
  },
  {
    privilege_id: 2,
    nodeId: '2',
    name: 'Reports',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 3,
        nodeId: '2.1',
        name: 'All Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: []
      },
      {
        privilege_id: 4,
        nodeId: '2.2',
        name: 'Services Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: []
      },
      {
        privilege_id: 5,
        nodeId: '2.3',
        name: 'Devices Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: []
      },
      {
        privilege_id: 6,
        nodeId: '2.4',
        name: 'Infrastructure Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 7,
        nodeId: '2.5',
        name: 'Policy Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 8,
        nodeId: '2.6',
        name: 'Users Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 63,
        nodeId: '2.7',
        name: 'Onboarding Reports',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
      }
    ]
  },
  {
    privilege_id: 9,
    nodeId: '3',
    name: 'Alerts',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: []
  },
  {
    privilege_id: 10,
    nodeId: '4',
    name: 'Policy Management',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 11,
        nodeId: '4.1.1',
        name: 'Entitlements',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: [
          {
            privilege_id: 12,
            nodeId: '4.1.2',
            name: 'Device Entitlements',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          },
          {
            privilege_id: 13,
            nodeId: '4.1.3',
            name: 'User Entitlements',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          }
        ]
      },
      {
        privilege_id: 14,
        nodeId: '4.2.1',
        name: 'Device Policy',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: [
          {
            privilege_id: 15,
            nodeId: '4.2.2',
            name: 'Security Policy',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          },
          {
            privilege_id: 16,
            nodeId: '4.2.3',
            name: 'OS Policy',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          },
          {
            privilege_id: 17,
            nodeId: '4.2.4',
            name: 'Software Policy',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          },
          {
            privilege_id: 18,
            nodeId: '4.2.5',
            name: 'Process Policy',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          },
          {
            privilege_id: 19,
            nodeId: '4.2.6',
            name: 'Data Volume Policy',
            createSelected: false,
            readSelected: false,
            updateSelected: false,
            deleteSelected: false
          }
        ]
      },
      {
        privilege_id: 20,
        nodeId: '4.3',
        name: 'Routing Policy',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 21,
    nodeId: '5',
    name: 'Infrastructure',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 22,
        nodeId: '5.1',
        name: 'Services',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 23,
        nodeId: '5.2',
        name: 'Service Groups',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 24,
        nodeId: '5.3',
        name: 'Controllers',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 25,
        nodeId: '5.4',
        name: 'Gateways',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 26,
        nodeId: '5.5',
        name: 'Gateways Groups',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 27,
        nodeId: '5.6',
        name: 'Server Bridges',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 28,
        nodeId: '5.7',
        name: 'Device Bridge (x86)',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 29,
        nodeId: '5.8',
        name: 'Device Bridge (ARM)',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 30,
        nodeId: '5.9',
        name: 'RDI Bridges',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 67,
        nodeId: '5.10',
        name: 'RBI Bridges',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 31,
        nodeId: '5.11',
        name: 'SaaS Bridges',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 32,
        nodeId: '5.12',
        name: 'LDAP Configuration',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 33,
        nodeId: '5.13',
        name: 'IDP Management',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 34,
    nodeId: '6',
    name: 'IAM',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 35,
        nodeId: '6.1',
        name: 'Admin Users',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: []
      },
      {
        privilege_id: 36,
        nodeId: '6.2',
        name: 'Roles and Permissions',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false,
        children: []
      },
      {
        privilege_id: 37,
        nodeId: '6.3',
        name: 'Users',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 38,
        nodeId: '6.4',
        name: 'User Groups',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 39,
    nodeId: '7',
    name: 'Device Management',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 40,
        nodeId: '7.1',
        name: 'Devices',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 41,
    nodeId: '8',
    name: 'S2S Microsegmentation',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 64,
        nodeId: '8.1',
        name: 'Workload Devices',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 65,
        nodeId: '8.2',
        name: 'Microsegments',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 66,
        nodeId: '8.3',
        name: 'Workloads',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 43,
    nodeId: '9',
    name: 'Client Management',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 44,
        nodeId: '9.1',
        name: 'End User Clients',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 45,
        nodeId: '9.2',
        name: 'Server Side Client',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 46,
        nodeId: '9.3',
        name: 'Groups',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 47,
    nodeId: '10',
    name: 'Organizations',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: []
  },
  {
    privilege_id: 48,
    nodeId: '11',
    name: 'Device Health Check',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 49,
        nodeId: '11.1',
        name: 'Software Master List',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 50,
        nodeId: '11.2',
        name: 'Unauthorized Software',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 51,
        nodeId: '11.3',
        name: 'Process Binding',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 52,
    nodeId: '12',
    name: 'Access Logs',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 53,
        nodeId: '12.1',
        name: 'Client',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 54,
        nodeId: '12.2',
        name: 'Admin',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  },
  {
    privilege_id: 55,
    nodeId: '13',
    name: 'Settings',
    createSelected: false,
    readSelected: false,
    updateSelected: false,
    deleteSelected: false,
    children: [
      {
        privilege_id: 56,
        nodeId: '13.1',
        name: 'Portal Settings',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 57,
        nodeId: '13.2',
        name: 'Client Settings',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 58,
        nodeId: '13.3',
        name: 'Core Settings',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 59,
        nodeId: '13.4',
        name: 'License Management',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 60,
        nodeId: '13.5',
        name: 'System Backup',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      },
      {
        privilege_id: 61,
        nodeId: '13.6',
        name: 'Software Update',
        createSelected: false,
        readSelected: false,
        updateSelected: false,
        deleteSelected: false
      }
    ]
  }
];
