import { Injectable } from '@angular/core';
import { HttpClient as AuthHttp } from '@angular/common/http';
import { BehaviorSubject, map } from 'rxjs';
import { ConfigService } from '../../services/config.service';

@Injectable({
  providedIn: 'root'
})
export class ViewRolesAndPermissionService {
  private apiBaseUrl: string;
  private rolesAndPermissionsApiUrl = '/templates';
  private assignedUserTemplatesSubject = new BehaviorSubject<any>(
    this.loadInitialValue()
  );
  assignedUserTemplates$ = this.assignedUserTemplatesSubject.asObservable();

  // Toggle State Dummy
  private toggle = new BehaviorSubject<boolean>(false);
  currentToggleState = this.toggle.asObservable();
  private currentState = false;

  toggleSideNavBar() {
    this.currentState = !this.currentState;
    this.toggle.next(this.currentState);
  }

  constructor(
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.rolesAndPermissionsApiUrl =
      this.apiBaseUrl + this.rolesAndPermissionsApiUrl;
  }

  setAssignedUserTemplates(data: any) {
    this.assignedUserTemplatesSubject.next(data);
  }

  loadInitialValue() {
    return JSON.parse(localStorage.getItem('Roles')) || {};
  }

  createTemplate(permissions) {
    return this.authHttp
      .post(this.rolesAndPermissionsApiUrl, permissions)
      .pipe(map(response => response));
  }

  getTemplatebyId(id) {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/${id}`)
      .pipe(map(response => response));
  }

  getAllTemplates() {
    return this.authHttp
      .get(this.rolesAndPermissionsApiUrl)
      .pipe(map(response => response));
  }

  getAssignedUserTemplates() {
    return this.authHttp
      .get(`${this.rolesAndPermissionsApiUrl}/user`)
      .pipe(map(response => response));
  }

  updateTemplate(id, permissions) {
    return this.authHttp
      .put(`${this.rolesAndPermissionsApiUrl}/${id}`, permissions)
      .pipe(map(response => response));
  }

  deleteTemplate(id) {
    return this.authHttp
      .delete(`${this.rolesAndPermissionsApiUrl}/${id}`)
      .pipe(map(response => response));
  }

  getUserRole() {}

  recoverFile(file, verificationPassword) {
    return this.authHttp
      .post(this.rolesAndPermissionsApiUrl, {
        attachment: file,
        verificationPassword
      })
      .pipe(map(response => response));
  }
}
