import { EventEmitter, Injectable } from '@angular/core';
import { ConfigService } from '../services/config.service';
import { HttpClient as AuthHttp, HttpHeaders } from '@angular/common/http';
import { AuthenticationService } from '../login/authentication.service';
import { Observable } from 'rxjs';
import { Setting } from '../models/setting';
import { LoginResponse } from '../models/login_response';
import { map } from 'rxjs/operators';

@Injectable()
export class SettingsService {
  private apiBaseUrl: string;
  private settingApiUrl = '/settings';
  userSettingsLoaded = new EventEmitter();

  httpOptions = {
    headers: new HttpHeaders({ 'Content-Type': 'application/json' })
  };
  constructor(
    private authenticationService: AuthenticationService,
    private authHttp: AuthHttp,
    private configService: ConfigService
  ) {
    this.apiBaseUrl = this.configService.getConfiguration().apiBaseUrl;
    this.settingApiUrl = this.apiBaseUrl + this.settingApiUrl;
  }
  getAllTimeZones() {
    return this.authHttp
      .get(`${this.settingApiUrl}/timeZones`)
      .pipe(map(response => response));
  }

  getSettings(settingUrl: string): Observable<any> {
    return this.authHttp
      .get(this.settingApiUrl + settingUrl)
      .pipe(map((response: Response) => response));
  }

  getTimeSettings(): Observable<any> {
    return this.authHttp
      .get(`${this.settingApiUrl}/timesetttings`)
      .pipe(map((response: Response) => response));
  }

  getAddressSpaceSettings(): Observable<any> {
    return this.authHttp
      .get(`${this.settingApiUrl}/addressSpace`)
      .pipe(map((response: Response) => response));
  }

  updateAddressSpaceSettings(addressSpacePayload): Observable<any> {
    return this.authHttp
      .post(
        `${this.settingApiUrl}/addressSpace`,
        { addressSpacePayload },
        this.httpOptions
      )
      .pipe(map(this.extractData));
    //.catch(this.handleError);
  }

  addEditSetting(setting: Setting): Observable<LoginResponse | any> {
    let queryString = '';
    if (setting.id != 0 && typeof setting.id != 'undefined') {
      queryString = '/' + setting.id;
      return this.authHttp
        .put(
          this.settingApiUrl + queryString,
          { setting: setting },
          this.httpOptions
        )
        .pipe(map(this.extractData));
      // .catch(this.handleError);
    } else {
      return this.authHttp
        .post(this.settingApiUrl, { setting: setting }, this.httpOptions)
        .pipe(map(this.extractData));
      //.catch(this.handleError);
    }
  }

  updateClientSetting(setting) {
    return this.authHttp
      .put(this.settingApiUrl + '/clientSetting', { setting: setting })
      .pipe(map(res => res));
  }

  updateCoreSetting(setting) {
    return this.authHttp
      .put(this.settingApiUrl + '/coreSetting', { setting: setting })
      .pipe(map(res => res));
  }

  /*addEditLicense(file): Observable<LoginResponse> {
    let headers = new Headers({ 'Content-Type': 'application/json' });
    let options = new RequestOptions({ headers: headers });
    let queryString = "";
    if (update.id != 0 && typeof update.id != "undefined") {
      queryString = "/" + update.id;
      return this.authHttp.put(this.updatesApiUrl + queryString, { update: update, attachment: file }, options)
        .map(this.extractData)
        .catch(this.handleError);
    } else {
      return this.authHttp.post(this.updatesApiUrl, { update: update, attachment: file }, options)
        .map(this.extractData)
        .catch(this.handleError);
    }
  }*/

  private extractData(res: Response) {
    let body = res;
    return body || {};
  }

  // private handleError(error: Response | any) {
  //   // In a real world app, we might use a remote logging infrastructure
  //   let errMsg: string;
  //   if (error) {
  //     const body = error || '';
  //     const err = body.error || JSON.stringify(body);
  //     errMsg = `${error.status} - ${error.statusText || ''} ${err}`;
  //   } else {
  //     errMsg = error.message ? error.message : error.toString();
  //   }
  //   console.error(errMsg);
  //   return Observable.throw(errMsg);
  // }

  logOutClients() {
    return this.authHttp.patch(`${this.settingApiUrl}/force/logout`, {});
  }

  getDarkSettings(){
    this.userSettingsLoaded.emit();
  }
}
