import { Component, OnInit, ViewChild, ChangeDetectorRef } from '@angular/core';
import { Router, ActivatedRoute } from '@angular/router';
import { Update } from '../../models/update';
import { UpdateService } from '../update.service';
import { PreserveSearch } from '../../models/preserve-search';
import { Pagination } from '../../models/pagination';
import { ProgressFile } from '../Progress';
import { GlobalMessageClass } from '../../global-messages/global-messages';
import { GlobalMessageService } from '../../global-messages/global-messages.service';
import { LicenseService } from '../../license/license.service';
import { ErrorCodeEnum } from '../../shared/emuns/error.enum';
import { DeviceOSEnum } from '../../shared/emuns/client.enum';
import {
  ContextHelpEnum,
  APIDataTypeEnum
} from '../../shared/emuns/global.enum';
import { Title } from '@angular/platform-browser';

@Component({
  selector: 'app-create-update',
  templateUrl: './create-update.component.html',
  styleUrls: ['./create-update.component.css']
})
export class CreateUpdateComponent implements OnInit {
  @ViewChild('updateForm', { static: false }) updateForm;
  @ViewChild('packageFile', { static: false }) packageFile;

  public alert: GlobalMessageClass;
  public selectedComponent: string = 'Controller';
  public update = new Update();
  public fileData = {};
  public sigFileData = {};
  public errorMessage: string;
  public search: PreserveSearch = new PreserveSearch();
  public pagination: Pagination = new Pagination();
  public pageTitle: string = 'Add Software Update';
  public uploadValid: boolean = false;
  public sigFileValid: boolean = false;
  private fileSizeLimit = 100 * 1024;
  public progress: ProgressFile = new ProgressFile();
  public uploading: boolean = false;
  public dateFormat: string = 'dd/MM/yyyy HH:mm';
  public ext: string;

  components = [
    { value: 'controller', display: 'Controller' },
    { value: 'client', display: 'Client' },
    { value: 's2s-client', display: 'S2S Client' },
    { value: 'gateway', display: 'Gateway' }
  ];

  oss = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];
  clientOs = [
    { value: DeviceOSEnum.Ubuntu18, display: 'Ubuntu Desktop' },
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    },
    { value: DeviceOSEnum.Windows, display: 'Windows' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.Macintosh, display: 'macOS' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];

  s2sClientOs = [
    { value: DeviceOSEnum.UbuntuServer18, display: 'Ubuntu Server' },
    { value: DeviceOSEnum.WindowsServer, display: 'Windows Server' },
    { value: DeviceOSEnum.CentosServer, display: 'CentOS Server' }
  ];

  gatewayOs = [
    { value: DeviceOSEnum.UbuntuServer18, display: ' Ubuntu Server' },
    // { value: 'Windowsserver', display: 'Windows Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    }
  ];

  controllerOs = [
    { value: DeviceOSEnum.UbuntuServer18, display: ' Ubuntu Server' },
    // { value: 'Windowsserver', display: 'Windows Server' },
    {
      value: DeviceOSEnum.UbuntuServer20armv8,
      display: 'Ubuntu Server (ARMv8)'
    }
  ];

  //Contextual Help
  public contextualHelp: Array<any>;
  public showContext: boolean = false;
  //Contextual Help

  loading: Boolean = false;
  disableSaveButton = false;

  constructor(
    private router: Router,
    private route: ActivatedRoute,
    private updateService: UpdateService,
    private globalMessageService: GlobalMessageService,
    private licenseService: LicenseService,
    private cdref: ChangeDetectorRef,
    private titleService: Title
  ) {
    this.contextualHelp = ['', ''];
    var localDateFormat = localStorage.getItem('dateFormat');
    if (typeof localDateFormat != 'undefined' || localDateFormat != null) {
      this.dateFormat = localDateFormat;
    }
  }

  showContextHelp(name: any): void {
    if (name == ContextHelpEnum.buildComponent) {
      this.contextualHelp = [
        'buildComponent',
        'Invisily component the update is intended for.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.buildVersion) {
      this.contextualHelp = ['buildVersion', 'Component release version.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.buildOs) {
      this.contextualHelp = ['buildOs', 'Supported OS of the component.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.description) {
      this.contextualHelp = ['description', 'Details about the update.'];
      this.showContext = true;
    } else if (name == ContextHelpEnum.build) {
      this.contextualHelp = [
        'build',
        'Select package/installer file of the component to upload.'
      ];
      this.showContext = true;
    } else if (name == ContextHelpEnum.sig) {
      this.showContext = true;
      this.contextualHelp = [
        'sig',
        'Select Valid SIgnature file for this package.'
      ];
    }
  }

  closeContextualHelp(): void {
    this.showContext = false;
  }

  ngOnInit() {
    this.disableSaveButton = false;
    let updateSearch = this.search.getSearch('updateSearch');
    if (updateSearch) {
      this.pagination.currentPage = updateSearch.current_page;
    }
    this.getPage(this.pagination.currentPage);
    this.titleService.setTitle(
      'Settings | Software Update | ' + this.pageTitle
    );
    this.licenseService.getLicense().subscribe(
      res => {
        if (Math.floor(parseFloat(res.headers?.get('X-License-Days'))) === -1) {
          this.onlyControllerSelectable();
        }
      },
      err => {
        if (err.status === ErrorCodeEnum.NotFound) {
          if (
            Math.floor(parseFloat(err.headers?.get('X-License-Days'))) === -1
          ) {
            this.onlyControllerSelectable();
          }
        }
      }
    );
  }

  onlyControllerSelectable() {
    this.components = [
      { value: undefined, display: '' },
      { value: 'controller', display: 'Controller' }
    ];
  }

  ngAfterViewInit(): void {
    if (!(typeof this.route.params['_value']['id'] != 'undefined')) {
      this.update.buildComponent = APIDataTypeEnum.Client;
      setTimeout(() => {
        this.update.buildOs = this.oss[0].value;
      }, 10);
      this.cdref.detectChanges();
    }
  }

  componentSelected() {
    const selectedComponent = this.update.buildComponent;
    this.updateForm.resetForm();
    this.update.is_settings_changed = false;
    this.packageFile.nativeElement.value = null;
    this.uploading = false;
    this.uploadValid = false;
    this.progress = new ProgressFile();

    this.update.buildComponent = selectedComponent;
    if (this.update.buildComponent === 'controller') {
      this.oss = this.controllerOs;

      setTimeout(() => {
        this.update.buildOs = this.oss[0].value;
      }, 10);
    } else if (this.update.buildComponent === 'gateway') {
      this.oss = this.gatewayOs;
      setTimeout(() => {
        this.update.buildOs = this.oss[0].value;
      }, 10);
    } else if (this.update.buildComponent === 's2s-client') {
      this.oss = this.s2sClientOs;
      this.update.buildOs = this.oss[0].value;
      setTimeout(() => {
        this.update.buildOs = this.oss[0].value;
      }, 10);
    } else {
      this.oss = this.clientOs;
      this.update.buildOs = this.oss[0].value;
    }
  }

  getPage(page: number) {
    this.pagination.currentPage = page;
    // If edit Updates
    if (typeof this.route.params['_value']['id'] != 'undefined') {
      this.loading = true;
      this.pageTitle = 'Edit Software Update';
      this.updateService
        .getUpdateDetail(this.route.params['_value']['id'])
        .subscribe(
          update => {
            this.loading = false;
            this.update = update;
          },
          err => {
            this.loading = false;
          }
        );
    }
  }

  saveUpdate() {
    this.loading = true;
    this.disableSaveButton = true;
    this.update.is_settings_changed = false;
    this.uploading = true;
    const { buildOs, buildComponent } = this.update;
    this.progress.show = true;
    this.progress.value = 0;
    const interval = setInterval(() => {
      if (this.progress.value <= 70) {
        this.progress.value += Math.floor(Math.random() * 2);
      }
    }, 1000);
    this.updateService
      .addEditUpdate(this.update, this.fileData, this.sigFileData)
      .subscribe(
        response => {
          this.loading = false;
          if (response.success) {
            this.progress.value = 100;
            this.progress.complete = true;
            setTimeout(() => {
              this.alert = {
                closed: false,
                type: 'success',
                message: 'Update has been added/updated successfully.'
              };
              this.globalMessageService.pushAlert(this.alert);
              this.router.navigate(['/updates']);
            }, 2000);
          } else {
            this.alert = {
              closed: false,
              type: 'danger',
              message: response.message || 'Failed to add/update Updates.'
            };
            this.globalMessageService.pushAlert(this.alert);
            this.errorMessage = response.message;
          }
          this.uploading = false;
          this.progress.value = 0;
          this.progress.show = false;
          clearInterval(interval);
        },
        error => {
          this.disableSaveButton = false;
          this.loading = false;
          this.uploading = false;
          clearInterval(interval);
        }
      );
  }

  handleSigFileUpload(event) {
    if (event.target.files && event.target.files.length > 0) {
      let file = event.target.files[0];
      if (file.size / 1024 < this.fileSizeLimit) {
        let name = file.name;
        //Getting Type
        var extension = name.split('.').pop();
        if (extension == 'sign') {
          let reader = new FileReader();
          reader.readAsDataURL(file);
          reader.onload = () => {
            let data = reader.result;
            this.sigFileData = {
              data: data,
              name: name,
              type: extension
            };
            this.sigFileValid = true;
          };
        } else {
          alert(
            'Please select a valid file type, these are allowed types `sign`'
          );
          this.sigFileValid = false;
        }
      } else {
        alert('File size should be less than ' + this.fileSizeLimit + ' 50MB');
        this.sigFileValid = false;
      }
    }
  }

  handleFileUpload(evt) {
    let self = this;
    self.progress.complete = false;
    self.progress.value = 0.0;
    //self.progress.show = true;
    if (evt.target.files && evt.target.files.length > 0) {
      let file = evt.target.files[0]; // FileList object
      if (file.size / 1024 < this.fileSizeLimit) {
        let name = file.name;
        let brokenName = name.split('-');
        var verWithExt = brokenName[brokenName.length - 1].split('.');
        verWithExt.splice(-1, 1);
        var version = verWithExt.join('.');
        let regex = /^(\d+\.)(\d+\.)(\d+\.)?(\d+)$/gm;
        let isValidVersion = regex.test(version);
        let type = file.type;
        if (isValidVersion == false) {
          alert(
            'Please select a valid file name format. Format is name-version.extension.'
          );
          self.uploadValid = false;
          return 0;
        }

        this.update.buildVersion = version;
        //Getting Type
        var local_name = name.split('.');
        this.ext = local_name[local_name.length - 1];
        var types = [
          'deb',
          'rpm',
          'dmg',
          'exe',
          'pkg',
          'application/x-apple-diskimage',
          'application/x-diskcopy',
          'application/x-msdownload',
          'application/vnd.debian.binary-package',
          'application/x-ms-dos-executable',
          'application/exe',
          'application/deb',
          'application/rpm',
          'application/dmg',
          'application/octet-stream'
        ];
        if (
          types.includes(type) ||
          this.ext === 'pkg' ||
          this.ext === 'ipa' ||
          this.ext === 'apk' ||
          this.ext === 'rpm' ||
          this.ext === 'deb'
        ) {
          // if (this.update.buildOs) {
          //   if (!this.checkPackageExtensionIsValid(this.ext)) {
          //     self.uploadValid = false;
          //     return 0;
          //   }
          // }
          if (this.ext === 'pkg') {
            type = this.ext;
          }
          if (this.ext === 'deb') {
            type = this.ext;
          }
          if (this.ext === 'rpm') {
            type = this.ext;
          }
          let reader = new FileReader();
          reader.readAsDataURL(file);
          reader.onload = () => {
            let data = reader.result;
            self.fileData = {
              data: data,
              name: name,
              type: type
            };
            self.uploadValid = true;
          };
          // reader.onloadstart = () => {
          //   self.progress.show = true;
          //   self.progress.value = 0;
          // };
          // reader.onprogress = data => {
          //   self.progress.show = true;
          //   //self.progress.value = (data.loaded / data.total) * 100;
          //   self.progress.value = 50;
          // };
          // reader.onloadend = () => {
          //   self.progress.value = 50;
          //   self.progress.complete = true;
          // };
        } else {
          alert(
            'Please select a valid file type, these are allowed types `deb`, `dmg`, `exe`, `ipa`, `apk`, `rpm`'
          );
          self.uploadValid = false;
        }
      } else {
        alert('File size should be less than ' + this.fileSizeLimit + ' 50MB');
        self.uploadValid = false;
      }
    }
  }

  navigate(title: string) {}
  onCancel(): void {
    this.router.navigate(['/updates']);
  }

  checkFileValidation() {
    if (this.ext) {
      if (!this.checkPackageExtensionIsValid(this.ext)) {
        this.uploadValid = false;
      } else {
        this.uploadValid = true;
      }
    }
  }

  checkPackageExtensionIsValid(ext): Boolean {
    let os = this.update.buildOs;
    if (os.includes(DeviceOSEnum.Ubuntu)) {
      if (ext !== 'deb') {
        alert('Please select a valid deb file for this OS.');
        return false;
      }
    } else if (os.includes(DeviceOSEnum.CentosServer)) {
      if (ext !== 'rpm') {
        alert('Please select a valid rpm file for this OS.');
        return false;
      }
    } else if (
      os === DeviceOSEnum.Windows ||
      os === DeviceOSEnum.WindowsServer
    ) {
      if (ext !== 'exe') {
        alert('Please select a valid exe file for this OS.');
        return false;
      }
    } else if (os === DeviceOSEnum.Macintosh) {
      if (ext !== 'pkg') {
        alert('Please select a valid exe file for this OS.');
        return false;
      }
    } else if (os === DeviceOSEnum.Android) {
      if (ext !== 'apk') {
        alert('Please select a valid apk file for this OS.');
        return false;
      }
    } else {
      //ios
      if (ext !== 'ipa') {
        alert('Please select a valid ipa file for this OS.');
        return false;
      }
    }
    return true;
  }
}
