import { Component, OnDestroy, OnInit } from '@angular/core';
import { UsersReportsService } from './reporting.service';
import { Pagination } from '../models/pagination';
import { Title } from '@angular/platform-browser';
import { Subject, debounceTime, distinctUntilChanged, switchMap } from 'rxjs';
import { ReportingService } from '../reportings/reporting.service';
import moment from 'moment';
@Component({
  selector: 'app-users-reports',
  templateUrl: './users-reports.component.html',
  styleUrls: ['./users-reports.component.css']
})
export class UsersReportsComponent implements OnInit, OnDestroy {
  summaryOfAllUsers: any = [];
  summaryOfAllServices: any = [];
  servicesAccesedByUsers: any = [];
  userDetails: any = [];
  topFiveUsers: any = [];
  clients: any = [];
  serviceType: any = [{Name:'HTTPS',service_type:'HTTPS'},{Name:'HTTP',service_type:'HTTP'}]
  usersDaterangepicker: Date[] = [new Date(moment().subtract(1, 'month').toDate()), new Date()];
  today: Date=new Date();

  serviceSortDir: string = 'asc';
  serviceSort: string;

  clientSortDir: string = 'asc';
  clientSort: string;

  selectedServiceType: string = '';
  searchSession: string = '';
  searchAuth: string = '';
  searchUser: string = '';
  $searchUsersSessions = new Subject();
  $searchUsersAuth = new Subject();
  $searchUserSub = new Subject();

  selectedService: {
    serviceSummary: Array<any>;
    clientsSummary: Array<any>;
    serviceSummaryGraph: Array<any>;
    clientsSummaryGraph: Array<any>;
  } = {
    serviceSummary: null,
    clientsSummary: null,
    serviceSummaryGraph: null,
    clientsSummaryGraph: null
  };

  selectedClient: {
    serviceSummary: Array<any>;
    clientsSummary: Array<any>;
    serviceSummaryGraph: Array<any>;
    clientsSummaryGraph: Array<any>;
  } = {
    serviceSummary: null,
    clientsSummary: null,
    serviceSummaryGraph: null,
    clientsSummaryGraph: null
  };

  summaryOfAllServicesPagination: Pagination = new Pagination();
  summaryOfAllUsersPagination: Pagination = new Pagination();
  servicesAccessedbyUserPagination: Pagination = new Pagination();
  topFiveUserPagination: Pagination = new Pagination();
  userDetailsPagination: Pagination = new Pagination();


  summaryOfAllServicesLoading: Boolean = true;
  summaryOfAllClientsLoading: Boolean = true;
  servicesAccessedbyUserLoading: Boolean = true;

  toggleRuleViewStyle = 'none';

  pageSizeSummaryOfUsers = [10, 25, 50, 100, 200];
  pageSizeSummaryOfServices = [10, 25, 50, 100, 200];

  OShashMap = {
    ubuntu: 'Ubuntu Desktop',
    ubuntuserver: 'Ubuntu Server',
    ubuntu18: 'Ubuntu Desktop',
    ubuntuserver18: 'Ubuntu Server',
    ubuntuserver20armv8: 'Ubuntu Server (ARMv8)',
    windows: 'Windows',
    windowsserver: 'Windows Server',
    centosserver: 'CentOS Server',
    macintosh: 'Macintosh',
    ios: 'iOS',
    android: 'Android',
    unknown: 'Unknown'
  };

  constructor(
    private UsersReportsService: UsersReportsService,
    private titleService: Title,
    private reportingService:ReportingService,
  ) {}

  ngOnInit() {
    this.titleService.setTitle('Reports | Users Reports');
    this.summaryOfAllUsersPagination.id = 'users';
    this.summaryOfAllServicesPagination.id = 'services';
    this.servicesAccessedbyUserPagination.id = 'servicesbyusers';
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
    this.getServicesAccessedByUser(this.servicesAccessedbyUserPagination);
    this.subscibeToSearch();
    this.subscibeToAuthSearch();
    this.subscribeToSearchUser();
    this.getAllClients();
  }

  ngOnDestroy(): void {
    // throw new Error('Method not implemented.');
  }

  onChangeClient(type: string, value) {
      this.servicesAccessedbyUserPagination.currentPage = 1;
      this.selectedClient.clientsSummary = value;

      this.servicesAccessedbyUserLoading = true;
      this.UsersReportsService.getServiceToUsersReport(
        this.servicesAccessedbyUserPagination,
        this.searchUser,
        this.selectedClient.clientsSummary,
        this.selectedServiceType,
        this.usersDaterangepicker
        ).subscribe({
        next: summary => {
          this.servicesAccessedbyUserPagination.totalItems = summary.data.recordsTotal;
          this.servicesAccesedByUsers = summary.data;
          this.servicesAccessedbyUserLoading = false;
        },
        error: err => {
          this.servicesAccessedbyUserLoading = false;
        }
      });
    
  }
  onChangeService(value) {
    this.servicesAccessedbyUserPagination.currentPage = 1;
    this.selectedServiceType = value?value.service_type:'';

    this.servicesAccessedbyUserLoading = true;
      this.UsersReportsService.getServiceToUsersReport(
        this.servicesAccessedbyUserPagination,
        this.searchUser,
        this.selectedClient.clientsSummary,
        this.selectedServiceType,
        this.usersDaterangepicker
        ).subscribe({
        next: summary => {
          this.servicesAccessedbyUserPagination.totalItems = summary.data.recordsTotal;
          this.servicesAccesedByUsers = summary.data;
          this.servicesAccessedbyUserLoading = false;
        },
        error: err => {
          this.servicesAccessedbyUserLoading = false;
        }
      });
  }
  userDateChange(){
    this.servicesAccessedbyUserPagination.currentPage = 1;

    this.servicesAccessedbyUserLoading = true;
      this.UsersReportsService.getServiceToUsersReport(
        this.servicesAccessedbyUserPagination,
        this.searchUser,
        this.selectedClient.clientsSummary,
        this.selectedServiceType,
        this.usersDaterangepicker
        ).subscribe({
        next: summary => {
          this.servicesAccessedbyUserPagination.totalItems = summary.data.recordsTotal;
          this.servicesAccesedByUsers = summary.data;
          this.servicesAccessedbyUserLoading = false;
        },
        error: err => {
          this.servicesAccessedbyUserLoading = false;
        }
      });
   
  }


  subscribeToSearchUser(){
    this.$searchUserSub
    .pipe(
      debounceTime(500),
      distinctUntilChanged(),
      switchMap((data: any) => {
        this.servicesAccessedbyUserLoading = true;
        this.searchUser = data;
        this.servicesAccessedbyUserPagination.currentPage = 1;
        return this.UsersReportsService.getServiceToUsersReport(
          this.servicesAccessedbyUserPagination,
          this.searchUser,
          this.selectedClient.clientsSummary,
          this.selectedServiceType,
          this.usersDaterangepicker
        );
      })
    )
    .subscribe({
      next: (users: any) => {
        this.servicesAccessedbyUserLoading = false;
        const { recordsTotal } = users.data;
        this.servicesAccesedByUsers = users.data;
        this.servicesAccessedbyUserPagination.totalItems = recordsTotal;
      },
      error: err => {
        this.servicesAccessedbyUserLoading = false;
      }
    });
  }
  subscibeToSearch() {
    this.$searchUsersSessions
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          this.summaryOfAllServicesLoading = true;
          this.searchSession = data;
          this.summaryOfAllServicesPagination.currentPage = 1;
          return this.UsersReportsService.getUserReports(
            this.summaryOfAllServicesPagination,
            this.serviceSort,
            this.serviceSortDir,
            this.searchSession
          );
        })
      )
      .subscribe({
        next: (users: any) => {
          this.summaryOfAllServicesLoading = false;
          const { recordsTotal } = users;
          this.summaryOfAllServices = users;
          this.summaryOfAllServicesPagination.totalItems = recordsTotal;
        },
        error: err => {
          this.summaryOfAllServicesLoading = false;
        }
      });
  }
  subscibeToAuthSearch() {
    this.$searchUsersAuth
      .pipe(
        debounceTime(500),
        distinctUntilChanged(),
        switchMap((data: any) => {
          this.summaryOfAllClientsLoading = true;
          this.searchAuth = data;
          this.summaryOfAllUsersPagination.currentPage = 1;
          return this.UsersReportsService.getUserAuthenticationFailedReports(
            this.summaryOfAllUsersPagination,
            this.serviceSort,
            this.serviceSortDir,
            this.searchAuth
          );
        })
      )
      .subscribe({
        next: (auth: any) => {
          this.summaryOfAllClientsLoading = false;
          const { recordsTotal } = auth;
          this.summaryOfAllUsers = auth;
          this.summaryOfAllUsersPagination.totalItems = recordsTotal;
        },
        error: err => {
          this.summaryOfAllClientsLoading = false;
        }
      });
  }

  getSummaryOfAllServices(pagination: Pagination) {
    this.summaryOfAllServicesLoading = true;
    this.UsersReportsService.getUserReports(
      pagination,
      this.serviceSortDir,
      this.serviceSort
    ).subscribe({
      next: summary => {
        this.summaryOfAllServicesPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllServices = summary;
        this.summaryOfAllServicesLoading = false;
      },
      error: err => {
        this.summaryOfAllServicesLoading = false;
      }
    });
  }

  getServicesAccessedByUser(pagination: Pagination) {
    this.servicesAccessedbyUserLoading = true;
    this.UsersReportsService.getServiceToUsersReport(
      pagination,
      this.searchUser,
      this.selectedClient.clientsSummary,
      this.selectedServiceType,
      this.usersDaterangepicker
      ).subscribe({
      next: summary => {
        this.servicesAccessedbyUserPagination.totalItems = summary.data.recordsTotal;
        this.topFiveUserPagination.totalItems=summary.topFive.recordsTotal;
        this.topFiveUsers = summary.topFive;
        this.servicesAccesedByUsers = summary.data;
        this.servicesAccessedbyUserLoading = false;
      },
      error: err => {
        this.servicesAccessedbyUserLoading = false;
      }
    });
  }

  clearfun() {
    this.searchSession = '';
  }
  clearfunUser() {
    this.searchUser = '';
  }
  clearfunAuth() {
    this.searchAuth = '';
  }
  clearDate(){
    this.usersDaterangepicker = [];
    this.userDateChange();
  }

  getSummaryOfAllUser(pagination: Pagination) {
    this.summaryOfAllClientsLoading = true;
    this.UsersReportsService.getUserAuthenticationFailedReports(
      pagination,
      this.clientSortDir,
      this.clientSort
    ).subscribe({
      next: summary => {
        this.summaryOfAllUsersPagination.totalItems = summary.recordsTotal;
        this.summaryOfAllUsers = summary;
        this.summaryOfAllClientsLoading = false;
      },
      error: err => {
        this.summaryOfAllClientsLoading = false;
      }
    });
  }

  pageChangeSummaryOfAllServices(page: any) {
    this.summaryOfAllServicesPagination.currentPage = page;
    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
  }

  pageChangeSummaryOfAllUsers(page: any) {
    this.summaryOfAllUsersPagination.currentPage = page;
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
  }

  pageChangeServicesAccessedByUser(page: any) {
    this.servicesAccessedbyUserPagination.currentPage = page;
    this.getServicesAccessedByUser(this.servicesAccessedbyUserPagination);
  }
  pageSizeChangeSummaryOfAllServices(size) {
    this.summaryOfAllServicesPagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.summaryOfAllServicesPagination.currentPage = 1;
      this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
    }, 0);
  }

  pageSizeChangeSummaryOfAllUsers(size) {
    this.summaryOfAllUsersPagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.summaryOfAllUsersPagination.currentPage = 1;
      this.getSummaryOfAllServices(this.summaryOfAllUsersPagination);
    }, 0);
  }

  pageSizeChangeServicesAccessedByUser(size) {
    this.servicesAccessedbyUserPagination.itemsPerPage = +size;

    //not a proper way to do, pagination dynamic change erorr fix
    setTimeout(() => {
      this.servicesAccessedbyUserPagination.currentPage = 1;
      this.getServicesAccessedByUser(this.servicesAccessedbyUserPagination);
    }, 0);
  }

  summaryOfAllUsersResultCount() {
    const pagination = this.summaryOfAllUsersPagination;
    let start =
      pagination.currentPage * pagination.itemsPerPage -
      (pagination.itemsPerPage - 1);
    let end = pagination.currentPage * pagination.itemsPerPage;
    start = start > 0 ? start : 1;
    start = pagination.totalItems > 0 ? start : 0;
    end = pagination.totalItems > end ? end : pagination.totalItems;
    return `Showing ${start} to ${end} of ${pagination.totalItems} entries`;
  }

  summaryOfAllServicesResultCount() {
    const pagination = this.summaryOfAllServicesPagination;
    let start =
      pagination.currentPage * pagination.itemsPerPage -
      (pagination.itemsPerPage - 1);
    let end = pagination.currentPage * pagination.itemsPerPage;
    start = start > 0 ? start : 1;
    start = pagination.totalItems > 0 ? start : 0;
    end = pagination.totalItems > end ? end : pagination.totalItems;
    return `Showing ${start} to ${end} of ${pagination.totalItems} entries`;
  }

  sortServiceRecord(term: string) {
    this.serviceSortDir = this.serviceSortDir == 'asc' ? 'desc' : 'asc';
    this.serviceSort = term;

    this.getSummaryOfAllServices(this.summaryOfAllServicesPagination);
  }

  sortClientRecord(term: string) {
    this.clientSortDir = this.clientSortDir == 'asc' ? 'desc' : 'asc';
    this.clientSort = term;
    this.getSummaryOfAllUser(this.summaryOfAllUsersPagination);
  }

  getOSFullName(key) {
    return this.OShashMap[key];
  }

  getDuration(CONNECTION_ATTEMPT_TIME, DISCONNECTION_TIME) {
    const connectionAttemptTime = new Date(CONNECTION_ATTEMPT_TIME);
    const disconnectionTime = new Date(DISCONNECTION_TIME);
    const durationMilliseconds =
      disconnectionTime.getTime() - connectionAttemptTime.getTime();

    const hours = Math.floor(durationMilliseconds / 3600000);
    const minutes = Math.floor((durationMilliseconds % 3600000) / 60000);
    const seconds = Math.floor((durationMilliseconds % 60000) / 1000);

    let durationString = '';
    if (hours > 0) {
      durationString += `${hours} Hours `;
    }
    if (minutes > 0) {
      durationString += `${minutes} Minutes `;
    }
    if (hours > 0 || minutes > 0 || seconds > 1) {
      durationString += `${seconds} Seconds`;
    } else {
      durationString += `${seconds} Second`;
    }

    return durationString;
  }

  ReformatTime(TIME): string {
    const date = new Date(TIME);
    return `${date.getDate()}/${date.getMonth() +
      1}/${date.getFullYear()} ${date.getHours()}:${date
      .getMinutes()
      .toString()
      .padStart(2, '0')}`;
  }
  getAllClients(){
    this.reportingService.getAllClients().subscribe(clients=>{
      this.clients = clients;
    });
  }
  servicesAccessed(id,agentid) {
    this.servicesAccessedbyUserLoading = true;
    this.userDetailsPagination.currentPage = 1;
      this.UsersReportsService.getServiceToUsersReport(
        this.userDetailsPagination,
        this.searchUser,
        [{agent_id:agentid}],
        this.selectedServiceType,
        this.usersDaterangepicker
        ).subscribe({
        next: summary => {
          this.toggleRuleViewStyle = 'block';
          this.userDetailsPagination.totalItems = summary.data.recordsTotal;
          this.userDetails = summary.data;
          this.servicesAccessedbyUserLoading = false;
        },
        error: err => {
          this.servicesAccessedbyUserLoading = false;
        }
      });
  }
  pageChangeUserDetails(agentid,page){
    this.userDetailsPagination.currentPage = page;
    this.servicesAccessedbyUserLoading = true;
    this.UsersReportsService.getServiceToUsersReport(
      this.userDetailsPagination,
      this.searchUser,
      [{agent_id:agentid}],
      this.selectedServiceType,
      this.usersDaterangepicker
      ).subscribe({
      next: summary => {
        this.userDetailsPagination.totalItems = summary.data.recordsTotal;
        this.userDetails = summary.data;
        this.servicesAccessedbyUserLoading = false;
      },
      error: err => {
        this.servicesAccessedbyUserLoading = false;
      }
    });
  }
  handleCancelModal(){
    this.toggleRuleViewStyle = 'none';
  }
  getClientName(client){
    let name=client
    name= name.split('@')[0].replace('{','').replace('}','');
    return name;
  }
}
